#ifndef _RootReader_h_
#define _RootReader_h_

#include <vector>
#include <string>
#include <map>
#include "TString.h"
/** 
    \class RootReader 
    \brief Read and navigate  root file base class
           for specific root files readers
              
     \author W.Beaumont UA
     \version 0.1 
     \date  02/2004
 

     Restrictions:
     - record numbers should be 0 or positive 

*/
class TFile;
class TObject;
class TDirectory;
class TString;
class TNamed;

class RootReader {

  public:
/** constructor     opens the TFile with the given name
    should be checked with the FileIsFound 
    @param  filename  
*/
    
RootReader(const char* filename, const char* opt="update");
virtual ~RootReader();

/** cd gRecord to the file home dir */
 bool cd();



/**find object by name if found returns the object else 0
   it starts looking for the current direcotry (gDirectory) 
   if looks for all subdirs to find the named object
   if it finds a directory with the given name it proceeds to find 
   the object in this   directory .
   So in case Pedestal/Pedestal;1  ( last is object ) will be found
   but
   Pedestal/NoPdestal;1
   Noise/Pedestal;1  will not be found .
   It returns the first object it finds. So if the there are more of these
   objects with the same name it is important to start from the correct 
   directory.
   If it finds the object the object is st
   @param name the name of the object in the root file to find
   @param chdir if true it goes to the directory where the object is found.
   @param fromfile  force to read the object from file creates a new object in memory 
   (only false works 0.1 ) 
   The function uses or 
   FindObjectAll ( check memory in case of cdir = flase and looks in
   the file ( FindKeyAll) if the object is not found in memory
 */ 
virtual TObject* Find(const char* name,bool chdir=true,bool fromfile=false);

/** the Update methode updates (appends) an object on the 
    current directory.
    It checks if the object exits in memory
    It checks if the objec exist in the file
    if both are presetn the object is written with the 
    name and the orignal object name 
    if not, nothing is written t
    to store an object use 
    Update( theobj->GetName());
*/
virtual bool Update(const char* name);
/** change gDirectory to Recordi  
    if not found gDirectory is set to file home.
    @param i  the record number 
    @return the record number , negative if not found 
*/ 
virtual bool SetRecord(int i=-1);  
virtual bool SetModule(int i);
bool SetModeDir(int mode,int recnr=-1,std::string modid="no"  );
// not needed  use TString 
// virtual bool SetModeDir(const char* mode,int recnr=-1,std::string modid="no"  );
bool SetModeDir(TString mode,int recnr=-1,std::string modid="no" );
bool FileFound(){ return theFileFound;}
/** returns the record nr in the actual path  */
virtual int GetRecordNr(TString run="") ;
virtual int GetModuleNr();
 virtual int ModCounter();  /*returns the number of modules in current record */
 TString  GetId();
virtual TString GetApvMode();
const char* getModType();
TString getSoftware();
// developments
/** refresh the map with record types 
    the record type is notindb, Modvalidation etc 
    the map is also used to loop over records.
    @return nr of records found 
*/
int  RootReader::FillRecordType() ;
/** set the default record to the first record found in the map of records
    sets the gDirectory to the home directory  of the file .
    If no record is found the gDirectory is set to file home.
    @param changedir if this is false only the record number is returned
    @return the record number negative no record found 
*/
int  GoToFirstRecord(bool changedir=true );
/** set the default record to the next record found in the map of records
    If no record is found the gDirectory is set to file home.
    if (GoToFirstRecord() > 0 ) {
    do {
    // action for this record 
    }while ( GetNextRecord() > 0);
    
    @return the recordnumber negative no record found last record     
*/
int GoToNextRecord(bool changedir=true);

/** set the default record to last record in the map of records
    If no record is found the gDirectory is set to file home.
    @return the recordnumber negative no record found last record 
*/
 int GetLastRecord(bool changedir=true);

/** obsolete */
int Write(const char* dirname,const char* objname,TObject* to,
	  int opt=0, bool makedir=false);
/** writes a root object to file 
    @ dirname the directory where the object should be written
    @ to the pointer to the root object 
    @ opt root write options 
    @ objname the name (key name ) of the object in the root file
    @ mkdir if true the part of the dirname will be created if not exist
    @ chdir if tru change to the given directory (normal false) 
*/
 int Write(const char* dirname,TObject* to,int opt=0,	 
	   const char* objname=0, bool mkdir=false ,bool chdir=false);
virtual TString GetDate();
virtual TString GetTestCenter();
virtual TString GetVersion();
virtual TString GetOperator();
bool checkopen() {return bfile;}
 protected: 
TFile* therootfile; 
TDirectory* thecurrentrec;
std::vector<int> theselrec; // records selected for analysis

 private :
   bool theFileFound;
 std::map<int,TString> therecords;
 std::map<int,TString>::iterator thecurrecpnt;
 bool Update(TNamed* name);
 TString  theid;
 TString soft;
 const char* modtype;
 int therec;
 bool bfile;
};
#endif
