#ifndef _TestAnalyzer_H_
#define _TestAnalyzer_H_


#include "AnalDefs.h"
#include <TH1F.h>
#include <vector>
/** 
    \class TestAnalyzer
\brief 
<p>     TestAnalyzer: Analyze data stored in root file 
and produce badchflags for each measured
variable; pedestal,noise, calibration etc

\author V.Zhukov  
\version 1.0
\date 02/2004

*/


class  AnalDescriptor;
class  RootReader;
class  AnalSetParser;



class TestAnalyzer {
   public:

TestAnalyzer(AnalSetParser*  s, RootReader* r);
virtual  ~TestAnalyzer();

virtual  int AnalyzeAllTests();
virtual  int AnalyzeCommonMode();
virtual  int AnalyzeNoise();
virtual  int AnalyzePedestal();
virtual  int AnalyzePeakTime();
virtual  int AnalyzeGain();
virtual  int AnalyzePinhole();
virtual  int AnalyzePulseHeight();
virtual  int AnalyzePipeline();
virtual  int AnalyzePulseShape();
virtual  int AnalyzeCalAmp();
virtual  int AnalyzeBackPlane();
virtual  int AnalyzeFBias();
virtual  int AnalyzeIV();


RootReader* getRootReader() {return reader;}
AnalSetParser* getAnalSetParser() {return setparser;}
LtApvMap&  getMapPedestal() {return mbadchpedestal;}
LtApvMap&  getMapNoise() {return mbadchnoise;}
LtApvMap&  getMapPeakTime() {return mbadchpeaktime;}
LtApvMap&  getMapGain()        {return mbadchgain;}
LtApvMap&  getMapPinhole()     {return mbadchpinhole;}
LtApvMap&  getMapPulseHeight() {return mbadchpulseheight;}
LtApvMap&  getMapPulseShape() {return mbadchpulseshape;}
LtApvMap&  getMapBackPlane()   {return mbadchbackplane;}
LtApvMap&  getMapFBias()       {return mbadchfbias;}
LtApvMap&  getMapCalAmp()       {return mbadchcalamp;}
const char* getModType() {return modtype;}
 int getNChip();
 int getNchan() {return nchan;}
 float getIleak();
 float getIsens();
 void changeModType(const char* cm) {modtype=cm;}

 protected:
 int  CalcChipAvg(TH1F* h, std::vector<double>& avg, std::vector<double>& rms);
 int  CalcAvg(TH1F* h, double& avg, double& rms);
  void printMap(LtApvMap m, char* testname);
  // 
RootReader* reader;
AnalSetParser*  setparser;
LtApvMap mbadchpedestal;
LtApvMap mbadchnoise;
LtApvMap mbadchpeaktime;
LtApvMap mbadchgain;
LtApvMap mbadchpinhole;
LtApvMap mbadchpulseheight;
LtApvMap mbadchpulseshape;
LtApvMap mbadchcalamp;
LtApvMap mbadchpipeline;
LtApvMap mbadchbackplane;
LtApvMap mbadchfbias;
LtMap  mpeaktime;
 
 float Ileak;  // IV current at 450 V
 float Isens;  // sensor current at 450 V

 int  nChip;
 int nchan;
const char* modtype;
};
#endif
