#ifndef _XmlSetParser_H
#define _XmlSetParser_H
#include "AnalSetParser.h"
#include "daqerrormsg.h"


#include <map>
#include <string>
         
/**
   \class AnalSetParser
   \author V.Zhukov
   \date 2004/02
   \version 1.2

   \brief read analsettings.xml using xercos and populate
          AnalDescriptors (analogous to setparser)   
 */

class DOM_Node;
class AnalDescriptor;

class XmlSetParser : public AnalSetParser
{
 
private:
  std::string fFileName;
  int nPedestalDescriptors;
  std::map<std::string,PedestalDescriptor> fPedestalDescriptorMap;
  int nNoiseDescriptors;
  std::map<std::string,NoiseDescriptor> fNoiseDescriptorMap;
  int nPeakTimeDescriptors;
  std::map<std::string,PeakTimeDescriptor> fPeakTimeDescriptorMap;
  int nGainDescriptors;
  std::map<std::string,GainDescriptor> fGainDescriptorMap;
  int nPinholeDescriptors;
  std::map<std::string,PinholeDescriptor> fPinholeDescriptorMap;
  int nPulseHeightDescriptors;
  std::map<std::string,PulseHeightDescriptor> fPulseHeightDescriptorMap;
  int nPipelineDescriptors;
  std::map<std::string,PipelineDescriptor> fPipelineDescriptorMap;
  int nIVDescriptors;
  std::map<std::string,IVDescriptor> fIVDescriptorMap;
  int nCalAmpDescriptors;
  std::map<std::string,CalAmpDescriptor> fCalAmpDescriptorMap;
  int nBackPlaneDescriptors;
  std::map<std::string,BackPlaneDescriptor> fBackPlaneDescriptorMap;
  int nFBiasDescriptors;
  std::map<std::string,FBiasDescriptor> fFBiasDescriptorMap;
  int nModFlagsDescriptors;
  std::map<std::string,ModFlagsDescriptor> fModFlagsDescriptorMap;




  int PedestalDescriptorParser(DOM_Node &ei);
  int NoiseDescriptorParser(DOM_Node &ei);
  int PulseShapeDescriptorParser(DOM_Node &ei);
  int PulseHeightDescriptorParser(DOM_Node &ei);
  int PinholeDescriptorParser(DOM_Node &ei);
  int PeakTimeDescriptorParser(DOM_Node &ei);
  int GainDescriptorParser(DOM_Node &ei);
  int PipelineDescriptorParser(DOM_Node &ei);
  int IVDescriptorParser(DOM_Node &ei);
  int CalAmpDescriptorParser(DOM_Node &ei);
  int FBiasDescriptorParser(DOM_Node &ei);
  int BackPlaneDescriptorParser(DOM_Node &ei);
  int ModFlagsDescriptorParser(DOM_Node &ei);
  
 public:
  XmlSetParser();
  ~XmlSetParser();
  int  parseFile(const char* name);
  std::string getFileName(){ return fFileName;}
 void setFitmod(int);
  /// Clear all Description
  void clear();
  
  int getNumberPedestalDescriptors() const {return nPedestalDescriptors;}
  int getNumberNoiseDescriptors()    const {return  nNoiseDescriptors;}
  int getNumberPipelineDescriptors() const {return  nPipelineDescriptors;}
  int getNumberPulseHeightDescriptors() const {return  nPulseHeightDescriptors;} 
  int getNumberPinholetDescriptors() const {return  nPinholeDescriptors;}
  int getNumberPeakTimeDescriptors() const {return  nPeakTimeDescriptors;}
  int getNumberGainDescriptors()     const {return  nGainDescriptors;}
  int getNumberIVDescriptors()       const {return  nIVDescriptors;}
  int getNumberCalAmpDescriptors()   const {return  nCalAmpDescriptors;}
  int getNumberFBiasDescriptors()    const {return  nFBiasDescriptors;}
  int getNumberBackPlaneDescriptors()   const {return  nBackPlaneDescriptors;}
  int getNumberModFlagsDescriptors() const {return nModFlagsDescriptors;}
  
  int setDescriptor(AnalDescriptor* nd);
  /** returns the pointer to the Description (settings) 
      @param selname  the id- name for the  settings  that should be found in the xml file.
  */
  // getters
PedestalDescriptor*   getPedestalDescriptor( std::string selname );
NoiseDescriptor*   getNoiseDescriptor( std::string selname ); 
PeakTimeDescriptor*   getPeakTimeDescriptor( std::string selname ); 
GainDescriptor*   getGainDescriptor( std::string selname ); 
PinholeDescriptor*   getPinholeDescriptor( std::string selname ); 
PulseHeightDescriptor*   getPulseHeightDescriptor( std::string selname); 
PipelineDescriptor*   getPipelineDescriptor( std::string selname ); 
IVDescriptor*   getIVDescriptor( std::string selname ); 
CalAmpDescriptor*   getCalAmpDescriptor( std::string selname );
FBiasDescriptor*   getFBiasDescriptor( std::string selname );
BackPlaneDescriptor*   getBackPlaneDescriptor( std::string selname );
ModFlagsDescriptor*   getModFlagsDescriptor( std::string selname );
  
AnalDescriptor*  getModuleDescriptor(std::string name);
};

#endif
