#include "AnalSetParser.h"
#include <util/PlatformUtils.hpp>
#include <util/XMLString.hpp>
#include <util/XMLUniDefs.hpp>
#include <framework/XMLFormatter.hpp>
#include <util/TranscodingException.hpp>
#include <dom/DOM_DOMException.hpp>
#include <parsers/DOMParser.hpp>
#include <dom/DOM.hpp>
#include <dom/DOM_NamedNodeMap.hpp>
#include "DOMTreeErrorReporter.hpp"
#include <cstdio>

using namespace std;


static bool                     gDoNamespaces          = false;
static bool                     gDoCreate              = false;
static XMLCh*                   gEncodingName          = 0;
//static XMLFormatter::UnRepFlags gUnRepFlags            = XMLFormatter::UnRep_CharRef;
static DOMParser::ValSchemes    gValScheme             = DOMParser::Val_Auto;
static XMLFormatter*            gFormatter             = 0;

AnalSetParser::AnalSetParser() {
   fFileName="none";
   clear();
   DT_NOTE(" init setting parser clear  done ");
}

AnalSetParser::~AnalSetParser(){ 
clear();
   }

void AnalSetParser::clear()
{ 
  DT_DEBUG_SP( " Clear PedestalDescriptor "<<nPedestalDescriptors); 
  fPedestalDescriptorMap.clear();
  nPedestalDescriptors=0; 
  
  DT_DEBUG_SP(" Clear NoiseDescriptor "<<nNoiseDescriptors); 
  fNoiseDescriptorMap.clear();
  nNoiseDescriptors=0;
   
  DT_DEBUG_SP(" Clear PeakTimeDescriptor "<<nPeakTimeDescriptors); 
  fPeakTimeDescriptorMap.clear();
  nPeakTimeDescriptors=0;

  DT_DEBUG_SP(" Clear PulseHeighDescriptor "<<nPulseHeightDescriptors); 
  fPulseHeightDescriptorMap.clear();
  nPulseHeightDescriptors=0;

  DT_DEBUG_SP(" Clear PinholeDescriptor "<<nPinholeDescriptors); 
  fPinholeDescriptorMap.clear();
  nPinholeDescriptors=0;
   
  DT_DEBUG_SP(" Clear PipelineDescriptor "<<nPipelineDescriptors); 
  fPipelineDescriptorMap.clear();
  nPipelineDescriptors=0;
  
  DT_DEBUG_SP(" Clear ModFlagsDescriptor "<<nModFlagsDescriptors); 
  fModFlagsDescriptorMap.clear();
  nModFlagsDescriptors=0;
  
 DT_DEBUG_SP(" IVDescriptor "<<fIVDescriptorMap.size()); 
 fIVDescriptorMap.clear();
 
 
 DT_NOTE(" clear done");
}
int  AnalSetParser::parseFile(const char* name) {    
  fFileName=name;
  DT_NOTE(" parseFile "<<fFileName);
  try
    {
      XMLPlatformUtils::Initialize();
    }
  
  catch(const XMLException& toCatch) { cerr << "Error during";
  cerr<<"Xerces-c Initialization.\n" << " Exception message:" <<
  DOMString(toCatch.getMessage()).transcode() << endl; return -2; }
 
  DOMParser *parser;
  parser = new DOMParser;

  parser->setValidationScheme(gValScheme);
  parser->setDoNamespaces(gDoNamespaces);
  parser->setCreateEntityReferenceNodes(gDoCreate);
  parser->setToCreateXMLDeclTypeNode(true);
  ErrorHandler *errReporter = new DOMTreeErrorReporter();
  
  parser->setErrorHandler(errReporter);
  
  bool errorsOccured = false;
  try
    {
      parser->parse(name);
      
    }
  
  catch (const XMLException& e)
    {
      cerr << "An error occured during parsing 1\n   Message: "
           << DOMString(e.getMessage()).transcode() << endl;
      errorsOccured = true;
    }
  
  
  catch (const DOM_DOMException& e)
    {
      cerr << "An error occured during parsing 2 \n   Message: "
           << DOMString(e.msg).transcode() << endl;
      errorsOccured = true;
    }
  
  catch (...)
    {
      cerr << "An error occured during parsing 4 \n " << endl;
      errorsOccured = true;
    }
  // If the parse was successful, output the document data from the DOM tree
  
  if (!errorsOccured)
    {
      DT_DEBUG_SP( " get doc===== ");
      DOM_Node doc = parser->getDocument();
      
      
      
      DOM_Node n1 = doc.getFirstChild();
      
      while (!n1.isNull())
        {
	  if (DOM_Node::TEXT_NODE == n1.getNodeType()) { DT_DEBUG_SP( "skip 2" );}
	  else
	    { DT_DEBUG_SP( n1.getNodeName().transcode() << " type "<< n1.getNodeType());
	    }
	  
          if (n1.getNodeType() == DOM_Node::ELEMENT_NODE) break;
          n1 = n1.getNextSibling();
        }

      DOM_Node n = n1.getFirstChild();
  
      if (n!=0 && n.isNull())
        cout <<" Bad element "<< n.getNodeName().transcode()<< endl;
      while( !n.isNull() )  { 
	// the node was really an element.
	DT_DEBUG_SP(" look for tags for node type  "<<n.getNodeType());
	if (DOM_Node::TEXT_NODE == n.getNodeType()) { 
	  DT_DEBUG_SP("skip 1");
	}
	else {
	  if (strcmp("PedestalDescriptor",n.getNodeName().transcode())==0)  {  
	    PedestalDescriptorParser(n);
	    DT_DEBUG_SP("pedestaldescriptor "<< n.getNodeName().transcode());
	  }
	  if (strcmp("NoiseDescriptor",n.getNodeName().transcode())==0)  {   
	    NoiseDescriptorParser(n);
	    DT_DEBUG_SP(" noisedescriptor"<< n.getNodeName().transcode());
	  }
         if (strcmp("PeakTimeDescriptor",n.getNodeName().transcode())==0) {  
	    PeakTimeDescriptorParser(n);
	    DT_DEBUG_SP("peaktimedescriptor"<< n.getNodeName().transcode());
	  }
        if (strcmp("PulseHeightDescriptor",n.getNodeName().transcode())==0) {  
	    PulseHeightDescriptorParser(n);
	    DT_DEBUG_SP("pulseheightdescriptor"<< n.getNodeName().transcode());
	  }
        if (strcmp("PinholeDescriptor",n.getNodeName().transcode())==0) {  
	    PinholeDescriptorParser(n);
	    DT_DEBUG_SP("pinholedescriptor"<< n.getNodeName().transcode());
	  }
       if (strcmp("GainDescriptor",n.getNodeName().transcode())==0) {  
	    GainDescriptorParser(n);
	    DT_DEBUG_SP("gaindescriptor"<< n.getNodeName().transcode());
	  }
	  if (strcmp("IVDescriptor",n.getNodeName().transcode())==0){  
	    IVDescriptorParser(n);
	    DT_DEBUG_SP("ivdescriptor"<< n.getNodeName().transcode());
	  }
	  if (strcmp("PipelineDescriptor",n.getNodeName().transcode())==0) {  
	    PipelineDescriptorParser(n);
	    DT_DEBUG_SP("pipelinedescriptor"<< n.getNodeName().transcode());
	  }
	  if (strcmp("ModFlagsDescriptor",n.getNodeName().transcode())==0) {  
	    //**	    SensorDescriptorParser(n); should be ...
	    ModFlagsDescriptorParser(n);
	    DT_DEBUG_SP("modflagsdescriptor"<< n.getNodeName().transcode());
	  } 

	}


          n = n.getNextSibling();
          if (n!=0 && n.isNull())
            cout << " Bad element "<< n.getNodeName().transcode() << endl;
        }
      DT_DEBUG_SP("file processed clean up parser" );
  DT_NOTE(" File parsed ");
    }

    
  delete gFormatter;

  delete gEncodingName;
  delete parser;

  return (int) errorsOccured ;
}

// descriptors  parsers 

int  AnalSetParser::PedestalDescriptorParser(DOM_Node &ei)
{
 DT_NOTE(" Pedestal ");
  DOM_NamedNodeMap attributes = ei.getAttributes();

  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");  
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
 PedestalDescriptor* a = new PedestalDescriptor();
 a->ParseSettings(e);
 fPedestalDescriptorMap[temp]= *a; // copy object by value to map
 delete a;
   nPedestalDescriptors++ ; 
  return 0;
}

int  AnalSetParser::NoiseDescriptorParser(DOM_Node &ei)
{
 DT_NOTE(" Pedestal ");
  DOM_NamedNodeMap attributes = ei.getAttributes();

  string nameid;
  char temp[30];
  //** change NoiseDescriptor variables name
  DOM_Node e = attributes.getNamedItem("name"); 
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  NoiseDescriptor* a = new NoiseDescriptor();
  a->ParseSettings(e);
  fNoiseDescriptorMap[temp]= *a;
  delete a;
  nNoiseDescriptors++ ; 
  return 0;
}

int  AnalSetParser::PulseHeightDescriptorParser(DOM_Node &ei)
{
 DT_NOTE(" PulseHeight ");
  DOM_NamedNodeMap attributes = ei.getAttributes();

  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name"); 
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  PulseHeightDescriptor* a = new PulseHeightDescriptor();
  a->ParseSettings(e);
  fPulseHeightDescriptorMap[temp]= *a;
  delete a;
  nPulseHeightDescriptors++ ; 
  return 0;
}

int  AnalSetParser::PeakTimeDescriptorParser(DOM_Node &ei)
{
 DT_NOTE(" PulseHeight ");
  DOM_NamedNodeMap attributes = ei.getAttributes();

  string nameid;
  char temp[30];

  DOM_Node e = attributes.getNamedItem("name"); 
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  PeakTimeDescriptor* a = new PeakTimeDescriptor();
  a->ParseSettings(e);
  fPeakTimeDescriptorMap[temp]= *a;
  delete a;
  nPeakTimeDescriptors++ ; 
  return 0;
}
//
int  AnalSetParser::PinholeDescriptorParser(DOM_Node &ei)
{
 DT_NOTE(" PulseHeight ");
  DOM_NamedNodeMap attributes = ei.getAttributes();

  string nameid;
  char temp[30];

  DOM_Node e = attributes.getNamedItem("name"); 
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  PinholeDescriptor* a = new PinholeDescriptor();
  a->ParseSettings(e);
  fPinholeDescriptorMap[temp]= *a;
  delete a;
  nPinholeDescriptors++ ; 
  return 0;
}
//
int  AnalSetParser::GainDescriptorParser(DOM_Node &ei)
{
 DT_NOTE(" PulseHeight ");
  DOM_NamedNodeMap attributes = ei.getAttributes();

  string nameid;
  char temp[30];

  DOM_Node e = attributes.getNamedItem("name"); 
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  GainDescriptor* a = new GainDescriptor();
  a->ParseSettings(e);
  fGainDescriptorMap[temp]= *a;
  delete a;
  nGainDescriptors++ ; 
  return 0;
}
//
int  AnalSetParser::PipelineDescriptorParser(DOM_Node &ei)
{
   DT_NOTE(" LatRun ");
  DOM_NamedNodeMap attributes = ei.getAttributes();
  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name"); 
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  PipelineDescriptor* a = new PipelineDescriptor();
  a->ParseSettings(e); 
 fPipelineDescriptorMap[temp]= *a;
 delete a;
   nPipelineDescriptors++ ; 
  return 0;
}
//
int  AnalSetParser::IVDescriptorParser(DOM_Node &ei)
{
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // why ?
  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");
  
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  IVDescriptor* a = new IVDescriptor();
  a->ParseSettings(e);  
 fIVDescriptorMap[temp]= *a;
 delete a;
   nIVDescriptors++ ; 
  return 0;
}
//
  int AnalSetParser::ModFlagsDescriptorParser(DOM_Node &ei)
{
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  //   
  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");
  
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  ModFlagsDescriptor* a = new ModFlagsDescriptor();
 a->ParseSettings(e); 
 fModFlagsDescriptorMap[temp]= *a;
 delete a;
   nModFlagsDescriptors++ ; 
  return 0;
}


AnalDescriptor*   AnalSetParser::getModuleDescriptor(string name) {
  if(getPedestalDescriptor(name)!=0) return  getPedestalDescriptor(name);
  if(getNoiseDescriptor(name)!=0) return  getNoiseDescriptor(name);
  if(getPulseHeightDescriptor(name)!=0) return  getPulseHeightDescriptor(name);
  if(getPeakTimeDescriptor(name)!=0) return  getPeakTimeDescriptor(name);
  if(getPipelineDescriptor(name)!=0) return  getPipelineDescriptor(name);
  if(getIVDescriptor(name)!=0) return  getIVDescriptor(name);
  if(getGainDescriptor(name)!=0) return  getGainDescriptor(name);
  if(getModFlagsDescriptor(name)!=0) return  getModFlagsDescriptor(name);
  return 0;
}

int  AnalSetParser::setDescriptor(AnalDescriptor* nd) {
    if(!nd) {
      DT_WARN("descriptor is null ... exit");
      return -1;
    }
    if(!(nd->Name).CompareTo("NoiseDescriptor")) {
      NoiseDescriptor* dtmp = new NoiseDescriptor();
      dtmp = (NoiseDescriptor*) nd;
      fNoiseDescriptorMap["dummy"] = *dtmp;
      fNoiseDescriptorMap["ghost"] = *dtmp;
      return 1;
    }
    else if(!(nd->Name).CompareTo("PedestalDescriptor")) {
      PedestalDescriptor* dtmp = new PedestalDescriptor();
      dtmp = (PedestalDescriptor*) nd;
      fPedestalDescriptorMap["dummy"] = *dtmp;
      fPedestalDescriptorMap["ghost"] = *dtmp;
      return 1;
    }
    else if(!(nd->Name).CompareTo("PeakTimeDescriptor")) {
      PeakTimeDescriptor* dtmp = new PeakTimeDescriptor();
      dtmp = (PeakTimeDescriptor*) nd;
      fPeakTimeDescriptorMap["dummy"] = *dtmp;
      fPeakTimeDescriptorMap["ghost"] = *dtmp;
      return 1;
    }
    else if(!(nd->Name).CompareTo("GainDescriptor")) {
      GainDescriptor* dtmp = new GainDescriptor();
      dtmp = (GainDescriptor*) nd;
      fGainDescriptorMap["dummy"] = *dtmp;
      fGainDescriptorMap["ghost"] = *dtmp;
      return 1;
    }
    else if(!(nd->Name).CompareTo("PinholeDescriptor")) {
      PinholeDescriptor* dtmp = new PinholeDescriptor();
      dtmp = (PinholeDescriptor*) nd;
      fPinholeDescriptorMap["dummy"] = *dtmp;
      fPinholeDescriptorMap["ghost"] = *dtmp;
      return 1;
    }
    else if(!(nd->Name).CompareTo("PulseHeightDescriptor")) {
      PulseHeightDescriptor* dtmp = new PulseHeightDescriptor();
      dtmp = (PulseHeightDescriptor*) nd;
      fPulseHeightDescriptorMap["dummy"] = *dtmp;
      fPulseHeightDescriptorMap["ghost"] = *dtmp;
      return 1;
    }
   else if(!(nd->Name).CompareTo("PipelineDescriptor")) {
      PipelineDescriptor* dtmp = new PipelineDescriptor();
      dtmp = (PipelineDescriptor*) nd;
      fPipelineDescriptorMap["dummy"] = *dtmp;
      fPipelineDescriptorMap["ghost"] = *dtmp;
      return 1;
    }
   else if(!(nd->Name).CompareTo("IVDescriptor")) {
      IVDescriptor* dtmp = new IVDescriptor();
      dtmp = (IVDescriptor*) nd;
      fIVDescriptorMap["dummy"] = *dtmp;
      fIVDescriptorMap["ghost"] = *dtmp;
      return 1;
    }
    else if(!(nd->Name).CompareTo("ModFlagsDescriptor")) {
      ModFlagsDescriptor* dtmp = new ModFlagsDescriptor();
      dtmp = (ModFlagsDescriptor*) nd;
      fModFlagsDescriptorMap["dummy"] = *dtmp;
      fModFlagsDescriptorMap["ghost"] = *dtmp;
      return 1;
    }
    else {
      std::cout<<"[AnaSetParser::setDescriptor] WARNING! "<<nd->Name
	  <<" is not a valid descriptor name"<<std::endl;
      return -1;
    }
  }
