#include "BasicTextDescriptor.h"
#include "PedestalDescriptor.h"
#include "NoiseDescriptor.h"
#include "PeakTimeDescriptor.h"
#include "GainDescriptor.h"
#include "PinholeDescriptor.h"
#include "PulseHeightDescriptor.h"
#include "PulseShapeDescriptor.h"
#include "FBiasDescriptor.h"
#include "IVDescriptor.h"
#include "PipelineDescriptor.h"
#include "BackPlaneDescriptor.h"
#include "CalAmpDescriptor.h"
#include "CalLightDescriptor.h"
#include "ModFlagsDescriptor.h"

#include "RecUtil.h"
using namespace RecUtil;
using namespace std;

BasicTextDescriptor::BasicTextDescriptor(){
  if(init()==0)
    isgood = false ;
  else isgood = true;
}
BasicTextDescriptor::~BasicTextDescriptor()
{
 if (fPedestalDescriptor)    delete fPedestalDescriptor;
 if (fNoiseDescriptor)       delete fNoiseDescriptor;   
 if (fPeakTimeDescriptor)    delete fPeakTimeDescriptor;
 if (fGainDescriptor)        delete fGainDescriptor;
 if (fPinholeDescriptor)     delete fPinholeDescriptor;
 if (fPulseHeightDescriptor) delete fPulseHeightDescriptor;
 if (fPipelineDescriptor)    delete fPipelineDescriptor;  
 if (fBackPlaneDescriptor)   delete fBackPlaneDescriptor; 
 if (fCalAmpDescriptor)      delete fCalAmpDescriptor;    
 if (fFBiasDescriptor)       delete fFBiasDescriptor;    
 if (fIVDescriptor)          delete fIVDescriptor;     
 if (fModFlagsDescriptor)    delete fModFlagsDescriptor;  
}

int BasicTextDescriptor:: parseFile(const char* name) {
  fFileName = name;
  fin.open(name);
  cout<<"fin.Good = "<<fin.good()<<endl;
  if( !fin.good() ) {
    cout<<"[BasicTextDescriptor::parseFile()] ERROR: config file "
	<<fFileName<<" not readeable ..."<<endl<<endl;
    return 1;
  }
  fillDescriptor();
  return 0;
}
// ------------------------------------------------------
int BasicTextDescriptor::init()
{
  fFileName = "none";
  fPedestalDescriptor    = 0;  
  fNoiseDescriptor       = 0; 
  fPeakTimeDescriptor    = 0;   
  fGainDescriptor        = 0;
  fPinholeDescriptor     = 0;
  fPulseHeightDescriptor = 0;
  fPipelineDescriptor    = 0;
  fBackPlaneDescriptor   = 0; 
  fCalAmpDescriptor      = 0;    
  fFBiasDescriptor       = 0;     
  fIVDescriptor          = 0;     	   
  fModFlagsDescriptor    = 0;  

  return 1;
}
int BasicTextDescriptor::fillDescriptor()
{
  
  if(!fillPedestalDescriptor())    fPedestalDescriptor    = 0; 
  if(!fillNoiseDescriptor())       fNoiseDescriptor       = 0;    
  if(!fillPeakTimeDescriptor())    fPeakTimeDescriptor    = 0;   
  if(!fillGainDescriptor())        fGainDescriptor        = 0;
  if(!fillPinholeDescriptor())     fPinholeDescriptor     = 0;
  if(!fillPulseHeightDescriptor()) fPulseHeightDescriptor = 0;
  if(!fillPipelineDescriptor())    fPipelineDescriptor    = 0; 
  if(!fillBackPlaneDescriptor())   fBackPlaneDescriptor   = 0;
  if(!fillCalAmpDescriptor())      fCalAmpDescriptor      = 0;
  if(!fillFBiasDescriptor())       fFBiasDescriptor       = 0;
  if(!fillIVDescriptor())          fIVDescriptor          = 0;
  if(!fillModFlagsDescriptor())    fModFlagsDescriptor    = 0;
  
  return 1;
}
int BasicTextDescriptor::fillPedestalDescriptor()
{
  fPedestalDescriptor = new PedestalDescriptor();
  char line[100];
  bool tag = false;
  TString modeName;
  char* TagName = "PedestalCuts";
  
  while (!tag) {
    if(fin.eof() || !fin.good()) break;
    fin.getline(line, 100); 
    TString fline(line);
    tag = ( fline.Contains(TagName) );
  }
  if(tag) {
    fin >> fPedestalDescriptor->pedCutType;
    fin >> modeName;  if( modeName.CompareTo("PeakOn"))  {
      cout<<"[BasicTextDescriptor::fillPedestalDescriptor] WARNING expect PeakOn mode"<<endl;
      return 0;
    }
    fin >> fPedestalDescriptor->minPedCutPk;
    fin >> fPedestalDescriptor->maxPedCutPk;
    fin >> modeName;  if( modeName.CompareTo("DecOn"))  {
      cout<<"[BasicTextDescriptor::fillPedestalDescriptor] WARNING expect DecOn mode"<<endl;
      return 0;
    }
    fin >> fPedestalDescriptor->minPedCutDC;
    fin >> fPedestalDescriptor->maxPedCutDC;    
  }
  
  return 1;
}
// -----------------------------------------------------
//
int BasicTextDescriptor::fillNoiseDescriptor()
{
  fNoiseDescriptor = new NoiseDescriptor();
  char line[100];
  bool tag = false;
  TString modeName;
  char* TagName = "NoiseCuts";
  
  while (!tag) {
    if(fin.eof() || !fin.good()) break;
    fin.getline(line, 100); 
    TString fline(line);
    tag = ( fline.Contains(TagName) );
  }
  if(tag) {
    fin >> fNoiseDescriptor->noiseCutType;
    fin >> modeName;  if( modeName.CompareTo("PeakOn"))  {
      cout<<"[BasicTextDescriptor::fillNoiseDescriptor] WARNING expect PeakOn mode"<<endl;
      return 0;
    }
    fin >> fNoiseDescriptor->minOneSensNoiseCutPk;
    fin >> fNoiseDescriptor->minTwoSensNoiseCutPk;
    fin >> fNoiseDescriptor->minNoiseCutPk;
    fin >> fNoiseDescriptor->maxNoiseCutPk;
    fin >> modeName;  if( modeName.CompareTo("DecOn"))  {
      cout<<"[BasicTextDescriptor::fillNoiseDescriptor] WARNING expect DecOn mode"<<endl;
      return 0;
    }
    fin >> fNoiseDescriptor->minOneSensNoiseCutDC;
    fin >> fNoiseDescriptor->minTwoSensNoiseCutDC;
    fin >> fNoiseDescriptor->minNoiseCutDC;
    fin >> fNoiseDescriptor->maxNoiseCutDC;
    
  }
  return 1;
}
// -----------------------------------------------------
//
int BasicTextDescriptor::fillPeakTimeDescriptor()
{
  fPeakTimeDescriptor = new PeakTimeDescriptor();
  char line[100];
  bool tag = false;
  TString modeName;
  char* TagName = "PeakTimeCuts";
  
  while (!tag) {
    if(fin.eof() || !fin.good()) break;
    fin.getline(line, 100); 
    TString fline(line);
    tag = ( fline.Contains(TagName) );
  }
  if(tag) {
    fin >> fPeakTimeDescriptor->peaktimeCutType;
    fin >> modeName;  if( modeName.CompareTo("PeakOn"))  {
      cout<<"[BasicTextDescriptor::fillPeakTimeDescriptor] WARNING expect PeakOn mode"<<endl;
      return 0;
    }
    fin >> fPeakTimeDescriptor->minTSPeakTimeCutPk;
    fin >> fPeakTimeDescriptor->minOSPeakTimeCutPk;
    fin >> fPeakTimeDescriptor->minPeakTimeCutPk;
    fin >> fPeakTimeDescriptor->maxPeakTimeCutPk;  
    fin >> modeName;  if( modeName.CompareTo("DecOn"))  {
      cout<<"[BasicTextDescriptor::fillPeakTimeDescriptor] WARNING expect DecOn mode"<<endl;
      return 0;
    }
    fin >> fPeakTimeDescriptor->minTSPeakTimeCutDC;
    fin >> fPeakTimeDescriptor->minOSPeakTimeCutDC;
    fin >> fPeakTimeDescriptor->minPeakTimeCutDC;  
    fin >> fPeakTimeDescriptor->maxPeakTimeCutDC;  
    
  }
  return 1;
}
// -----------------------------------------------------
//
int BasicTextDescriptor::fillPinholeDescriptor()
{
  fPinholeDescriptor = new PinholeDescriptor();
  char line[100];
  bool tag = false;
  TString modeName;
  char* TagName = "PinholeCuts";
  
  while (!tag) {
    if(fin.eof() || !fin.good()) break;
    fin.getline(line, 100); 
    TString fline(line);
    tag = ( fline.Contains(TagName) );
  }
  if(tag) {
    fin >> fPinholeDescriptor->pinholeCutType;
    fin >> modeName;  if( modeName.CompareTo("PeakOn"))  {
      cout<<"[BasicTextDescriptor::fillPinholeDescriptor] WARNING expect PeakOn mode"<<endl;
      return 0;
    }
    fin >> fPinholeDescriptor->maxLowPHCutPk;
    fin >> fPinholeDescriptor->maxHighPHCutPk;
    fin >> modeName;  if( modeName.CompareTo("DecOn"))  {
      cout<<"[BasicTextDescriptor::fillPinholeDescriptor] WARNING expect DecOn mode"<<endl;
      return 0;
    }
    fin >> fPinholeDescriptor->maxLowPHCutDC;
    fin >> fPinholeDescriptor->maxHighPHCutDC;
  }
  return 1;
}
// -----------------------------------------------------
//
int BasicTextDescriptor::fillGainDescriptor()
{
  fGainDescriptor = new GainDescriptor();
  char line[100];
  bool tag = false;
  TString modeName;
  char* TagName = "GainCuts";
  
  while (!tag) {
    if(fin.eof() || !fin.good()) break;
    fin.getline(line, 100); 
    TString fline(line);
    tag = ( fline.Contains(TagName) );
  }
  if(tag) {
    fin >> fGainDescriptor->gainCutType;
    fin >> modeName;  if( modeName.CompareTo("PeakOn"))  {
      cout<<"[BasicTextDescriptor::fillGainDescriptor] WARNING expect PeakOn mode"<<endl;
      return 0;
    }
    fin >> fGainDescriptor->minOSGainCutPk;
    fin >> fGainDescriptor->minGainCutPk;
    fin >> fGainDescriptor->maxGainCutPk;
    fin >> fGainDescriptor->maxPHGainCutPk;
    fin >> modeName;  if( modeName.CompareTo("DecOn"))  {
      cout<<"[BasicTextDescriptor::fillGainDescriptor] WARNING expect DecOn mode"<<endl;
      return 0;
    }
    fin >> fGainDescriptor->minOSGainCutDC;
    fin >> fGainDescriptor->minGainCutDC;
    fin >> fGainDescriptor->maxGainCutDC;
    fin >> fGainDescriptor->maxPHGainCutDC;
    
  }
  return 1;
}
// -----------------------------------------------------
//
int BasicTextDescriptor::fillPulseHeightDescriptor()
{
  fPulseHeightDescriptor = new PulseHeightDescriptor();
  char line[100];
  bool tag = false;
  TString modeName;
  char* TagName = "PulseHeightCuts";
  
  while (!tag) {
    if(fin.eof() || !fin.good()) break;
    fin.getline(line, 100); 
    TString fline(line);
    tag = ( fline.Contains(TagName) );
  }
  if(tag) {
    fin >> fPulseHeightDescriptor->pulseheightCutType;
    fin >> modeName;  if( modeName.CompareTo("PeakOn"))  {
      cout<<"[BasicTextDescriptor::fillPulseHeightDescriptor] WARNING expect PeakOn mode"<<endl;
      return 0;
    }
    fin >> fPulseHeightDescriptor->minPulseHeightCutPk;
    fin >> fPulseHeightDescriptor->maxPulseHeightCutPk;
    fin >> modeName;  if( modeName.CompareTo("DecOn"))  {
      cout<<"[BasicTextDescriptor::fillPulseHeightDescriptor] WARNING expect DecOn mode"<<endl;
      return 0;
    }
    fin >> fPulseHeightDescriptor->minPulseHeightCutDC;
    fin >> fPulseHeightDescriptor->maxPulseHeightCutDC;    
  }
  return 1;
}
int BasicTextDescriptor::fillPipelineDescriptor()
{
  return 1;
}

int BasicTextDescriptor::fillBackPlaneDescriptor()
{
  return 1;
}
int BasicTextDescriptor::fillCalAmpDescriptor()
{
  return 1;
}
int BasicTextDescriptor::fillFBiasDescriptor()
{
  return 1;
}
int BasicTextDescriptor::fillIVDescriptor()
{
  return 1;
}

int BasicTextDescriptor::fillModFlagsDescriptor()
{
  fModFlagsDescriptor = new ModFlagsDescriptor();
  return 1;
}
// -------
PedestalDescriptor*   BasicTextDescriptor::getPedestalDescriptor(string s)
{
  if(!fPedestalDescriptor)
    cout<<"[BasicTextDescriptor::getPedestalDescriptor] WARNING : descriptor is null"<<endl;
  return fPedestalDescriptor;
}
// -------
NoiseDescriptor*      BasicTextDescriptor::getNoiseDescriptor(string s)
{
  if(!fNoiseDescriptor)
    cout<<"[BasicTextDescriptor::getNoiseDescriptor] WARNING : descriptor is null"<<endl;
  return fNoiseDescriptor;
}
// -------
PeakTimeDescriptor*      BasicTextDescriptor::getPeakTimeDescriptor(string s)
{
  if(!fPeakTimeDescriptor)
    cout<<"[BasicTextDescriptor::getPeakTimeDescriptor] WARNING : descriptor is null"<<endl;
  return fPeakTimeDescriptor;
}
// -------
GainDescriptor*      BasicTextDescriptor::getGainDescriptor(string s)
{
  if(!fGainDescriptor)
    cout<<"[BasicTextDescriptor::getGainDescriptor] WARNING : descriptor is null"<<endl;
  return fGainDescriptor;
}
// -------
PinholeDescriptor*      BasicTextDescriptor::getPinholeDescriptor(string s)
{
  if(!fPinholeDescriptor)
    cout<<"[BasicTextDescriptor::getPinholeDescriptor] WARNING : descriptor is null"<<endl;
  return fPinholeDescriptor;
}
// -------
PulseHeightDescriptor*      BasicTextDescriptor::getPulseHeightDescriptor(string s)
{
  if(!fPulseHeightDescriptor)
    cout<<"[BasicTextDescriptor::getPulseHeightDescriptor] WARNING : descriptor is null"<<endl;
  return fPulseHeightDescriptor;
}
// -------
PipelineDescriptor*   BasicTextDescriptor::getPipelineDescriptor(string s)
{
  if(!fPipelineDescriptor)
    cout<<"[BasicTextDescriptor::getPipelineDescriptor] WARNING : descriptor is null"<<endl;
  return fPipelineDescriptor;
}

BackPlaneDescriptor*  BasicTextDescriptor::getBackPlaneDescriptor(string s)
{
  if(!fBackPlaneDescriptor)
    cout<<"[BasicTextDescriptor::getBackPlaneDescriptor] WARNING : descriptor is null"<<endl;
  return fBackPlaneDescriptor;
}
CalAmpDescriptor*     BasicTextDescriptor::getCalAmpDescriptor(string s)
{
  if(!fCalAmpDescriptor)
    cout<<"[BasicTextDescriptor::getCalAmpDescriptor] WARNING : descriptor is null"<<endl;
  return fCalAmpDescriptor;
} 
FBiasDescriptor*      BasicTextDescriptor::getFBiasDescriptor(string s)
{
  if(!fFBiasDescriptor)
    cout<<"[BasicTextDescriptor::getFBiasDescriptor] WARNING : descriptor is null"<<endl;
  return fFBiasDescriptor;
}
IVDescriptor*         BasicTextDescriptor::getIVDescriptor(string s)
{
  if(!fIVDescriptor)
    cout<<"[BasicTextDescriptor::getIVDescriptor] WARNING : descriptor is null"<<endl;
  return fIVDescriptor;
}

ModFlagsDescriptor*   BasicTextDescriptor::getModFlagsDescriptor(string s)
{
  if(!fModFlagsDescriptor)
    cout<<"[BasicTextDescriptor::getModFlagsDescriptor] WARNING : descriptor is null"<<endl;
  return fModFlagsDescriptor;
}
void   BasicTextDescriptor::setFitmod(int mfit)
{
  if(fPulseHeightDescriptor)
    fPulseHeightDescriptor->fitmode = mfit;
}
