#include <iostream>

#include "TLegend.h"
#include "TPaveLabel.h"
#include "TPaveText.h"

#include "DefAnaHistogrammer.h"
#include "RecUtil.h"
#include "TestAnalyzer.h"
#include "ModAnalyzer.h"
#include "ModLtAnalyzer.h"
#include "StructLtAnalyzer.h"
#include "ModLtRootReader.h"
#include "ModRootReader.h"
#include "StructLtRootReader.h"

#include "daqerrormsg.h"
#include "DefaultPath.h"
#include "cmdlinedecode.h"

using namespace RecUtil;
using namespace lttest;
using namespace std;

DefAnaHistogrammer::DefAnaHistogrammer(DefAnalyzer* da, BaseAnalyzer *ma){
  a = da->getTestAnalyzer();
  if(StructLtAnalyzer* m = dynamic_cast<StructLtAnalyzer*>(ma)){
    rr =  (StructLtRootReader*) a->getRootReader();
  }
  else    if(ModAnalyzer* m = dynamic_cast<ModAnalyzer*>(ma)){
    rr =  (ModRootReader*) a->getRootReader();
  }
  else    if(ModLtAnalyzer* m = dynamic_cast<ModLtAnalyzer*>(ma)){
    rr =  (ModLtRootReader*) a->getRootReader();
  }
  init();
}
void DefAnaHistogrammer::init()
{
  cout<<"[DefAnaHistogrammer::init()]                                  "<<endl;
  nChip = a->getNChip();
  nChanPerChip = a->getNchan()/nChip;
  TString softname = (a->getRootReader())->getSoftware();
  if(softname.Contains("arc")) {
    SetPedBound(35., 120.);
    SetNoiseBound(0.1, 4.8);
    SetPeakTimeBound(-100., 20.);
    SetGainBound(0.9, 1.5);
    SetXLabBound(0.6, 0.9);
    SetYLabBound(0.9, 1.);
  }
  else if(softname.Contains("lt")) {
    SetPedBound(100., 400.);
    SetNoiseBound(0.1, 4.8);
    SetPeakTimeBound(-100., 20.);
    SetGainBound(0.9, 1.5);
    SetXLabBound(0.6, 0.9);
    SetYLabBound(0.9, 1.);
  }
  TString modnum = (a->getRootReader())->GetId();
  int recnum = (a->getRootReader())->GetRecordNr();
  
  TPaveLabel canHeader(5,23.5,20,24.5,Form("%s; Record%d; %s",modnum.Data(),recnum,rr->GetDate().Data()),"br");
}
void DefAnaHistogrammer::PrintHistogramms()
{
  cout<<"[DefAnaHistogrammer::PrintHistogramms()]                      "<<endl;

  PlotPeakTime();
  PlotNoise();
  PlotPedestal();
  PlotPulseHeight();
  PlotGain("Slope");
  PlotGain("Chi2");
  PlotGain("Offset");
  PlotPinhole();
}
// ----------------------------------------------------------------------------
  void DefAnaHistogrammer::SetPedBound(double min, double max) 
{
pedHistMin = min; 
pedHistMax = max;
}
// ----------------------------------------------------------------------------
  void DefAnaHistogrammer::SetNoiseBound(double min, double max) 
{
noiseHistMin = min; 
noiseHistMax = max;
}
// ----------------------------------------------------------------------------
  void DefAnaHistogrammer::SetPeakTimeBound(double min, double max) 
{
peaktimeHistMin = min; 
peaktimeHistMax = max;
}
// ----------------------------------------------------------------------------
  void DefAnaHistogrammer::SetGainBound(double min, double max) 
{
gainHistMin = min; 
gainHistMax = max;
}
// ----------------------------------------------------------------------------
  void DefAnaHistogrammer::SetXLabBound(double min, double max) 
{
xMinLab = min; 
xMaxLab = max;
}
// ----------------------------------------------------------------------------
  void DefAnaHistogrammer::SetYLabBound(double min, double max) 
{
yMinLab = min; 
yMaxLab = max;
}
// ----------------------------------------------------------------------------
int DefAnaHistogrammer::PlotPeakTime()
{
  TString modnum = (a->getRootReader())->GetId();
  int recnum = (a->getRootReader())->GetRecordNr();
  char *ofname = Form("%s/hPeakTime_%s_Rec%d.eps",DefaultDataPath().c_str(),
		      modnum.Data(),recnum);
  
  char *name = Form("PSPeakTime_Rec%d",DefaultDataPath().c_str(),recnum);

  TCanvas* canvas1;
  canvas1 = new TCanvas();
  canvas1->Divide(2,2);
  TH1F* h;
  for(int imode = 0; imode<NModeType; imode++) {
    (a->getRootReader())->SetModeDir(ModeType[imode], recnum);
    TH1F *h;
    TString softname = (a->getRootReader())->getSoftware();
    TString hname;
    if(softname.Contains("arc")) {
  canvas1->SetName("PeakTime");
  canvas1->SetTitle("PeakTime");
  canvas1->SetFillColor(0);
  canvas1->Draw();
  hname = "PeakTime"; // ARC
  }
    else if(softname.Contains("lt")) {
  canvas1->SetName("RiseTime");
  canvas1->SetTitle("RiseTime");
  canvas1->SetFillColor(0);
  canvas1->Draw();
  hname = "RiseTime"; //LT
  }
    h = (TH1F*) (a->getRootReader())->Find(hname);
    if(!h) {
      cout <<"[DefAnaHistogrammer::PlotPeakTime] WARNING: "<<gDirectory->GetPath()
	   <<" is missing plots "<<hname<<endl;
      return -1;
    }
    TH1F *hsub  = (TH1F*) h->Clone();
    hsub->SetMinimum(peaktimeHistMin);
    hsub->SetMaximum(peaktimeHistMax);
    TH1F *hmean = (TH1F*) h->Clone(); hmean->Reset("ICE");
    
    vector<double> vpt_avg;  vector<double> vpt_rms;
    CalcChipAvg(h, vpt_avg, vpt_rms);
    for(int i = 0; i < nChip; i++) 
      for (int is = 0; is < nChanPerChip; is++)
	hmean->SetBinContent(i*nChanPerChip + is + 1,vpt_avg[i]);
    hsub->Add(hmean, -1.);
    canvas1->cd(imode+1);
    hsub->Draw();

    TLegend *text = new TLegend(0.15,0.65,0.5,0.90);
    text->SetFillColor(0); text->SetFillStyle(0); text->SetBorderSize(0);
    text->SetHeader(ModeType[imode]);
    text->SetTextSize(0.05);
    text->SetTextAlign(11);
    text->Draw("same");
  }
  canvas1->Update();
  //  RecUtil::wait();
  canvas1->SaveAs(ofname);
  canvas1->Draw();
  
  return 1;
}
// ----------------------------------------------------------------------------
int DefAnaHistogrammer::PlotPedestal()
{
  TString modnum = (a->getRootReader())->GetId();
  int recnum = (a->getRootReader())->GetRecordNr();
  char *ofname = Form("%s/hPedestal_%s_Rec%d.eps",DefaultDataPath().c_str(),
		      modnum.Data(),recnum);
  
  char *name = Form("PSPedestal_Rec%d",DefaultDataPath().c_str(),recnum);
  
  TCanvas* canvas2;
  canvas2 = new TCanvas();
  canvas2->Divide(2,2);
  canvas2->SetName("Pedestal");
  canvas2->SetTitle("Pedestal");
  canvas2->SetFillColor(0);
  canvas2->Draw();

  TH1F* h;
  for(int imode = 0; imode<NModeType; imode++) {
    (a->getRootReader())->SetModeDir(ModeType[imode], recnum);
    h = (TH1F*) (a->getRootReader())->Find("Pedestal");
    if(!h) {
      cout <<"[DefAnaHistogrammer::PlotPedestal] WARNING: "<<gDirectory->GetPath()
	   <<" is missing plots  Pedestal"<<endl;
      return -1;
    }
    h->SetMinimum(pedHistMin);
    h->SetMaximum(pedHistMax);
    canvas2->cd(imode+1);
    h->Draw();

    TLegend *text = new TLegend(0.15,0.65,0.5,0.90);
    text->SetFillColor(0); text->SetFillStyle(0); text->SetBorderSize(0);
    text->SetHeader(ModeType[imode]);
    text->SetTextSize(0.05);
    text->SetTextAlign(11);
    text->Draw("same");
  }
  canvas2->Update();
  //  RecUtil::wait();
  canvas2->SaveAs(ofname);
  canvas2->Draw();
  
//  delete h;
  return 1;
}
// ----------------------------------------------------------------------------
int DefAnaHistogrammer::PlotPulseHeight()
{
  TString modnum = (a->getRootReader())->GetId();
  int recnum = (a->getRootReader())->GetRecordNr();
  char *ofname = Form("%s/hPulseHeight_%s_Rec%d.eps",DefaultDataPath().c_str(),
		      modnum.Data(),recnum);
  
  char *name = Form("PSPulseHeight_Rec%d",DefaultDataPath().c_str(),recnum);
  
  TCanvas* canvas3;
  canvas3 = new TCanvas();
  canvas3->Divide(2,2);
  canvas3->SetName("PulseHeight");
  canvas3->SetTitle("PulseHeight");
  canvas3->SetFillColor(0);
  canvas3->Draw();

  TH1F* h;
  for(int imode = 0; imode<NModeType; imode++) {
    (a->getRootReader())->SetModeDir(ModeType[imode], recnum);
    h = (TH1F*) (a->getRootReader())->Find("PulseHeight");
    if(!h) {
      cout <<"[DefAnaHistogrammer::PlotPulseHeight] WARNING: "<<gDirectory->GetPath()
	   <<" is missing plots  PulseHeight"<<endl;
      return -1;
    }
    canvas3->cd(imode+1);
    h->Draw();

    TLegend *text = new TLegend(0.15,0.65,0.5,0.90);
    text->SetFillColor(0); text->SetFillStyle(0); text->SetBorderSize(0);
    text->SetHeader(ModeType[imode]);
    text->SetTextSize(0.05);
    text->SetTextAlign(11);
    text->Draw("same");
  }
  canvas3->Update();
  //  RecUtil::wait();
  canvas3->SaveAs(ofname);
  canvas3->Draw();
  
//  delete h;
  return 1;
}
// ----------------------------------------------------------------------------
int DefAnaHistogrammer::PlotNoise()
{
  TString modnum = (a->getRootReader())->GetId();
  int recnum = (a->getRootReader())->GetRecordNr();
  char *ofname = Form("%s/hNoise_%s_Rec%d.eps",DefaultDataPath().c_str(),
		      modnum.Data(),recnum);
  
  char *name = Form("PSNoise_Rec%d",DefaultDataPath().c_str(),recnum);
  
  TCanvas* canvas4;
  canvas4 = new TCanvas();
  canvas4->Divide(2,2);
  canvas4->SetName("Noise");
  canvas4->SetTitle("Noise");
  canvas4->SetFillColor(0);
  canvas4->Draw();

  TH1F *hraw, *hcms;
  for(int imode = 0; imode<NModeType; imode++) {
    (a->getRootReader())->SetModeDir(ModeType[imode], recnum);
    hraw = (TH1F*) (a->getRootReader())->Find("RawNoise");
    hcms = (TH1F*) (a->getRootReader())->Find("CMSubtractedNoise");
    if(!hraw || !hcms) {
      cout <<"[TestAnalyzer::AnalyzePeakTime] WARNING: "<<gDirectory->GetPath()
	   <<" is missing plots  PeakTime"<<endl;
      return -1;
    }
    hraw->SetMinimum(noiseHistMin);     hraw->SetMaximum(noiseHistMax);
    canvas4->cd(imode+1);
    hraw->SetLineColor(kRed);      hraw->Draw();
    hcms->SetLineColor(kBlue);     hcms->Draw("same");
 
    TLegend *text = new TLegend(0.15,0.65,0.5,0.90);
    text->SetFillColor(0); text->SetFillStyle(0); text->SetBorderSize(0);
    text->SetHeader(ModeType[imode]);
    text->AddEntry(hraw,"Raw Noise","l");
    text->AddEntry(hcms,"CMS Noise","l");
    text->SetTextSize(0.05);
    text->SetTextAlign(11);
    text->Draw("same");
    
  }
  canvas4->Update();
  //  RecUtil::wait();
  canvas4->SaveAs(ofname);
  canvas4->Draw();
  
//   delete hraw, hcms;
   return 1;
}
// ----------------------------------------------------------------------------
int DefAnaHistogrammer::PlotGain(TString flgname)
{
  TString modnum = (a->getRootReader())->GetId();
  int recnum = (a->getRootReader())->GetRecordNr();
  char *ofname = Form("%s/hGain%s_%s_Rec%d.eps",flgname.Data(),
		      DefaultDataPath().c_str(),modnum.Data(),recnum);
  
  char *name = Form("PSGain%s_Rec%d",flgname.Data(),DefaultDataPath().c_str(),recnum);
  
  TCanvas* canvas5;
  canvas5 = new TCanvas();
  canvas5->Divide(2,2);
  canvas5->SetName("Gain");
  canvas5->SetTitle(flgname);
  canvas5->SetFillColor(0);
  canvas5->Draw();

  TH1F *hslope, *hoff, *hchi2;
  for(int imode = 0; imode<NModeType; imode++) {
    (a->getRootReader())->SetModeDir(ModeType[imode], recnum);
    hslope = (TH1F*) (a->getRootReader())->Find("Gain_Slope");
    hoff = (TH1F*) (a->getRootReader())->Find("Gain_Offset");
    hchi2 = (TH1F*) (a->getRootReader())->Find("Gain_Chi2");
    if(!hslope || !hoff || !hchi2) {
      cout <<"[DefAnaHistogrammer::PlotGain] WARNING: "<<gDirectory->GetPath()
	   <<" is missing plots"<<endl;
      canvas5->Close();
      return -1;
    }

    canvas5->cd(imode+1);
    if(!flgname.CompareTo("Chi2")) {
      hchi2->Draw();
    }
    if(!flgname.CompareTo("Offset")) {
      hoff->Draw();
    }
    if(!flgname.CompareTo("Slope")) {
     hslope->SetMinimum(gainHistMin);
     hslope->SetMaximum(gainHistMax);
     hslope->Draw();
    }
 
    TLegend *text = new TLegend(0.15,0.65,0.5,0.90);
    text->SetFillColor(0); text->SetFillStyle(0); text->SetBorderSize(0);
    text->SetHeader(ModeType[imode]);
    text->SetTextSize(0.05);
    text->SetTextAlign(11);
    text->Draw("same");
    
  }
   canvas5->Update();
   //    RecUtil::wait();
  canvas5->SaveAs(ofname);
  canvas5->Draw();
  
//  delete hslope, hchi2, hoff;
  return 1;
}
// ----------------------------------------------------------------------------
int DefAnaHistogrammer::PlotPinhole()
{
  TString modnum = (a->getRootReader())->GetId();
  int recnum = (a->getRootReader())->GetRecordNr();
  char *ofname = Form("%s/hPinhole_%s_Rec%d.eps",
		      DefaultDataPath().c_str(),modnum.Data(),recnum);
  
  char *name = Form("PSPinhole_Rec%d",DefaultDataPath().c_str(),recnum);
  
  TCanvas* canvas6;
  canvas6 = new TCanvas();
  canvas6->Divide(1,2);
  canvas6->SetName("Pinhole");
  canvas6->SetTitle("Pinhole");
  canvas6->SetFillColor(0);
  canvas6->Draw();

  TH1F *hdiff, *hmax;
  (a->getRootReader())->SetModeDir("PeakInvOff", recnum); // TEMPORARY
  hdiff = (TH1F*) (a->getRootReader())->Find("Pinhole_Diff");
  hmax  = (TH1F*) (a->getRootReader())->Find("Pinhole_Max");
  if(!hdiff || !hmax) {
    cout <<"[DefAnaHistogrammer::PlotPinhole] WARNING: "<<gDirectory->GetPath()
	 <<" is missing plots Pinhole_Diff and Pinhole_Max"<<endl;
    canvas6->Close(); return -1;
  }
  
  canvas6->cd(1);
  hdiff->Draw();
  
  TLegend *text = new TLegend(0.15,0.65,0.5,0.90);
  text->SetFillColor(0); text->SetFillStyle(0); text->SetBorderSize(0);
  text->SetHeader("PeakInvOff");
  text->SetTextSize(0.05);
  text->SetTextAlign(11);
  text->Draw("same");
  
  canvas6->cd(2);
  hmax->Draw();
  text->Draw("same");
  
  canvas6->Update();
  //  RecUtil::wait();
  
  canvas6->SaveAs(ofname);
  canvas6->Draw();
  
//  delete hmax, hdiff;
  return 1;
}
// ----------------------------------------------------------------------------
void DefAnaHistogrammer::DrawCutLines(float min, float max)
{

}
// ----------------------------------------------------------------------------
void DefAnaHistogrammer::DrawChipLines(float min, float max)
{

}
//** --------------------------------------------------------------------------
int DefAnaHistogrammer::CalcChipAvg(TH1F* h, vector<double>&avg, vector<double>& rms)
{
  avg.clear();
  rms.clear();
  double sum(0.), sum2(0.);
  double val(0.);
  for(int i=0; i< a->getNChip(); i++) {
    sum = sum2 = 0.;
    for(int j=0; j<nChanPerChip; j++) {
      int ichan = i*nChanPerChip + j;
      val   = h->GetBinContent(ichan+1);
      sum  += val;
      sum2 += val*val;
    } //loop on chanels
    double mean = sum/nChanPerChip;
    avg.push_back(mean);
    rms.push_back(sqrt(sum2/nChanPerChip - mean*mean));
  } // loop on chip
  return 1;
}
