#include  "DefAnalyzer.h"
#include  "LtDefs.h"
#include  "AnalDescriptor.h"
#include  "AnalSetParser.h"
#include "PedestalDescriptor.h"
#include "NoiseDescriptor.h"
#include "PeakTimeDescriptor.h"
#include "GainDescriptor.h"
#include "PinholeDescriptor.h"
#include "PulseHeightDescriptor.h"
#include "PulseShapeDescriptor.h"
#include "FBiasDescriptor.h"
#include "IVDescriptor.h"
#include "PipelineDescriptor.h"
#include "BackPlaneDescriptor.h"
#include "CalAmpDescriptor.h"
#include "CalLightDescriptor.h"
#include "ModFlagsDescriptor.h"
#include  "IVDescriptor.h"
#include  "RootReader.h"
#include  "ModRootReader.h"
#include  "RecUtil.h"
#include  "TestAnalyzer.h"
#include <iostream>
using namespace std;
using namespace RecUtil;


DefAnalyzer::DefAnalyzer(TestAnalyzer* anal) 
{
  thetestanalyzer=anal;
  thesets=thetestanalyzer->getAnalSetParser();
  modtype=thetestanalyzer->getModType();
  mbadchannels.clear();  //for all modes
  Nbadch = 0;
}
// -------------------------------------------------------------------------
DefAnalyzer::~DefAnalyzer() 
{
}
// -------------------------------------------------------------------------
void DefAnalyzer::AnalyzeBadCh() 
{
  cout<<" analyse badch "<<endl; 
  ModFlagsDescriptor* f = thetestanalyzer->getAnalSetParser()->getModFlagsDescriptor(thetestanalyzer->getModType());
  
  if(f==0) {
    cout << " ModFlagsDescriptor of " << thetestanalyzer->getModType() 
	 << " not found"<<endl; 
    return;
  }
  LtMap  mbadch;
  //  cout<<" analyse badch before  init  "<<endl;
  //    f->print();
  //    f->init();
  // wait();
  //  cout<<" f init "<<endl;
  for(int im = 0; im < NModeType; im++) {
    mbadch.clear();
    
    for(int ich = 0; ich < thetestanalyzer->getNchan(); ich++) {
      //1. Unknown (?)
      if(findUnknown(im,ich))      mbadch[ich] = f->chfunknown;
      //2. Noisy (NOIS)
      if(findNoisy(im,ich))        mbadch[ich] = f->chfnoisy;
      //3. likely pinhole (PHL-) 
      if(findLikePinhole(im,ich))  mbadch[ich] = f->chflikepin; 
      //      cout <<f->chflikepin<<endl; wait();
      //4. likely one sensor unbounded (OSO-)
      if(findLikeOS(im,ich))       mbadch[ich] = f->chflikeunbound2;
      //5. One sensor unbounded (OSO+)
      if(findOS(im,ich))           mbadch[ich] = f->chfunbound2;
      //6. likely two sensor unbounded (TSO-)
      if(findLikeTS(im,ich))       mbadch[ich] = f->chflikeunbound1;
      //7. Two sensor unbounded (TSO+)
      if(findTS(im,ich))           mbadch[ich] = f->chfunbound1;
      //8. Like mid open (MSO-)
      if(findLikeMidOpen(im,ich))  mbadch[ich] = f->chflikeopen;
      //9. like open (MSO-)
      //      if(findLikeOpen(im,ich))     mbadch[ich] = f->chflikeopen;  removed because is like OSO  and TSO
      //10. like or short
      if(findShort(im,ich)==2)     mbadch[ich] = f->chflikeshort;
      if(findShort(im,ich)==1)     mbadch[ich] = f->chfshort;
      
      //12. Pinhole test positive
      if(findPinhole(im,ich)==1)  mbadch[ich] = f->chflikepin;
      if(findPinhole(im,ich)==2)  mbadch[ich] = f->chfunknown;
      if(findPinhole(im,ich)==3)  mbadch[ich] = f->chfpin;
    } // loop on channels
    mbadchannels[ModeType[im]] = mbadch;
  } // loop on apvmodes
}
//
void DefAnalyzer::AnalyzeSensor() 
{
  cout<<" analyse sensor "<<endl; 
  RootReader* reader=thetestanalyzer->getRootReader();
  AnalSetParser* setparser=thetestanalyzer->getAnalSetParser(); 
  IVDescriptor* sets=setparser->getIVDescriptor(thetestanalyzer->getModType());
  
}
//
void DefAnalyzer::AnalyzeErrors() 
{
  cout<<" analyse errors "<<endl; 
  RootReader* reader=thetestanalyzer->getRootReader();
  AnalSetParser* setparser=thetestanalyzer->getAnalSetParser(); 
  //DaqErrorsDescriptor* sets=setparser->getDaqErrorsDescriptor(thetestanalyzer->getModType());
  
  
}
// ---------------------------------------------------------------------------
void DefAnalyzer::saveBadChMap() 
{
  LtMap badchtot;
  RootReader* reader = thetestanalyzer->getRootReader();
  saveBadChPed();
  saveBadChCal();
  saveBadChPin();
  RemoveNoisyChannels();
  reader->SetRecord();
  
  // -------------------
  TString softname = reader->getSoftware();
  TString vname = "TIbadchlist";
  LtApvMap m = mbadchannels;
  for(LtApvMap::iterator itapv = m.begin(); itapv != m.end(); itapv++) {
    cout<<"Status for Mode "<<(*itapv).first<<endl;
    for(LtMap::iterator it = ((*itapv).second).begin(); 
	it != ((*itapv).second).end(); it++) {
      DT_DEBUG_RANA("Chanel "<<(*it).first<<" is flagged as "<<(*it).second);
    }
  }
  //   cout<<endl; wait();
  // ---------------
  for(int im = 0; im < NModeType; im++) {
    reader->Write("Summary",vname,&totv(mbadchannels[ModeType[im]]),1,1);
    masktot(badchtot,mbadchannels[ModeType[im]]);
  }
  vector<int> vbadchlist; 
  vbadchlist.clear();
  for(map<int , int, less<int> >::const_iterator i=badchtot.begin();i!=badchtot.end();i++)
    vbadchlist.push_back(i->first);
  reader->SetRecord();
  reader->Write("Summary","TIbadch",&totv(vbadchlist),1,1);
  reader->Write("Summary","TIbadchlist",&totv(badchtot),1,1);
  
  Nbadch = badchtot.size();
  reader->Write("Summary","TInbadch",&totv(Nbadch),1,1);
}
// -------------------------------------------------------------------------
void DefAnalyzer::saveBadChPed() 
{
  LtMap mbadch;
  LtApvMap  mbadchnoise    = thetestanalyzer->getMapNoise();
  LtApvMap  mbadchpedestal = thetestanalyzer->getMapPedestal();
  
  RootReader* reader = thetestanalyzer->getRootReader();
  
  // badchped should not contains pedeatal info (Test Module meeting, june 04)
  //  for(LtApvMap::iterator itapv = mbadchpedestal.begin(); itapv != mbadchpedestal.end(); itapv++) {
  //for(LtMap::iterator it = ((*itapv).second).begin(); 
  //it != ((*itapv).second).end(); it++) {
  //  mbadch[(*it).first] |= (*it).second;
  //}
  //}
  for(LtApvMap::iterator itapv = mbadchnoise.begin(); itapv != mbadchnoise.end(); itapv++) {
    for(LtMap::iterator it = ((*itapv).second).begin(); 
	it != ((*itapv).second).end(); it++) {
      mbadch[(*it).first] |= (*it).second;
    }
  }
  reader->SetRecord();
  TString softname = reader->getSoftware();
  TString vname;
  if(softname.Contains("arc")) vname = "TIbadchpedestal"; // ARC
  else if(softname.Contains("lt")) vname = "TIbadchped"; //LT
  reader->Write("Summary",vname,&totv(mbadch),1,1);
}
// ----------------------------------------------------------------------
void DefAnalyzer::saveBadChCal() 
{
  LtMap mbadch;
  LtApvMap  mbadchpeaktime    = thetestanalyzer->getMapPeakTime();
  LtApvMap  mbadchpulseheight = thetestanalyzer->getMapPulseHeight();
  
  RootReader* reader = thetestanalyzer->getRootReader();
  
  for(LtApvMap::iterator itapv = mbadchpeaktime.begin(); itapv != mbadchpeaktime.end(); itapv++) {
    for(LtMap::iterator it = ((*itapv).second).begin(); 
	it != ((*itapv).second).end(); it++) {
      mbadch[(*it).first] |= (*it).second;
      cout<<"mbadchpeaktime["<<(*it).first<<"] = "<<(*it).second<<endl;
    }
  } //wait();
  for(LtApvMap::iterator itapv = mbadchpulseheight.begin(); itapv != mbadchpulseheight.end(); itapv++) {
    for(LtMap::iterator it = ((*itapv).second).begin(); 
	it != ((*itapv).second).end(); it++) {
      mbadch[(*it).first] |= (*it).second;
      cout<<"mbadchpulseheight["<<(*it).first<<"] = "<<(*it).second<<endl;
    }
  }
  //  wait();
  reader->SetRecord();
  reader->Write("Summary","TIbadchcalprof",&totv(mbadch),1,1);
}
// ----------------------------------------------------------------------
void DefAnalyzer::saveBadChPin() 
{
  LtMap mbadch;
  LtApvMap  mbadchpinhole     = thetestanalyzer->getMapPinhole();
  
  RootReader* reader = thetestanalyzer->getRootReader();
  
  for(LtApvMap::iterator itapv = mbadchpinhole.begin(); itapv != mbadchpinhole.end(); itapv++) {
    cout<<"Pinhole Status for Mode "<<(*itapv).first<<endl;
    for(LtMap::iterator it = ((*itapv).second).begin(); 
	it != ((*itapv).second).end(); it++) {
      mbadch[(*it).first] |= (*it).second;
      //  cout<<"Chanel "<<(*it).first<<" is flagged as "<<(*it).second<<endl;
    }
  }
  reader->SetRecord();
  reader->Write("Summary","TIbadchpinhole",&totv(mbadch),1,1);
}
// ----------------------------------------------------------------------
int DefAnalyzer::findUnknown(int im, int ichan)
{
  //  cout<<" findUknown "<<endl;
  LtApvMap  mbadchnoise=thetestanalyzer->getMapNoise();
  LtApvMap  mbadchpeaktime=thetestanalyzer->getMapPeakTime();
  LtApvMap  mbadchpulseheight=thetestanalyzer->getMapPulseHeight();
  //  cout<<" size "<<mbadchnoise.size()<<endl;
  //  if(mbadchnoise.size()==0) return 0;
  // if(mbadchpeaktime.size()==0) return 0;
  //if(mbadchpulseheight.size()==0) return 0;
  if(  mbadchnoise[ModeType[im]][ichan]!=0
       || mbadchpeaktime[ModeType[im]][ichan]!=0
       || mbadchpulseheight[ModeType[im]][ichan]!=0 ) {
    return 1;
  }
  //  cout<<" return 0"<<endl;
  return  0;
}
// ----------------------------------------------------------------------
int DefAnalyzer::findNoisy(int im, int ichan)
{
  NoiseDescriptor f;       f.init();
  LtApvMap  mbadchnoise=thetestanalyzer->getMapNoise();
  LtApvMap  mbadchpeaktime=thetestanalyzer->getMapPeakTime();
  LtApvMap  mbadchpulseheight=thetestanalyzer->getMapPulseHeight();
  LtApvMap  mbadchpinhole=thetestanalyzer->getMapPinhole();
  //  cout<<" size "<<mbadchnoise.size()<<endl;
  if(mbadchnoise.size()==0) return 0;
  if(  mbadchnoise[ModeType[im]][ichan] == f.flagNoise_High
       && mbadchpeaktime[ModeType[im]][ichan] == 0
       && mbadchpulseheight[ModeType[im]][ichan] == 0
       && mbadchpinhole[ModeType[im]][ichan] == 0) {
    return 1;
  }

  return 0;
}
// ----------------------------------------------------------------------
int DefAnalyzer::findLikePinhole(int im, int ichan)
{
  NoiseDescriptor f1;       f1.init();
  PeakTimeDescriptor f2;    f2.init();
  PulseHeightDescriptor f3; f3.init();
  
  LtApvMap  mbadchnoise=thetestanalyzer->getMapNoise();
  LtApvMap  mbadchpeaktime=thetestanalyzer->getMapPeakTime();
  LtApvMap  mbadchpulseheight=thetestanalyzer->getMapPulseHeight();
  if(mbadchnoise.size()==0) return 0;
  if(mbadchpeaktime.size()==0) return 0;
  if(mbadchpulseheight.size()==0) return 0;
  bool bplow = (mbadchnoise[ModeType[im]][ichan]==f1.flagNoise_Low) 
    &&(mbadchpeaktime[ModeType[im]][ichan]==f2.flagPeakTime_Low);
  bool bpp = (mbadchpulseheight[ModeType[im]][ichan]==f3.flagPulseHeight_Min);
  if(bplow&&bpp)   {
    cout<<mbadchnoise[ModeType[im]][ichan]<<endl;
    cout<<mbadchpeaktime[ModeType[im]][ichan]<<endl;
    cout<<mbadchpulseheight[ModeType[im]][ichan]<<endl;
    cout<<"flagged PHL-"<<endl;// wait();    
    return 1;
  } 
  return 0;
}
// ----------------------------------------------------------------------
int DefAnalyzer::findLikeOS(int im, int ichan)
{
  NoiseDescriptor f1;       f1.init();
  PeakTimeDescriptor f2;    f2.init();
  PulseHeightDescriptor f3; f3.init();
  
  LtApvMap  mbadchnoise=thetestanalyzer->getMapNoise();
  LtApvMap  mbadchpeaktime=thetestanalyzer->getMapPeakTime();
  LtApvMap  mbadchpulseheight=thetestanalyzer->getMapPulseHeight();
  //  if(mbadchnoise.size()==0) return 0;
  // if(mbadchpeaktime.size()==0) return 0;
  // if(mbadchpulseheight.size()==0) return 0;
  bool bposul = (mbadchnoise[ModeType[im]][ichan]==f1.flagNoise_OSUnbound)
    ||(mbadchpeaktime[ModeType[im]][ichan]==f2.flagPeakTime_OSUnbound);
  bool bpph = (mbadchpulseheight[ModeType[im]][ichan]!= f3.flagPulseHeight_Min);
  if(bposul&&bpph) {
    cout<<mbadchnoise[ModeType[im]][ichan]<<endl;
    cout<<mbadchpeaktime[ModeType[im]][ichan]<<endl;
    cout<<mbadchpulseheight[ModeType[im]][ichan]<<endl;
    cout<<ichan<<" et "<<ModeType[im]<<" est flagged OS-"<<endl;// wait();    
    return 1; 
  }	
  return 0;
}
// ----------------------------------------------------------------------
int DefAnalyzer::findOS(int im, int ichan)
{
  NoiseDescriptor f1;       f1.init();
  PeakTimeDescriptor f2;    f2.init();
  PulseHeightDescriptor f3; f3.init();

  LtApvMap  mbadchnoise=thetestanalyzer->getMapNoise();
  LtApvMap  mbadchpeaktime=thetestanalyzer->getMapPeakTime();
  LtApvMap  mbadchpulseheight=thetestanalyzer->getMapPulseHeight();
  //  if(mbadchnoise.size()==0) return 0;
  //if(mbadchpeaktime.size()==0) return 0;
  //if(mbadchpulseheight.size()==0) return 0;
  bool bposu = (mbadchnoise[ModeType[im]][ichan]==f1.flagNoise_OSUnbound) 
    && (mbadchpeaktime[ModeType[im]][ichan]==f2.flagPeakTime_OSUnbound);
  bool bpph = (mbadchpulseheight[ModeType[im]][ichan]!=f3.flagPulseHeight_Min);
  if(bposu&&bpph)    {
    cout<<mbadchnoise[ModeType[im]][ichan]<<endl;
    cout<<mbadchpeaktime[ModeType[im]][ichan]<<endl;
    cout<<mbadchpulseheight[ModeType[im]][ichan]<<endl;
    cout<<ichan<<" et "<<ModeType[im]<<" est flagged OS+"<<endl;// wait();    
    return 1;}
 	
  return 0;
}
// ----------------------------------------------------------------------
int DefAnalyzer::findLikeTS(int im, int ichan)
{
  NoiseDescriptor f1;       f1.init();
  PeakTimeDescriptor f2;    f2.init();
  PulseHeightDescriptor f3; f3.init();

  LtApvMap  mbadchnoise=thetestanalyzer->getMapNoise();
  LtApvMap  mbadchpeaktime=thetestanalyzer->getMapPeakTime();
  LtApvMap  mbadchpulseheight=thetestanalyzer->getMapPulseHeight();
  //  if(mbadchnoise.size()==0) return 0;
  //if(mbadchpeaktime.size()==0) return 0;
  //if(mbadchpulseheight.size()==0) return 0;
  bool bposul2 = (mbadchnoise[ModeType[im]][ichan] == f1.flagNoise_TSUnbound)
    || (mbadchpeaktime[ModeType[im]][ichan]==f2.flagPeakTime_TSUnbound);
  bool bpph = bpph = (mbadchpulseheight[ModeType[im]][ichan]!=f3.flagPulseHeight_Min);
  if(bposul2&&bpph) {
    cout<<mbadchnoise[ModeType[im]][ichan]<<endl;
    cout<<mbadchpeaktime[ModeType[im]][ichan]<<endl;
    cout<<mbadchpulseheight[ModeType[im]][ichan]<<endl;
    cout<<"flagged TS-"<<endl;// wait();    
    return 1;
  }	
  return 0;
}
// ----------------------------------------------------------------------
int DefAnalyzer::findTS(int im, int ichan)
{
  NoiseDescriptor f1;       f1.init();
  PeakTimeDescriptor f2;    f2.init();
  PulseHeightDescriptor f3; f3.init();
  
  LtApvMap  mbadchnoise=thetestanalyzer->getMapNoise();
  LtApvMap  mbadchpeaktime=thetestanalyzer->getMapPeakTime();
  LtApvMap  mbadchpulseheight=thetestanalyzer->getMapPulseHeight();
  //  if(mbadchnoise.size()==0) return 0;
  //if(mbadchpeaktime.size()==0) return 0;
  //if(mbadchpulseheight.size()==0) return 0;
  bool bpph = (mbadchpulseheight[ModeType[im]][ichan]!=f3.flagPulseHeight_Min);
  bool bposu2 = (mbadchnoise[ModeType[im]][ichan]==f1.flagNoise_TSUnbound)
    && (mbadchpeaktime[ModeType[im]][ichan]==f2.flagPeakTime_TSUnbound);
  if(bposu2&&bpph) {
    return 1;
  }	
  return 0;
}
// ----------------------------------------------------------------------
int DefAnalyzer::findLikeMidOpen(int im, int ichan)
{
  NoiseDescriptor f1;       f1.init();
  PeakTimeDescriptor f2;    f2.init();
  PulseHeightDescriptor f3; f3.init();

  LtApvMap  mbadchnoise=thetestanalyzer->getMapNoise();
  LtApvMap  mbadchpeaktime=thetestanalyzer->getMapPeakTime();
  LtApvMap  mbadchpulseheight=thetestanalyzer->getMapPulseHeight();
  //    if(mbadchnoise.size()==0) return 0;
  //if(mbadchpeaktime.size()==0) return 0;
  //if(mbadchpulseheight.size()==0) return 0;
  bool bpph = (mbadchpulseheight[ModeType[im]][ichan]!=f3.flagPulseHeight_Min);
  bool bpnh = (mbadchnoise[ModeType[im]][ichan]==f1.flagNoise_High);
  bool bppt = ((mbadchpeaktime[ModeType[im]][ichan]==f2.flagPeakTime_OSUnbound)
	       || (mbadchpeaktime[ModeType[im]][ichan]==f2.flagPeakTime_TSUnbound)
	       || (mbadchpeaktime[ModeType[im]][ichan]==f2.flagPeakTime_Low));
  
  if(bpph&&bpnh&&bppt)    {
    cout<<mbadchnoise[ModeType[im]][ichan]<<endl;
    cout<<mbadchpeaktime[ModeType[im]][ichan]<<endl;
    cout<<mbadchpulseheight[ModeType[im]][ichan]<<endl;
    cout<<ichan<<" et "<<ModeType[im]<<" est flagged MSO- (1)"<<endl;// wait();    
    return 1;
  }
  return 0;
}
// ----------------------------------------------------------------------
int DefAnalyzer::findLikeOpen(int im, int ichan)
{
  NoiseDescriptor f1;       f1.init();
  PeakTimeDescriptor f2;    f2.init();
  PulseHeightDescriptor f3; f3.init();

  LtApvMap  mbadchnoise=thetestanalyzer->getMapNoise();
  LtApvMap  mbadchpeaktime=thetestanalyzer->getMapPeakTime();
  LtApvMap  mbadchpulseheight=thetestanalyzer->getMapPulseHeight();
  //  if(mbadchnoise.size()==0) return 0;
  //if(mbadchpeaktime.size()==0) return 0;
  //if(mbadchpulseheight.size()==0) return 0;
  bool bpph = bpph = (mbadchpulseheight[ModeType[im]][ichan]!=f3.flagPulseHeight_Min);
  bool bpnp1 = ((mbadchpeaktime[ModeType[im]][ichan]==f2.flagPeakTime_TSUnbound)
		&& (mbadchnoise[ModeType[im]][ichan]==f1.flagNoise_TSUnbound)) ;
  bool bpnp2 = ((mbadchpeaktime[ModeType[im]][ichan]==f2.flagPeakTime_OSUnbound)
		&& (mbadchnoise[ModeType[im]][ichan]==f1.flagNoise_OSUnbound));
  if((bpnp1 || bpnp2) && bpph) {
    cout<<mbadchnoise[ModeType[im]][ichan]<<endl;
    cout<<mbadchpeaktime[ModeType[im]][ichan]<<endl;
    cout<<mbadchpulseheight[ModeType[im]][ichan]<<endl;
    cout<<"flagged MSO- (2)"<<endl; //wait();    
    //    return 1;
  }
  return 0;
}
// ----------------------------------------------------------------------
int DefAnalyzer::findShort(int im, int ichan)
{
  NoiseDescriptor f1;       f1.init();
  PulseHeightDescriptor f3; f3.init();

  LtApvMap  mbadchnoise=thetestanalyzer->getMapNoise();
  LtApvMap  mbadchpulseheight=thetestanalyzer->getMapPulseHeight();
  //  if(mbadchnoise.size()==0) return 0;
  //if(mbadchpulseheight.size()==0) return 0;
  int flag =0;  // return 1 for short and 2 for likely short
  if(mbadchpulseheight[ModeType[im]][ichan] == f3.flagPulseHeight_Min) { 
    // +1	   
    if(mbadchpulseheight[ModeType[im]][ichan+1]==f3.flagPulseHeight_Min)  { 
      flag = 2;
      if(mbadchnoise[ModeType[im]][ichan]!=0 
	 || mbadchnoise[ModeType[im]][ichan+1]!=0 )
	flag = 1;
    }  
    // +2
    if(mbadchpulseheight[ModeType[im]][ichan+2]==f3.flagPulseHeight_Min) { 
      flag = 2;      
      if(mbadchnoise[ModeType[im]][ichan]!=0 
	 || mbadchnoise[ModeType[im]][ichan+2]!=0 ) 
	flag = 1;
    }
    // -1
    if(mbadchpulseheight[ModeType[im]][ichan-1]==f3.flagPulseHeight_Min) { 
      flag = 2;      
      if(mbadchnoise[ModeType[im]][ichan]!=0 
	 || mbadchnoise[ModeType[im]][ichan-1]!=0 )
	flag = 1;
    }	
    // -2
    if(mbadchpulseheight[ModeType[im]][ichan-2]==f3.flagPulseHeight_Min) { 
      flag = 2;
      if(mbadchnoise[ModeType[im]][ichan]!=0 
	 || mbadchnoise[ModeType[im]][ichan-2]!=0 ) 
	flag = 1;
    }	
  } // end of short  
  return flag;
}
// ----------------------------------------------------------------------
int DefAnalyzer::findPinhole(int im, int ichan)
{
  NoiseDescriptor f1;    f1.init();
  PeakTimeDescriptor f2; f2.init();
  PinholeDescriptor f3;  f3.init();
  
  LtApvMap  mbadchpinhole=thetestanalyzer->getMapPinhole();
  LtApvMap  mbadchnoise=thetestanalyzer->getMapNoise();
  LtApvMap  mbadchpeaktime=thetestanalyzer->getMapPeakTime();
  //    if(mbadchnoise.size()==0) return 0;
  //if(mbadchpeaktime.size()==0) return 0;
  //if(mbadchpinhole.size()==0) return 0; 
  int flag = 0;// 1-> likepin ; 2->unknown ; 3->pinhole
  bool bplph  = false;
  if(mbadchpinhole[apvmodedirname(PEAKINVOFF)][ichan]==f3.flagPinhole_Max2) {
    bplph = true;
    flag  = 1;
  }  
  bool bpnh = (mbadchnoise[ModeType[im]][ichan]==f1.flagNoise_High) ;
  if(bplph && bpnh){
    flag = 2;
  }
  bool bplow = (mbadchnoise[ModeType[im]][ichan]==f1.flagNoise_Low) 
    &&(mbadchpeaktime[ModeType[im]][ichan]==f2.flagPeakTime_Low);
  if(bplph && bplow)   flag =3;

  return flag;
}
// ----------------------------------------------------------------------
int DefAnalyzer::RemoveNoisyChannels()
{
  ModFlagsDescriptor* f = thetestanalyzer->getAnalSetParser()->getModFlagsDescriptor(thetestanalyzer->getModType());
  f->init();
  int  nChip = thetestanalyzer->getNChip();
  for(LtApvMap::iterator itapv = mbadchannels.begin(); 
      itapv != mbadchannels.end(); itapv++) 
    for(LtMap::iterator it = ((*itapv).second).begin(); 
	it != ((*itapv).second).end(); it++)
      if( ( (*it).first%NAPVCH==0               // edge of the chip
	    || (*it).first%NAPVCH==(NAPVCH-1)   // idem
	    || (*it).first==1                   // edge of the sensor
	    || (*it).first==(nChip*NAPVCH-2)   )// idem
	  &&  (*it).second == f->chfnoisy ){
	((*itapv).second).erase(it);
      }
}
// ----------------------------------------------------------------------
int DefAnalyzer::GradeModule()
{
  int ichan = thetestanalyzer->getNchan();
  RootReader* reader=thetestanalyzer->getRootReader();
  float isens = thetestanalyzer->getIsens();    // TEMPORARY should be taken from root file if available
  float ileak = thetestanalyzer->getIleak();
  int NSens = 1;         // TEMPORARY should be taken from rootfile
  int grade = 0;                                            // Grade A  
  if(Nbadch < 0.01*ichan && ileak > 5.*isens)   grade = 2;  // grade A_F
  if(Nbadch > 0.01*ichan && Nbadch < 0.02*ichan){
    grade = 4;                                              // grade B
    if(ileak > 5.*isens) grade = 8;                         // grade B_F
  }
  // grade C
  if(Nbadch > 0.02*ichan)                    grade = -1; 
  if( ileak>NSens*10.)                       grade = -2;
  if(Nbadch > 0.02*ichan && ileak>NSens*10.) grade = -4; 
  reader->SetRecord();
  reader->Write("Summary","TIMODULBASIC_val",&totv(grade),1,1);
  return 1;
}
// ----------------------------------------------------------------------
int DefAnalyzer::GradeLTModule()
{
  RootReader* reader=thetestanalyzer->getRootReader();
  const  int testnr = 3;
  int itest=0;
  char* tests[testnr] = {"MODULLTFIRST", "MODULLTCOLD", "MODULLTLAST"};
  vector<LtMap> mltbadch; mltbadch.clear();
  LtMap  mltbadchtot;
  vector<int> vltapv; vltapv.clear();
  int  iltapvtot = 0;
  vector<int> vltpll; vltpll.clear();
  int  iltplltot = 0;
  vector<int> vltmux; vltmux.clear();
  int  iltmuxtot = 0;
  vector<int> vltdcu; vltdcu.clear();
  int  iltdcutot = 0;
  vector<int> vltsens; vltsens.clear();
  int  iltsenstot = 0;
  vector<int> vlthyb; vlthyb.clear();
  int  ilthybtot = 0;
  vector<int> vltmodv; vltmodv.clear();
  int  iltmodvtot = 0;
  int nbadchtot = 0;
  // --------- find for objects
  for(itest =0; itest< testnr; itest++) {
    reader->SetRecord(reader->GetRecordNr(tests[itest]));
    if(reader->Find("TIbadchlist"))
      mltbadch.push_back(tvtomap((TVector*) reader->Find("TIbadchlist")));
    if(reader->Find("TIstatApv"))
      vltapv.push_back(tvtoint((TVector*) reader->Find("TIstatApv")));
    if(reader->Find("TIstatPll"))
      vltpll.push_back(tvtoint((TVector*) reader->Find("TIstatPll")));
    if(reader->Find("TIstatMux"))
      vltmux.push_back(tvtoint((TVector*) reader->Find("TIstatMux")));
    if(reader->Find("TIstatDcu"))
      vltdcu.push_back(tvtoint((TVector*) reader->Find("TIstatDcu")));
    if(reader->Find("TIstatSensor"))
      vltsens.push_back(tvtoint((TVector*) reader->Find("TIstatSensor")));
    if(reader->Find("TIstatHybrid"))
      vlthyb.push_back(tvtoint((TVector*) reader->Find("TIstatHybrid")));
    if(reader->Find("TIMODULBASIC_val"))
      vltmodv.push_back(tvtoint((TVector*) reader->Find("TIMODULBASIC_val")));
  }
  
  // ------- do OR
  for(itest =0; itest< mltbadch.size(); itest++) 
    masktot(mltbadchtot,mltbadch[itest]);
  //  wait();
  for (map<int, int, less<int> >::const_iterator i=mltbadchtot.begin();
       i !=mltbadchtot.end(); i++) {
    int ch=i->first;
    int flag=i->second;
    //    cout<<"et channel = "<<ch<<" et flag = "<<flag<<endl;
  } //wait();
  for(itest =0; itest < vltapv.size(); itest++)  iltapvtot  |= vltapv[itest];
  for(itest =0; itest < vltpll.size(); itest++)  iltplltot  |= vltpll[itest];
  for(itest =0; itest < vltmux.size(); itest++)  iltmuxtot  |= vltmux[itest];
  for(itest =0; itest < vltdcu.size(); itest++)  iltdcutot  |= vltdcu[itest];
  for(itest =0; itest < vltsens.size(); itest++) iltsenstot |= vltsens[itest];
  for(itest =0; itest < vlthyb.size(); itest++)  ilthybtot  |= vlthyb[itest];
  for(itest =0; itest < vltmodv.size(); itest++) iltmodvtot |= vltmodv[itest];
  
  nbadchtot = mltbadchtot.size();
  // -------- write
  reader->cd();
  reader->Write("Summary","TIbadch",&totv(mltbadchtot),1,1);
  reader->Write("Summary","TInbadch",&totv(nbadchtot),1,1);
  reader->Write("Summary","TIstatApv",&totv(iltapvtot),1,1);
  reader->Write("Summary","TIstatPll",&totv(iltplltot),1,1);
  reader->Write("Summary","TIstatMux",&totv(iltmuxtot),1,1);
  reader->Write("Summary","TIstatSensor",&totv(iltsenstot),1,1);
  reader->Write("Summary","TIstatHybrid",&totv(ilthybtot),1,1);
  reader->Write("Summary","TIMODULLTSUMMARY_val",&totv(iltmodvtot),1,1);
  // 
  //  wait();
  return 1;
}
