#include  "ModAnalyzer.h"
#include  "AnalDefs.h"
#include  "AnalDescriptor.h"
#include  "AnalSetParser.h"
#include  "TextSummarizer.h"
#include  "DefAnalyzer.h"
#include  "RecUtil.h"
#include  "TestAnalyzer.h"
#include  "DefAnaHistogrammer.h"
#include <vector>
#include <map>

#include <TCanvas.h>
#include <TGraph.h>
#include <TProfile.h>
#include <TH1.h>
#include <TVector.h>
#include <TFile.h>


//#include "DbParser.h"

using namespace std;
using namespace RecUtil;

ModAnalyzer::ModAnalyzer(AnalSetParser* sets, RootReader* reader) 
{
  thesettings=sets;
  thereader=reader; 
}

ModAnalyzer::~ModAnalyzer() {
}

// ----------------------------------------------------------------------------
const char* ModAnalyzer::getModeType()
{
  const char* modtype;
  TNamed* n=(TNamed*) thereader->Find("TNmodtype");
   cout<<"[ModAnalyzer::GetModeType()] n = "<<n<<endl;
  if(n==NULL) modtype = "ghost";
  else if(strcmp(n->GetTitle(),"")==0)  modtype = "dummy";
  else modtype=tntochar(n);
  cout <<"[ModAnalyzer::GetModeType] modetype is "<<modtype<<endl;
  return modtype;
}
// ---------------------------------------------------------------------------
  LtApvMap ModAnalyzer::getBadChannelMap()
{ 
return mbadchannels;
}
// ---------------------------------------------------------------------------
void ModAnalyzer::Analyze() {
  // identify moduletype
  //  TNamed* n=(TNamed*)thereader->Find("TNmodtype");
  //  const char* modtype=tntochar(n);
  // analyze tests
  TestAnalyzer* testanalyzer=new TestAnalyzer(thesettings,thereader);
  testanalyzer->AnalyzeCommonMode();
  testanalyzer->AnalyzePinhole();
  testanalyzer->AnalyzeGain();
  testanalyzer->AnalyzePedestal();
  testanalyzer->AnalyzeNoise();
  testanalyzer->AnalyzePulseShape();
  testanalyzer->AnalyzePulseHeight();
  testanalyzer->AnalyzePeakTime();
  testanalyzer->AnalyzePipeline();
  testanalyzer->AnalyzeIV();
  // or AnalyzeAll();
  DefAnalyzer* defanalyzer=new DefAnalyzer(testanalyzer); 
  defanalyzer->AnalyzeBadCh();
  // or AnalyzeBadCh()
  defanalyzer->AnalyzeSensor();
  defanalyzer->AnalyzeErrors();
  defanalyzer->saveBadChMap();
  defanalyzer->GradeModule();

 
}
//
void ModAnalyzer::AnalyzeAndPrint() {
  // identify moduletype
  //  TNamed* n=(TNamed*)thereader->Find("TNmodtype");
  //  const char* modtype=tntochar(n);
  // analyze tests
  TestAnalyzer* testanalyzer=new TestAnalyzer(thesettings,thereader);
  testanalyzer->AnalyzeCommonMode();
  testanalyzer->AnalyzePinhole();
  testanalyzer->AnalyzeGain();
  testanalyzer->AnalyzePedestal();
  testanalyzer->AnalyzeNoise();
  testanalyzer->AnalyzePulseShape();
  testanalyzer->AnalyzePulseHeight();
  testanalyzer->AnalyzePeakTime();
  testanalyzer->AnalyzePipeline();
  testanalyzer->AnalyzeIV();
  // or AnalyzeAll();
  DefAnalyzer* defanalyzer=new DefAnalyzer(testanalyzer); 
  defanalyzer->AnalyzeBadCh();
  // or AnalyzeBadCh()
  defanalyzer->AnalyzeSensor();
  defanalyzer->AnalyzeErrors();
  defanalyzer->saveBadChMap();
  defanalyzer->GradeModule();

  // write ascci summary
  TextSummarizer *fout = new TextSummarizer(defanalyzer, this);
  fout->PrintSummary();
  delete fout;

  // write postscript summary
  DefAnaHistogrammer *hout = new DefAnaHistogrammer(defanalyzer, this);
  hout->PrintHistogramms();
  delete hout;
}
//
