#include "ModFlagsDescriptor.h"

#include <parsers/DOMParser.hpp>
#include <dom/DOM_NamedNodeMap.hpp>
#include <dom/DOM.hpp>
#include "DOMTreeErrorReporter.hpp"
#include <cstdio>
#include <TString.h>


using namespace std;

ModFlagsDescriptor::ModFlagsDescriptor(): 
AnalDescriptor() {
  init();
}

//
void ModFlagsDescriptor::init() {
  cout<<" init modflag "<<endl;
    Name = "ModFlagsDescriptor";  
    /* OFFICIAL DATABASE CONVENTIONS
       http://www-ekp.physik.uni-karlsruhe.de/~zhukov/Cms/database.html
       <!--  2= open pa-sensor1, 4= likely open pa-sensor1 , 8= open sensor-sensor, 16= likely open sensor-sensor, 32= open uknwn,
       64 noisy,  128= short, 256= lowleakpinhole ,512= high leak pinhole, 1024= unknown. (lt: all tags are defined in the settings file)  -->  */
    chfunbound1     =    2; 
    chflikeunbound1 =    4; 
    chfunbound2     =    8; 
    chflikeunbound2 =   16; 
    chfpin          =  512; 
    chflikepin      =  256; 
    chflikemidopen  = 2048; // not in the database 
    chfnoisy        =   64; 
    chfshort        =  128; 
    chflikeshort    = 4096; // not in the database 
    chflikeopen     =   32;
    chfunknown      = 1024;
    cout<<" init done "<<endl;
}

void ModFlagsDescriptor::initmap() { 
    // init character map 
    cflagmap[chfunbound1]="OSO+";
    cflagmap[chflikeunbound1]="OSO-";
    cflagmap[chfunbound2 ]="TSO+";
    cflagmap[chflikeunbound2]="TSO-";
    cflagmap[chfpin]="PHL+";
    cflagmap[chflikepin]="PHL-";
    cflagmap[chflikemidopen]="MOS-";
    cflagmap[chfnoisy]="NOI";
    cflagmap[chfshort]="SHT";
    cflagmap[chflikeshort]="SHT-";
    cflagmap[chflikeopen]="OPN?";
    cflagmap[chfunknown]="UNK";
  }
//
 void  ModFlagsDescriptor::print() {
cout <<"[ModFlagsDescriptor::print] ---------------------------------" <<std::endl;
cout <<"    chfunbound1      = "<<chfunbound1   <<endl;   
cout <<"    chflikeunbound1  = "<<chflikeunbound1   <<endl;
cout <<"    chfunbound2      = "<<chfunbound2  <<endl;
cout <<"   chflikeunbound2  = "<<chflikeunbound2   <<endl;
cout <<"    chfpin           = "<< chfpin   <<endl;
cout <<"   chflikepin       = "<< chflikepin   <<endl;
cout <<"    chflikemidopen   = "<< chflikemidopen  <<endl;
cout <<"    chfnoisy         = "<< chfnoisy   <<endl;
cout <<"    chfshort         = "<< chfshort   <<endl; 
cout <<"    chflikeshort     = "<< chflikeshort   <<endl;
cout <<"    chflikeopen      = "<< chflikeopen   <<endl;
cout <<"    chfunknown       = "<<  chfunknown   <<endl;
cout<<"[ModFlagsDescriptor::print] ------------------------------- end"<<std::endl;
  }
//
int  ModFlagsDescriptor::ParseSettings(DOM_Node &ei) {
 DT_NOTE(" ModFlags "); 
 
DOM_NamedNodeMap attributes = ei.getAttributes();

DOM_Node e = attributes.getNamedItem("chfunbound1");
 if(! e.isNull() ) { chfunbound1=atoi(e.getNodeValue().transcode());  }
e = attributes.getNamedItem("chflikeunbound1");
 if(! e.isNull() ) { chflikeunbound1 =atoi(e.getNodeValue().transcode());  }
e = attributes.getNamedItem("chfunbound2");
 if(! e.isNull() ) { chfunbound2=atoi(e.getNodeValue().transcode());  }
e = attributes.getNamedItem("chflikeunbound2");
 if(! e.isNull() ) { chflikeunbound2=atoi(e.getNodeValue().transcode());  }
e = attributes.getNamedItem("chfpin");
 if(! e.isNull() ) { chfpin =atoi(e.getNodeValue().transcode());  }
e = attributes.getNamedItem("chflikepin");
 if(! e.isNull() ) { chflikepin=atoi(e.getNodeValue().transcode());  }
e = attributes.getNamedItem("chflikemidopen");
 if(! e.isNull() ) { chflikemidopen=atoi(e.getNodeValue().transcode());  }
e = attributes.getNamedItem("chflikemidopen");
 if(! e.isNull() ) { chflikemidopen=atoi(e.getNodeValue().transcode());  }
e = attributes.getNamedItem("chflikemidopen");
 if(! e.isNull() ) { chflikemidopen=atoi(e.getNodeValue().transcode());  }
e = attributes.getNamedItem("chflikemidopen");
 if(! e.isNull() ) { chflikemidopen=atoi(e.getNodeValue().transcode());  }
e = attributes.getNamedItem("chflikemidopen");
 if(! e.isNull() ) { chflikemidopen=atoi(e.getNodeValue().transcode());  }
e = attributes.getNamedItem("chflikemidopen");
 if(! e.isNull() ) { chflikemidopen=atoi(e.getNodeValue().transcode());  }
e = attributes.getNamedItem("chflikemidopen");
 if(! e.isNull() ) { chflikemidopen=atoi(e.getNodeValue().transcode());  }


initmap();
  return 0;
}

TString  ModFlagsDescriptor:: getcflag(int i) {
  if(cflagmap.find(i)!=cflagmap.end())
    { return  cflagmap[i];
 }else return "-1";
  return "-1";
}
