#include  "ModAnalyzer.h"
#include  "ModLtAnalyzer.h"
#include  "AnalDescriptor.h"
#include  "DefAnalyzer.h"
#include  "AnalSetParser.h"
#include  "RootReader.h"
#include  "DbParser.h"
#include  "ModRootReader.h"
#include  "ModLtRootReader.h"
#include  "RecUtil.h"
#include  "TestAnalyzer.h"
#include  "TextSummarizer.h"
#include  "DefAnaHistogrammer.h"
#include <vector>
#include <map>
#include <TVector.h>
#include <TGraph.h>
#include <TProfile.h>
#include <TH1.h>
#include <TFile.h>

//#include  "DbParser.h"


using namespace std;
using namespace RecUtil;


ModLtAnalyzer::ModLtAnalyzer(AnalSetParser* sets, RootReader* reader) {
  thesettings=sets;
  thereader=reader;
}

ModLtAnalyzer::~ModLtAnalyzer() {
}

//
void ModLtAnalyzer::Analyze() {
  // main framework example
  vector<int> selrec;
  selrec.push_back(1);// select record

  // ANALYZE MODULELTFIRST
  thereader->SetRecord(thereader->GetRecordNr("MODULLTFIRST"));
  TestAnalyzer* testanalfirst=new TestAnalyzer(thesettings,thereader);
  testanalfirst-> AnalyzeAllTests();

  DefAnalyzer* defanalfirst=new DefAnalyzer(testanalfirst); 
  defanalfirst->AnalyzeBadCh();
  defanalfirst->AnalyzeSensor();
  defanalfirst->AnalyzeErrors();
  defanalfirst->RemoveNoisyChannels();
  defanalfirst->saveBadChMap();
  defanalfirst->GradeModule();

  delete testanalfirst;
  delete defanalfirst;
  
  // ANALYZE MODULELTCOLD
  thereader->SetRecord(thereader->GetRecordNr("MODULLTCOLD"));
  TestAnalyzer* testanalcold=new TestAnalyzer(thesettings,thereader);
  testanalcold-> AnalyzeAllTests();
  
  DefAnalyzer* defanalcold=new DefAnalyzer(testanalcold); 
  defanalcold->AnalyzeBadCh();
  defanalcold->AnalyzeSensor();
  defanalcold->AnalyzeErrors();
  defanalcold->RemoveNoisyChannels();
  defanalcold->saveBadChMap();
  defanalcold->GradeModule();
  
  delete testanalcold;
  delete defanalcold;
  
  // ANALYZE MODULELTLAST
  thereader->SetRecord(thereader->GetRecordNr("MODULLTLAST"));
  TestAnalyzer* testanallast=new TestAnalyzer(thesettings,thereader);
  testanallast-> AnalyzeAllTests();
  
  DefAnalyzer* defanallast=new DefAnalyzer(testanallast); 
  defanallast->AnalyzeBadCh();
  defanallast->AnalyzeSensor();
  defanallast->AnalyzeErrors();
  defanallast->RemoveNoisyChannels();
  defanallast->saveBadChMap();
  defanallast->GradeModule();
  defanallast->GradeLTModule();
  
  delete testanallast;
  delete defanallast;
  
  thereader->GoToFirstRecord(); // needed for db parser
}
void ModLtAnalyzer::AnalyzeAndPrint() {
  // main framework example
  vector<int> selrec;
  selrec.push_back(1);// select record

  // ANALYZE MODULELTFIRST
  thereader->SetRecord(thereader->GetRecordNr("MODULLTFIRST"));
  TestAnalyzer* testanalfirst=new TestAnalyzer(thesettings,thereader);
  testanalfirst-> AnalyzeAllTests();

  DefAnalyzer* defanalfirst=new DefAnalyzer(testanalfirst); 
  defanalfirst->AnalyzeBadCh();
  defanalfirst->AnalyzeSensor();
  defanalfirst->AnalyzeErrors();
  defanalfirst->saveBadChMap();
  defanalfirst->GradeModule();

  
  // write ascci summary
  TextSummarizer *foutf = new TextSummarizer(defanalfirst, this);
  foutf->PrintSummary();
  delete foutf;

  // write postscript summary
  DefAnaHistogrammer *houtf = new DefAnaHistogrammer(defanalfirst, this);
  houtf->PrintHistogramms();
  delete houtf;

  delete testanalfirst;
  delete defanalfirst;
  
  // ANALYZE MODULELTCOLD
  thereader->SetRecord(thereader->GetRecordNr("MODULLTCOLD"));
  TestAnalyzer* testanalcold=new TestAnalyzer(thesettings,thereader);
  testanalcold-> AnalyzeAllTests();
  
  DefAnalyzer* defanalcold=new DefAnalyzer(testanalcold); 
  defanalcold->AnalyzeBadCh();
  defanalcold->AnalyzeSensor();
  defanalcold->AnalyzeErrors();
  defanalcold->saveBadChMap();
  defanalcold->GradeModule();
  
  // write ascci summary
  TextSummarizer *foutc = new TextSummarizer(defanalcold, this);
  foutc->PrintSummary();
  delete foutc;

  // write postscript summary
  DefAnaHistogrammer *houtc = new DefAnaHistogrammer(defanalcold, this);
  houtc->PrintHistogramms();
  delete houtc;

  delete testanalcold;
  delete defanalcold;

  
  // ANALYZE MODULELTLAST
  thereader->SetRecord(thereader->GetRecordNr("MODULLTLAST"));
  TestAnalyzer* testanallast=new TestAnalyzer(thesettings,thereader);
  testanallast-> AnalyzeAllTests();
  
  DefAnalyzer* defanallast=new DefAnalyzer(testanallast); 
  defanallast->AnalyzeBadCh();
  defanallast->AnalyzeSensor();
  defanallast->AnalyzeErrors();
  defanallast->saveBadChMap();
  defanallast->GradeModule();
  defanallast->GradeLTModule();
  
  // write ascci summary
  TextSummarizer *fout = new TextSummarizer(defanallast, this);
  fout->PrintSummary();
  delete fout;

  // write postscript summary
  DefAnaHistogrammer *hout = new DefAnaHistogrammer(defanallast, this);
  hout->PrintHistogramms();
  delete hout;

  delete testanallast;
  delete defanallast;
  

  thereader->GoToFirstRecord(); // needed for db parser
}
