#include "RootReader.h"
#include "daqerrormsg.h"
#include <iostream>
#include <TFile.h> 
#include <TKey.h>
#include "RecUtil.h"
#include <map>

using namespace std;
using namespace RecUtil;
//using namespace ReadUtil;

RootReader::RootReader(const char*   tf, const char* opt) {
  bfile=false;  
  therootfile=new TFile(tf,opt);
  theFileFound= ! therootfile->IsZombie();
  if(! theFileFound) { cout<<" file not found "<< tf<<endl;; return;}
  bfile=true;
  therecords.clear();
  FillRecordType();
  DT_DEBUG_LTAPPL(" created rootreader ");
  thecurrecpnt=therecords.begin();
  theid = GetId() ;
  soft = "null";
}
/**/
RootReader::~RootReader() {
  delete therootfile;
}
//
bool RootReader::cd() { 
   return therootfile->cd();
}
//

const char* RootReader::getModType() {
  TNamed* n=(TNamed*) Find("TNmodtype");
  DT_WARN("[RootReader::GetModeType()] n = "<<n);
  if(n==NULL) modtype = "ghost";
  else if(!strcmp(n->GetTitle(),""))  modtype = "dummy";
  else modtype=n->GetTitle();
  DT_WARN("[RootReader::getModType()] : "<<modtype);
  return modtype;
}
// --------------------------------------------------------------------------
// Preliminary : if TNprogramversion==NULL -> arc
TString RootReader::getSoftware() {

  if(!soft.Contains("null")) return soft;

  SetRecord(GetRecordNr());
  TNamed* n=(TNamed*) Find("TNprogramversion");
  DT_WARN("[ModAnalyzer::getSoftware()] n = "<<n);
  if(n==NULL) soft = "arc";
  else if(!strcmp(n->GetTitle(),""))  soft = "dummy";
  else soft = n->GetTitle();
  DT_WARN("[RootReader::getSoftware()] : "<<soft);
  return soft;
}
//
int RootReader::FillRecordType() {
  therootfile->cd();
  TList* tlist= therootfile->GetListOfKeys();
  therecords.clear();
  TObject*  to=tlist->First();
  bool lastmod=false;
  do { 
    if(to == tlist->Last())  {lastmod=true;} 
    TKey* k=(TKey*)to;
    TString rname(k->GetName());
    TString r="Record";
    Ssiz_t rs=rname.Index(r);
    if( rs >= 0) {
      rs=rs+r.Length();
      DT_DEBUG_RANA( " name "<<rname.Data() << " rs "<< rs );
      rname.Remove(0,rs);
      int recnr = atoi(rname.Data());
      if (SetRecord(recnr)) {
	TNamed* rectype=(TNamed*) Find("TNcdb");
	if (rectype) { 
	  // no check for existing records 
	  therecords[recnr]=rectype->GetTitle();
	}
	else {
	  therecords[recnr]="unknown";
	}
      }
      else {
	DT_WARN("unexpected state ");
      }
    }
    to=tlist->After(to);
  }while (!lastmod);

  therootfile->cd();
  // --------
 for(map<int,TString>::iterator it = therecords.begin(); it != therecords.end(); it++) {
   //   cout<<"RECORD IS "<<(*it).first<<" et" <<(*it).second<<endl;
 }// wait();
 // --------
 return therecords.size(); 
}
//
int RootReader::GoToFirstRecord(bool changedir ){
  int recnr=-1;
  if(changedir) {
    therootfile->cd();
  }
  if( therecords.size()) { 
    thecurrecpnt=therecords.begin();
    recnr = thecurrecpnt->first;
    DT_DEBUG_RANA(" first record nr = "<<recnr);
    if( changedir ) {
      if (!  SetRecord(recnr) ) {return -1; }
    }
  }
  // else recnr stays -1 
  return recnr ;
}
//
int RootReader::GoToNextRecord(bool changedir){
  int recnr=-1;
  if(changedir) {
    therootfile->cd();
  }
  if( therecords.size()) { 
    thecurrecpnt++; 
    if( thecurrecpnt != therecords.end()) {
      recnr = thecurrecpnt->first;
      if( changedir ) {
	if (!  SetRecord(recnr) ) {return -1; }
      }
    }
    else { return -2;}
  }
  // else recnr stays -1 
  return recnr ;
}
//
int RootReader::GetLastRecord(bool changedir){
int recnr=-1;
  if(changedir) {
    therootfile->cd();
  }
  if( therecords.size()) { 
    thecurrecpnt=therecords.end();
    thecurrecpnt--;
    recnr = thecurrecpnt->first;
    if( changedir ) {
      if (!  SetRecord(recnr) ) {return -1; }
    }
  }
  // else recnr stays -1 
  return recnr ;
}


/* find object in the record */
TObject*  RootReader::Find(const char* fullobjectname, bool chdir,bool fromfile ){
  TString  tdcur(gDirectory->GetPath());
  DT_DEBUG_RANA("at the beginnining cur dir  "<<gDirectory->GetPath());
  TString  p = fullobjectname;
  TString  objectname=fullobjectname;
  int  length = p.Length();
  int  il = p.Last('/');
  TString p2 = p;
  if(il >= 0) {
    objectname= p(il+1,length);
    p2 = p(0, il+1);
    if( !gDirectory->cd(p2)) {
      gDirectory->cd( tdcur);
      return 0;
    }
  }
  
  TObject*  to=0;
  DT_DEBUG_RANA("check if the object " <<objectname << " is present in file" );
  TKey* tn=gDirectory->FindKeyAny(objectname);
  DT_DEBUG_RANA("cur dir  "<<gDirectory->GetPath());
  if(tn) {
    while( tn->IsFolder()){ 
      gDirectory->cd(objectname);
      DT_DEBUG_RANA("is dir  cd into dir  "<<gDirectory->GetPath());
      tn=gDirectory->FindKeyAny(objectname);
    }
  }
  if( tn) { 
    // if found we are in the diretory 
    if( ! fromfile) { 
      to= gDirectory->FindObject(objectname);
    }
    if(! to) { to=tn->ReadObj();    }
  }
  if(to) {
    // found so sure it is a TNamed   ? 
    TNamed* tn=(TNamed*) to;
    // patch for TNcdb
    TString classname(to->ClassName());
    if(classname  == "TNamed" ) {
      DT_DEBUG_RANA(" TNamed object found");
      // if title is set it is assumed that name is same as objectname in root file
      if( ! TString(tn->GetTitle()).Length()  ){
	// so first char ==0 so empty 
       tn->SetTitle(tn->GetName());
      }
    }
    if(classname != "TVector")    tn->SetName(objectname);
    DT_DEBUG_RANA("object "<< tn->GetName()<<" title  "<<tn->GetTitle() 
		  <<" currentdir " << gDirectory->GetPath());
    bool supported=false;
    TString rootclasstype(to->ClassName());
    if( !chdir) { gDirectory->cd(tdcur);}
  }
  else {
	cout << " currentdir " << gDirectory->GetPath() << endl;
    DT_WARN(" ERROR ! No objects "<<objectname<<" has been found");
    to = 0;
    gDirectory->cd(tdcur);
  }
  return to;
}

/* set record  */
bool  RootReader::SetRecord(int i) {
  if(i==-1) i = therec;
  TString rec="Record";
  rec+=i;
  therootfile->cd();
  Bool_t isHere = gDirectory->cd(rec);
  if(isHere==kFALSE) {
    cout<<"[RootReader::SetRecord] Record"<<i <<" does not exist"
	<<"                        Here are the available records "<<endl;
    gDirectory->ls("-d");
  }
  //  cout << "SETRECORDNR IS " << rec<<endl; //wait();
  therec = i;
  return isHere;
}


bool  RootReader::SetModule(int i) {
  TString rec="Module";
  rec+=i;
//  therootfile->cd();
  Bool_t isHere = gDirectory->cd(rec);
  if(isHere==kFALSE) {
    cout<<"[RootReader::SetModule] Module"<<i <<" does not exist"
	<<"                        Here are the available Modules "<<endl;
    gDirectory->ls("-d");
  }
  return isHere;
}
//bool  RootReader::SetModeDir(int mode,int recnr,string modid  ){
//  return SetModeDir( apvmodedirname(mode),recnr,modid);
//}
 
//bool  RootReader::SetModeDir(const char* mode,int recnr,string modid  ){
//  TString ts=mode;
//  return SetModeDir( ts,recnr,modid);
//} 

bool  RootReader::SetModeDir(TString  mode,int recnr,string modid  ){
  bool retvalue=true;
  if(recnr >= 0) {
    retvalue = SetRecord(recnr) ;
  }
  if( retvalue) {
    retvalue = gDirectory->cd(mode );
  }
  return retvalue;
}
// ------------------------------------------------------------------------
int  RootReader::Write( const char* dirname, const char* objname, 
			TObject* to, int opt, bool makedir)
{
  return  Write(dirname,to,opt,objname, makedir);
}
// -------------------------------------------------------------------------
int   RootReader::Write(const char* dirname, TObject* to,int opt,
			const char* objname, bool makedir ,bool chdir) 
{
  int byteswrite=0;
  TString curpath = gDirectory->GetPath();
  if (dirname) { 
    if( ! gDirectory->cd(dirname) ) {
      gDirectory->cd(curpath);
      if ( makedir) {
	TString p= dirname;	
	if (p.First('/')==0) {
	  therootfile->cd();
	  p=p(1,p.Length());
	}
	do {
	  TString p2 = p;
	  int s= p2.First('/');
	  if (s != -1) {
	    p2=p2(0,s);
	    p=p(s+1,p.Length());
	    if( ! gDirectory->cd(p2) ) {
	      if( ! gDirectory->mkdir(p2)){return -4;  }
	      else {  gDirectory->cd(p2);}
	    }
	  }
	} while ( p.First('/') != -1);
	if ( p.Length() ) {
	  if( ! gDirectory->cd(p) ) {
	    if( ! gDirectory->mkdir(p)){return -3;}
	    else { gDirectory->cd(p);}
	  }      
	}
      }
      else {
	return -1;
      }   
    }
  }
  
  if(! (byteswrite = to->Write(objname,opt)) ) {
    gDirectory->cd(curpath);
    return byteswrite;
  }
  //else 
  if ( ! chdir) {
    gDirectory->cd(curpath);
  }
  return byteswrite;
}

//
bool   RootReader::Update(const char* objectname){
  TNamed* to=(TNamed*)  gDirectory->FindObject(objectname);
  if( ! to) { return false;}
   // else 
  return Update( to);
}
//
bool   RootReader::Update(TNamed* tname){
  if(! tname) {return false;}
  TKey* tn=gDirectory->FindKey(tname->GetName());
  if( ! tn) { return false; }
  //else
  TObject* fileobj=tn->ReadObj();
  TString objname(fileobj->GetName());
  delete fileobj;
  TString filename(tname->GetName());
  tname->SetName(objname.Data());
  if ( fileobj) {
    if( tname->Write(filename.Data() )){
      DT_DEBUG_RANA("Title "<<tname->GetTitle()<<" "<<gDirectory->GetPath());
      return true;
    }
    else { 
      DT_DEBUG_RANA("failed write Title "<<tname->GetTitle());
      return false;
    }
  }
  else {   
    DT_DEBUG_RANA("no file object file found ");
    return false;}
} 



int  RootReader::GetRecordNr(TString run) {
  int recnr=-1;
  if(run.Sizeof()<=1) {
    TString p1=gDirectory->GetPath();
    TString r="Record";
    Ssiz_t rs=p1.Index(r);
    if( rs >= 0) {
      TString p2=p1;
      rs+=r.Length();
      p2.Remove(0,rs);
      int rl=p2.First('/');
      if(rl >=0){ p2.Remove(rl);}
      recnr = atoi(p2.Data());
    }
  }
  else {
    for(map<int,TString>::iterator it = therecords.begin(); it != therecords.end(); it++) {
      if(!run.CompareTo((*it).second)) {
	//	cout<<(*it).second<<endl;
	therec = (*it).first;
      }
    }// wait();
    
  }
  //  cout << "GETRECORDNR IS " << recnr<<endl; //wait();
  return recnr;

}


int RootReader::GetModuleNr() {
  TString p1=gDirectory->GetPath();
  int modnr=-1;
  TString r="Module";
  Ssiz_t rs=p1.Index(r);
  if( rs >= 0) {
    // nothing to check yet
    TString p2=p1;
    rs+=r.Length();
    p2.Remove(0,rs);
    int rl=p2.First('/');
    if(rl >=0){p2.Remove(rl);}  
    modnr  = atoi(p2.Data());
  } 
  return modnr;
}

TString RootReader::GetId() {
  TString p1 = therootfile->GetName();
  Ssiz_t curs = p1.Last('/');
  TString modid = p1;
  if(curs >= 0) modid.Remove(0,curs+1);
  
  curs = modid.Index(".root");
  if(curs >=0) modid.Remove(curs);  
  
  return modid;
}
int RootReader::ModCounter(){
int counter = 0;
char str[40];
 for (int i=1;i<28;i++)
 {
      sprintf(str,"Record%d/Module%d",GetRecordNr(),i);
     TDirectory*  dd=(TDirectory*)therootfile->Get(str);
      if (dd) counter++;
  }

return counter;
} 
 

TString RootReader::GetApvMode() {
  TString p1=gDirectory->GetPath();
  TString  apvmode=apvmodedirname(PEAKINVON);
  Ssiz_t rs=p1.Index(apvmode);
  if( rs >= 0) { return apvmode; }
  apvmode=apvmodedirname(PEAKINVOFF);
  rs=p1.Index(apvmode);
  if( rs >= 0) { return apvmode; }
  apvmode=apvmodedirname(DECINVON);
  rs=p1.Index(apvmode);
  if( rs >= 0) { return apvmode; }
  apvmode=apvmodedirname(DECINVOFF);
  rs=p1.Index(apvmode);
  if( rs >= 0) { return apvmode; }
}

TString RootReader::GetDate()
{
  SetRecord(GetRecordNr());
  TNamed* data= (TNamed*) Find("Date_C");
   if(data)  return data->GetTitle();
      else return "None";
}
TString RootReader::GetTestCenter()
{
  SetRecord(GetRecordNr());
  TNamed* data= (TNamed*) Find("Testcenter_C");
 if(data) return data->GetTitle();
 else  return "None";
}
TString RootReader::GetVersion()
{
  SetRecord(GetRecordNr());
  TNamed* data= (TNamed*) Find("TNprogramversion");
  if(data) return data->GetTitle();
  else return "None";
}
TString RootReader::GetOperator()
{
  SetRecord(GetRecordNr());
  TNamed* data= (TNamed*) Find("Operator_C");
 if(data) return data->GetTitle();
 else  return "None";
}

