#include  "AnalDefs.h"
#include  "AnalDescriptor.h"
#include  "AnalSetParser.h"
#include  "StructLtAnalyzer.h"
#include  "DefAnalyzer.h"
#include  "TestAnalyzer.h"
#include  "TextSummarizer.h"
#include  "RecUtil.h"
#include  "RootReader.h"
#include  "DefAnaHistogrammer.h"

#include <vector>
#include <map>
#include <TVector.h>
#include <TGraph.h>
#include <TProfile.h>
#include <TH1.h>
#include <TFile.h>


using namespace std;
using namespace RecUtil;

StructLtAnalyzer::StructLtAnalyzer(AnalSetParser* sets, RootReader* r)  {
  thesettings=sets;
  thereader=r; 
 }
//
StructLtAnalyzer::~StructLtAnalyzer() {
}
//
void StructLtAnalyzer::AnalyzeAndPrint() 
{

  // ANALYZE SECLTFIRST
  thereader->SetRecord(thereader->GetRecordNr("SECLTFIRST"));
  int numrec=thereader->ModCounter();
  
  for (int i=1;i<numrec+1;i++){
  thereader->SetModule(i);

  // analyze tests
  TestAnalyzer* testanalfirst=new TestAnalyzer(thesettings,thereader);
  testanalfirst->AnalyzePinhole();
  testanalfirst->AnalyzeGain();
  testanalfirst->AnalyzePedestal();
  testanalfirst->AnalyzeNoise();
  testanalfirst->AnalyzePulseShape();
  testanalfirst->AnalyzePulseHeight();
  testanalfirst->AnalyzePeakTime();
  testanalfirst->AnalyzePipeline();
  // or AnalyzeAll();
  DefAnalyzer* defanalfirst=new DefAnalyzer(testanalfirst); 
  defanalfirst->AnalyzeBadCh();
  // or AnalyzeBadCh()
  defanalfirst->AnalyzeSensor();
  defanalfirst->AnalyzeErrors();

  
  // write ascci summary
    TextSummarizer *foutfirst = new TextSummarizer(defanalfirst, this);
    foutfirst->PrintSummary();
    delete foutfirst;

  // write postscript summary
    DefAnaHistogrammer *houtfirst = new DefAnaHistogrammer(defanalfirst, this);
    houtfirst->PrintHistogramms();
    delete houtfirst;

  delete testanalfirst;
  delete defanalfirst;
}// end of loop in modules for SECLTFIRST


  // ANALYZE SECLTCOLD
  thereader->SetRecord(thereader->GetRecordNr("SECLTCOLD"));
  numrec=thereader->ModCounter();
  
  for (int i=1;i<numrec+1;i++){
  thereader->SetModule(i);

  // analyze tests
  TestAnalyzer* testanalcold=new TestAnalyzer(thesettings,thereader);
  testanalcold->AnalyzePinhole();
  testanalcold->AnalyzeGain();
  testanalcold->AnalyzePedestal();
  testanalcold->AnalyzeNoise();
  testanalcold->AnalyzePulseShape();
  testanalcold->AnalyzePulseHeight();
  testanalcold->AnalyzePeakTime();
  testanalcold->AnalyzePipeline();
  // or AnalyzeAll();
  DefAnalyzer* defanalcold=new DefAnalyzer(testanalcold); 
  defanalcold->AnalyzeBadCh();
  // or AnalyzeBadCh()
  defanalcold->AnalyzeSensor();
  defanalcold->AnalyzeErrors();

  
  // write ascci summary
    TextSummarizer *foutcold = new TextSummarizer(defanalcold, this);
    foutcold->PrintSummary();
    delete foutcold;

  // write postscript summary
    DefAnaHistogrammer *houtcold = new DefAnaHistogrammer(defanalcold, this);
    houtcold->PrintHistogramms();
    delete houtcold;

  delete testanalcold;
  delete defanalcold;
}// end of loop in modules for SECLTCOLD


  // ANALYZE SECLTLAST
  thereader->SetRecord(thereader->GetRecordNr("SECLTLAST"));
  numrec=thereader->ModCounter();
  
  for (int i=1;i<numrec+1;i++){
  thereader->SetModule(i);

  // analyze tests
  TestAnalyzer* testanallast=new TestAnalyzer(thesettings,thereader);
  testanallast->AnalyzePinhole();
  testanallast->AnalyzeGain();
  testanallast->AnalyzePedestal();
  testanallast->AnalyzeNoise();
  testanallast->AnalyzePulseShape();
  testanallast->AnalyzePulseHeight();
  testanallast->AnalyzePeakTime();
  testanallast->AnalyzePipeline();
  // or AnalyzeAll();
  DefAnalyzer* defanallast=new DefAnalyzer(testanallast); 
  defanallast->AnalyzeBadCh();
  // or AnalyzeBadCh()
  defanallast->AnalyzeSensor();
  defanallast->AnalyzeErrors();

  
  // write ascci summary
    TextSummarizer *foutlast = new TextSummarizer(defanallast, this);
    foutlast->PrintSummary();
    delete foutlast;

  // write postscript summary
    DefAnaHistogrammer *houtlast = new DefAnaHistogrammer(defanallast, this);
    houtlast->PrintHistogramms();
    delete houtlast;

  delete testanallast;
  delete defanallast;
}// end of loop in modules for SECLTLAST

  thereader->GoToFirstRecord(); // needed for db parser
	
}
//

void StructLtAnalyzer::Analyze() 
{
  // ANALYZE SECLTFIRST
  thereader->SetRecord(thereader->GetRecordNr("SECLTFIRST"));
  int numrec=thereader->ModCounter();
  
  for (int i=1;i<numrec+1;i++){
  thereader->SetModule(i);

  // analyze tests
  TestAnalyzer* testanalfirst=new TestAnalyzer(thesettings,thereader);
  testanalfirst->AnalyzePinhole();
  testanalfirst->AnalyzeGain();
  testanalfirst->AnalyzePedestal();
  testanalfirst->AnalyzeNoise();
  testanalfirst->AnalyzePulseShape();
  testanalfirst->AnalyzePulseHeight();
  testanalfirst->AnalyzePeakTime();
  testanalfirst->AnalyzePipeline();
  // or AnalyzeAll();
  DefAnalyzer* defanalfirst=new DefAnalyzer(testanalfirst); 
  defanalfirst->AnalyzeBadCh();
  // or AnalyzeBadCh()
  defanalfirst->AnalyzeSensor();
  defanalfirst->AnalyzeErrors();

  delete testanalfirst;
  delete defanalfirst;
}// end of loop in modules for SECLTFIRST


  // ANALYZE SECLTCOLD
  thereader->SetRecord(thereader->GetRecordNr("SECLTCOLD"));
  numrec=thereader->ModCounter();
  
  for (int i=1;i<numrec+1;i++){
  thereader->SetModule(i);

  // analyze tests
  TestAnalyzer* testanalcold=new TestAnalyzer(thesettings,thereader);
  testanalcold->AnalyzePinhole();
  testanalcold->AnalyzeGain();
  testanalcold->AnalyzePedestal();
  testanalcold->AnalyzeNoise();
  testanalcold->AnalyzePulseShape();
  testanalcold->AnalyzePulseHeight();
  testanalcold->AnalyzePeakTime();
  testanalcold->AnalyzePipeline();
  // or AnalyzeAll();
  DefAnalyzer* defanalcold=new DefAnalyzer(testanalcold); 
  defanalcold->AnalyzeBadCh();
  // or AnalyzeBadCh()
  defanalcold->AnalyzeSensor();
  defanalcold->AnalyzeErrors();
  
  delete testanalcold;
  delete defanalcold;
}// end of loop in modules for SECLTCOLD


  // ANALYZE SECLTLAST
  thereader->SetRecord(thereader->GetRecordNr("SECLTLAST"));
  numrec=thereader->ModCounter();
  
  for (int i=1;i<numrec+1;i++){
  thereader->SetModule(i);

  // analyze tests
  TestAnalyzer* testanallast=new TestAnalyzer(thesettings,thereader);
  testanallast->AnalyzePinhole();
  testanallast->AnalyzeGain();
  testanallast->AnalyzePedestal();
  testanallast->AnalyzeNoise();
  testanallast->AnalyzePulseShape();
  testanallast->AnalyzePulseHeight();
  testanallast->AnalyzePeakTime();
  testanallast->AnalyzePipeline();
  // or AnalyzeAll();
  DefAnalyzer* defanallast=new DefAnalyzer(testanallast); 
  defanallast->AnalyzeBadCh();
  // or AnalyzeBadCh()
  defanallast->AnalyzeSensor();
  defanallast->AnalyzeErrors();

  delete testanallast;
  delete defanallast;
}// end of loop in modules for SECLTLAST

  thereader->GoToFirstRecord(); // needed for db parser
}
//
