#include "TestAnalyzer.h"
#include "AnalDescriptor.h"
#include "BackPlaneDescriptor.h"
#include "CalAmpDescriptor.h"
#include "FBiasDescriptor.h"
#include "GainDescriptor.h"
#include "IVDescriptor.h"
#include "NoiseDescriptor.h"
#include "PeakTimeDescriptor.h"
#include "PedestalDescriptor.h"
#include "PinholeDescriptor.h"
#include "PipelineDescriptor.h"
#include "PulseHeightDescriptor.h"
#include "RecUtil.h"
#include "LtDefs.h"
#include "RootReader.h"
#include "AnalSetParser.h"
#include "daqerrormsg.h"

#include <vector>
#include <map>
#include <TF1.h>
#include <TGraph.h>
#include <TFile.h>
#include <TFolder.h>
#include <TGraph.h>
#include <TKey.h>
#include <TList.h>
#include <TMath.h>
#include <TProfile.h>
#include <TRandom.h>
#include <TVector.h>
#include <TString.h>

using namespace RecUtil;
using namespace std;

TestAnalyzer::TestAnalyzer(AnalSetParser* s, RootReader* r)  
{
  setparser=s;
  reader=r;
  TNamed* n=(TNamed*) reader->Find("TNmodtype");
  if(n==NULL) modtype = "ghost";
  else if(strcmp(n->GetTitle(),"")==0)  modtype = "dummy";
  else modtype=tntochar(n);
  	DT_DEBUG_RANA("TestAnalyzer:modetype  is "<<modtype);
  nChip = 0;
  nChip = getNChip();
}

TestAnalyzer::~TestAnalyzer() 
{
}

//** -------------------------------------------------------------------------
int TestAnalyzer::AnalyzeAllTests() {
  int stat=1;
  stat*= AnalyzeCommonMode();
  stat*= AnalyzePedestal();
  stat*= AnalyzeNoise();
  stat*= AnalyzeGain();
  stat*= AnalyzePulseShape();
  stat*= AnalyzePulseHeight();
  stat*= AnalyzePeakTime();
  stat*= AnalyzePipeline();
  stat*= AnalyzePinhole();
  stat*= AnalyzeIV();
  stat*= AnalyzeBackPlane();
  stat*= AnalyzeCalAmp();

  return stat;
}
/** ReaAnalyze PulseShape and calculate PeakTime and Height
    Loop over all pulse Profiles stored in the root file 
**/
// ---------------------------------------------------------------------------
int TestAnalyzer::AnalyzePulseShape() 
{

  PulseHeightDescriptor* sets = setparser->getPulseHeightDescriptor(modtype);
  if(!sets)  return 0;
  TString softname = reader->getSoftware();
  int mfit = mfit=sets->fitmode;
  //  DT_DEBUG_RANA("MFIT = "<<mfit); wait();
  char hname[80];
  for(int imode = 0; imode < NModeType; imode++)  {
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    TH1F* hcala=(TH1F*) reader->Find("PulseHeight");
    if(!hcala)   
      hcala= new TH1F("PulseHeight","calPApeak",nChip*NAPVCH, 0.1,float(nChip*NAPVCH)+0.1);
    TString hnamecaltr;
    if(softname.Contains("arc")) hnamecaltr     = "PeakTime"; // ARC
    else if(softname.Contains("lt")) hnamecaltr = "RiseTime"; //LT
      reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
      TH1F* hcaltr=(TH1F*) reader->Find(hnamecaltr);
      if(!hcaltr) 
	hcaltr=new TH1F("PeakTime","calPTrise",nChip*NAPVCH, 0.1,float(nChip*NAPVCH)+0.1);
      
      if(mfit==1 || mfit==2 || mfit==3) {  // recalculate the histogramms
	reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
	gDirectory->cd("PulseShape/Profiles");
      for(int ch=0; ch<nChip; ch++) {
	for(int is=0;is<NAPVCH;is++) {
	  int l=ch*NAPVCH+is;
	  sprintf(hname,"TPcalprof_%.3d",l);
	  TH1F* hp=(TH1F*) reader->Find(hname);
	  if(!hp) continue;
	  //fill hist (can do fit here)
	  float xmax0=hp->GetMaximum();
	  float xmin=hp->GetMinimum();
	  float xtmx=hp->GetMaximumBin()*hp->GetBinWidth(0);
	  float xtmn=hp->GetMinimumBin()*hp->GetBinWidth(0);
	  float dtrise0=xtmx;
	  
	  if(mfit==1) {
	    // max of the histogramm
	    hcala->SetBinContent(ch*NAPVCH+is+1,xmax0);
	    hcaltr->SetBinContent(ch*NAPVCH+is+1,dtrise0);	
	  } 
	  else if(mfit==2) {
	    //gaussian fit around peak
	    float x1 = xtmx-30.;
	    float x2 = xtmx+30.;
	    TF1 *fitfunc = new TF1("gaus","gaus",x1,x2);
	    hp->Fit(fitfunc,"MNOQ","",x1,x2);
	    float xmax=fitfunc->GetParameter(0);
	    float dtrise=fitfunc->GetParameter(1);
	    fitfunc->Delete();
	    if(dtrise>0.&&dtrise<100.&&fabs(xmax)<200.) {
	      hcala->SetBinContent(ch*NAPVCH+is+1,xmax);
	      hcaltr->SetBinContent(ch*NAPVCH+is+1,dtrise);
	    } else {
	      hcala->SetBinContent(ch*NAPVCH+is+1,xmax0);
	      hcaltr->SetBinContent(ch*NAPVCH+is+1,dtrise0);	
	    }
	  } 
	  else if(mfit==3) {
	    // arc like fit
	    // Bail out of fit if pulse height is too low
	    if (fabs(xmax0) < 10) {
	      hcala->SetBinContent(ch*NAPVCH+is+1,xmax0);
	      hcaltr->SetBinContent(ch*NAPVCH+is+1,0);	
	    } else {
		TF1* fitfunc;
		float signal;
		float offset;
		float tau;
		float  fitrange_low;
		float  fitrange_high;
		float dtrise;
		float xmax;
		if(ModeType[imode].Contains("Peak")) {
		  fitrange_low=xtmx-30.;
		  fitrange_high=xtmx+80.;
		  tau = 50.;
		  offset=xtmx-tau;
		  signal = xmax0;
		  fitfunc = 
		    new TF1("PulseShapeFit",peak_Func,fitrange_low,fitrange_high,3);
		} else {
		  fitrange_low=xtmx-20.;
		  fitrange_high=xtmx+20.;
		  tau = 25.;
		  offset = xtmx;
		  signal = xmax0;
		  fitfunc = new TF1("PulseShapeFit",dec_Func,
				    fitrange_low,fitrange_high,3);
		}
		fitfunc->SetParameters(signal,offset,tau);
		hp->Fit("PulseShapeFit","MNOQ","",fitrange_low,fitrange_high);
		// iterate the fit twice with the results from the previos fit
		for (int i=0; i<2; i++) {
		  signal=fitfunc->GetParameter(0);
		  offset=fitfunc->GetParameter(1);
		  tau=fitfunc->GetParameter(2);
		  if(ModeType[imode].Contains("Peak")) {
		    fitrange_low=offset+tau-40;
		    fitrange_high=offset+tau+100;
		  } else {
		    fitrange_low=offset-20;
		    fitrange_high=offset+20;
		  }
		  fitfunc->SetParameters(signal,offset,tau);
		  hp->Fit("PulseShapeFit","MNOQ","",fitrange_low,fitrange_high);
		}
		xmax=fitfunc->GetParameter(0);
		// in Peak mode peak time = tau + offset
		// in Dec mode peak time = offset
		if(ModeType[imode].Contains("Peak")) {
		  dtrise=fitfunc->GetParameter(1)+fitfunc->GetParameter(2);
		} else {
		  dtrise=fitfunc->GetParameter(1);
		}
		fitfunc->Delete();
		// throw out fit if difference in fitted pulse height and original pulse height is greater than 20 ADC
		// or the difference in fitted rise time and original rise time is greater than 20 ms
		if (xmax>200. || xmax<0. ) {xmax=xmax0;}
		if (dtrise>100. || dtrise<0. ) {dtrise=dtrise0;}
		hcala->SetBinContent(ch*NAPVCH+is+1,xmax);
		hcaltr->SetBinContent(ch*NAPVCH+is+1,dtrise);
	    } //if xmax0 >10
	  } //mfit==3
	} // channel loop
      } //apv loop
      } // mfit === 1,2,3 
      else if(mfit==4) { // use CRRC profile
      float fitBound = 20.;
      TF1* fitfunc;
      if(ModeType[imode].Contains("Peak")) fitfunc = new TF1("CRRC",crrc_Func, 0.,250.,3);
      //      else if(ModeType[imode].Contains("Dec")) fitfunc = new TF1("Gaus",dec_Func, 0.,250.,3);
      else if(ModeType[imode].Contains("Dec")) fitfunc = new TF1("gaus","gaus", 0.,250.);
      reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
      gDirectory->cd("PulseShape/Profiles");
      
      for(int ch=0; ch<nChip; ch++)
	for(int is=0;is<NAPVCH;is++) {
	  int l=ch*NAPVCH+is;
	  sprintf(hname,"TPcalprof_%.3d",l);
	  TH1F* hp=(TH1F*) reader->Find(hname);
	  if(!hp) continue;
	  double binMax      = hp->GetMaximumBin();
	  double binMaxWidth = hp->GetBinWidth((int) binMax);
	  double theMax      = (binMax-0.5)*binMaxWidth;
	  double binTime     = hp->GetMaximumBin();
	  double theTime     = (binTime-0.5)*binMaxWidth;
	  double thePeak     = hp->GetMaximum();
	  
	  double theFittedTime, theFittedPeak;
	  if (ModeType[imode].Contains("Peak")) {
	    fitfunc->SetParameters(thePeak*2.71828,25.,theTime);
	    //	    fitfunc->SetParLimits(0,0.,1000.); //solve fit convergence probleme
	    //fitfunc->SetParLimits(1,0.,50.); // solve fit convergence probleme
	    //fitfunc->SetParLimits(2,0.,200.); // solve fit convergence probleme
	    hp->Fit("CRRC","MNOQ","",theMax-40.,theMax+80.);
	    TF1* myfunc = hp->GetFunction("CRRC");
	    theFittedTime = myfunc->GetParameter(2);
	    if(theFittedTime>160.) theFittedTime=160.; // PinHole 
	    theFittedPeak = (myfunc->GetParameter(0))/2.71828;
	    if(theFittedPeak>200.) theFittedPeak=200.; // PinHole 
	    DT_DEBUG_RANA(hname<< " Time=" << theFittedTime << " Peak=" <<theFittedPeak);
	    //	    DT_DEBUG_RANA( hname << " Time=" << theFittedTime << "("<<theTime<<") et Peak=" <<theFittedPeak <<"("<<thePeak<<") et par(1) = "<<myfunc->GetParameter(1));	  
	  }
	  else {
	    //	    fitfunc->SetParameters(thePeak,theTime,15.);
	    //	    fitfunc->SetParLimits(0,0.,250.);// solve fit convergence probleme
	    // fitfunc->SetParLimits(1,20.,150.);// solve fit convergence probleme
	    //	    hp->Fit("Gaus","MNOQ","",theMax-fitBound,theMax+fitBound);
	    hp->Fit("gaus","NQ","",theMax-fitBound,theMax+fitBound);
	    //	    DT_DEBUG_RANA( hname << " Time=" << theFittedTime << "("<<theTime<<") et Peak=" <<theFittedPeak <<"("<<thePeak<<") et par(1) = "<<fitfunc->GetParameter(1));	  
	    //	    TF1* myfunc = hp->GetFunction("Gaus");
	    TF1* myfunc = hp->GetFunction("gaus");
	    theFittedTime = myfunc->GetParameter(1);
	    if(theFittedTime>100.) theFittedTime=100.; // PinHole 
	    theFittedPeak = myfunc->GetParameter(0);
	    if(theFittedPeak>200.) theFittedPeak=200.; // PinHole 
	    DT_DEBUG_RANA(hname << " Time=" << theFittedTime << " Peak=" <<theFittedPeak);
	  }
	  hcala->SetBinContent(l+1,theFittedPeak);
	  hcaltr->SetBinContent(l+1,theFittedTime);
	}//wait();
    }
    // write results
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    const char* dirname = "PulseShape";
    reader->Write(dirname,"PulseHeight",hcala,1,1);
    reader->Write(dirname,"PeakTime",hcaltr,1,1);
    reader->Write(dirname,"RiseTime",hcaltr,1,1);
    // here Recalculate the mean value
    double meanph, rmsph, meanpt, rmspt;
    CalcAvg(hcala,  meanph, rmsph);
    CalcAvg(hcaltr, meanpt, rmspt);
    reader->Write(dirname,"tRMS_RiseTime_F",&totv((float)rmspt),1);
    reader->Write(dirname,"tRMS_Height_F",&totv((float)rmsph),1);
    reader->Write(dirname,"tMean_RiseTime_F",&totv((float)meanpt),1);
    if(ModeType[imode].Contains("On")) 
      reader->Write(dirname,"tMean_Height_F",&totv(-(float)meanph),1);
    else  reader->Write(dirname,"tMean_Height_F",&totv((float)meanph),1);
    
    } // apvmode loop

  return 1;
}
// -------------------------------------------------------------------------
int TestAnalyzer::AnalyzeNoise() 
{
  mbadchnoise.clear();
  LtMap badch; badch.clear();
  LtMap badchtot; badchtot.clear();
  
  NoiseDescriptor* sets=setparser->getNoiseDescriptor(modtype);
  if(!sets){ return 0;}
  //  sets->print(); wait();
  for(int imode = 0; imode < NModeType; imode++) {
    badch.clear();
    DT_DEBUG_RANA("RecordNumber = "<<reader->GetRecordNr());
    if(!reader->SetModeDir(ModeType[imode], reader->GetRecordNr())) {
      DT_DEBUG_RANA(" quit NoisAnal "); 
      return -1;
    }
    TH1F *h_noi_raw = (TH1F*) reader->Find("RawNoise");
    TH1F *h_noi_cms = (TH1F*) reader->Find("CMSubtractedNoise");
    if( !h_noi_raw || !h_noi_cms) {
      DT_DEBUG_RANA("[TestAnalyzer::AnalyzeNoise] WARNING: missing "<<gDirectory->GetPath());
    } else {
    //** normalize/average histos for CM
    vector<double> vnoi_raw_avg;
    vector<double> vnoi_raw_rms;
    CalcChipAvg(h_noi_raw,vnoi_raw_avg, vnoi_raw_rms);
    vector<double> vnoi_cms_avg;
    vector<double> vnoi_cms_rms;
    CalcChipAvg(h_noi_cms,vnoi_cms_avg, vnoi_cms_rms);
    TH1F* h_rawnoi_chipavg = new TH1F("RawNoiseChipAvg","Raw Noise chip Avg",
				      nChip, 0., float (nChip));
    TH1F* h_rawnoi_chiprms = new TH1F("RawNoiseChipRms","Raw Noise chip rms",
				      nChip, 0., float (nChip));
    TH1F* h_cmsnoi_chipavg = new TH1F("CmsNoiseChipAvg","Cms Noise chip Avg",
				      nChip, 0., float (nChip));
    TH1F* h_cmsnoi_chiprms = new TH1F("CmsNoiseChipRms","Cms Noise chip rms",
				      nChip, 0., float (nChip));
    for(int ic = 0; ic < nChip; ic++) {
      h_rawnoi_chipavg->SetBinContent(ic+1, vnoi_raw_avg[ic]); 
      h_rawnoi_chiprms->SetBinContent(ic+1, vnoi_raw_rms[ic]); 
      h_cmsnoi_chipavg->SetBinContent(ic+1, vnoi_cms_avg[ic]); 
      h_cmsnoi_chiprms->SetBinContent(ic+1, vnoi_cms_rms[ic]); 
    }
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    const char* dirname = "Noise";
    reader->Write(dirname, h_rawnoi_chipavg->GetName(), h_rawnoi_chipavg,1,1);
    reader->Write(dirname, h_rawnoi_chiprms->GetName(), h_rawnoi_chiprms,1,1);
    reader->Write(dirname, h_cmsnoi_chipavg->GetName(), h_cmsnoi_chipavg,1,1);
    reader->Write(dirname, h_cmsnoi_chiprms->GetName(), h_cmsnoi_chiprms,1,1);
    
    //** Get cut values (if pct)
    double maxCut[nChip], minCut[nChip], minOSCut[nChip], minTSCut[nChip];
    for(int ichip=0; ichip<nChip; ichip++) {
      if(ModeType[imode].Contains("Peak")) {
	minCut[ichip]   = sets->minNoiseCutPk;
	maxCut[ichip]   = sets->maxNoiseCutPk;
	minOSCut[ichip] = sets->minOneSensNoiseCutPk;
	minTSCut[ichip] = sets->minTwoSensNoiseCutPk;
	DT_DEBUG_RANA("ichip="<<ichip<<" et Peak mincut="<<minCut[ichip]
		       <<" et Peak maxcut="<<maxCut[ichip]);
      }
      else if(ModeType[imode].Contains("Dec")) {
	minCut[ichip]   = sets->minNoiseCutDC;
	maxCut[ichip]   = sets->maxNoiseCutDC;
	minOSCut[ichip] = sets->minOneSensNoiseCutDC;
	minTSCut[ichip] = sets->minTwoSensNoiseCutDC;
      }
      
      if( !(sets->noiseCutType).CompareTo("Percentage") ) {
	maxCut[ichip]   = vnoi_cms_avg[ichip] + maxCut[ichip]*fabs(vnoi_cms_avg[ichip]); 
	minCut[ichip]   = vnoi_cms_avg[ichip] - minCut[ichip]*fabs(vnoi_cms_avg[ichip]); 
	minOSCut[ichip] = vnoi_cms_avg[ichip] - minOSCut[ichip]*fabs(vnoi_cms_avg[ichip]); 
	minTSCut[ichip] = vnoi_cms_avg[ichip] - minTSCut[ichip]*fabs(vnoi_cms_avg[ichip]); 
	DT_DEBUG_RANA("Pct mincut="<<minCut[ichip]<<" maxcut="<<maxCut[ichip]
		      <<" vnoi_cms_avg[ichip]= "<<vnoi_cms_avg[ichip]);
      }      

      if( !(sets->noiseCutType).CompareTo("Relative") ) {
	maxCut[ichip]   = min(vnoi_cms_avg[ichip] + maxCut[ichip],3.0); 
	minCut[ichip]   = vnoi_cms_avg[ichip] - minCut[ichip]; 
	minOSCut[ichip] = vnoi_cms_avg[ichip] - minOSCut[ichip]; 
	minTSCut[ichip] = max(vnoi_cms_avg[ichip] - minTSCut[ichip],0.6); 
	DT_DEBUG_RANA("Rel mincut="<<minCut[ichip]<<" maxcut="<<maxCut[ichip]
		      <<" vnoi_cms_avg[ichip]= "<<vnoi_cms_avg[ichip]);
      }      
    } // loop on chip


    TH1F* hnoi_avsub = new TH1F("CMSNoiseAveSub","CMSNoiseAveSub",nChip*NAPVCH, 0.1,float(nChip*NAPVCH)+0.1);
    TH1F* hnoi_avnorm = new TH1F("CMSNoiseAveNorm","CMSNoiseAveNorm",nChip*NAPVCH, 0.1,float(nChip*NAPVCH)+0.1);
    
      //** Get status
    vector<float> vminCut;
    int ichan = 0;
    for(int i=0; i<nChip; i++) {
      for(int j=0 ; j<NAPVCH; j++) {
	ichan = i * NAPVCH + j;     
	double val = h_noi_cms->GetBinContent(ichan+1);
	double valavesub = val - vnoi_cms_avg[i];
	double valavenorm = val/vnoi_cms_avg[i];
	hnoi_avsub->SetBinContent(ichan,valavesub);
	hnoi_avnorm->SetBinContent(ichan,valavenorm);
	//	DT_DEBUG_RANA("ichan = "<<ichan<<" et val = "<<val);      
	if(val > maxCut[i])  maskbadmap(badch,ichan,sets->flagNoise_High);
	else if(val<minCut[i] && val>minOSCut[i])  
	  maskbadmap(badch,ichan,sets->flagNoise_OSUnbound); 
	else if(val<=minOSCut[i] && val>minTSCut[i]) 
	  maskbadmap(badch,ichan,sets->flagNoise_TSUnbound); 
	else if(val<=minTSCut[i] )  maskbadmap(badch,ichan,sets->flagNoise_Low);
      } // loop on strips
      vminCut.push_back(minCut[i]);
    } // loop on chip
    mbadchnoise[ModeType[imode]] = badch;
    // here write to root files results as histogramm
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    reader->Write(dirname,"Bad_CMSubtractedNoise",toth(badch,nchan),1,1); 
    reader->Write(dirname,"CMSNoiseAveSub",hnoi_avsub,1,1);
    reader->Write(dirname,"CMSNoiseAveNorm",hnoi_avnorm,1,1);
        // here produce map with badch from all apvmode
    masktot(badchtot,badch);
    } // end of else block
  } //** end of loop on modetypes
  
  // here write the total badchlist from noise study intop root file
  reader->SetRecord();
  reader->Write("Summary","TIbadchnoise",&totv(badchtot),1,1);
  
  printMap(mbadchnoise,"Noise");  //wait();

  return 1;
} 
// -------------------------------------------------------------------------
int TestAnalyzer::AnalyzePedestal() 
{
  mbadchpedestal.clear();
  LtMap badch; badch.clear();
  LtMap badchtot; badchtot.clear();
 
  PedestalDescriptor* sets = setparser->getPedestalDescriptor(modtype);
  if(!sets) return 0;
  for(int imode = 0; imode < NModeType; imode++) {
    badch.clear();
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    
    TH1F *h_ped = (TH1F*) reader->Find("Pedestal");
    if(!h_ped) {
      DT_DEBUG_RANA("[TestAnalyzer::AnalyzePedestal] WARNING: missing"<<gDirectory->GetPath());
      return -1;
    }
    vector<double> vped_avg;  vector<double> vped_rms;
    CalcChipAvg(h_ped, vped_avg, vped_rms);
    double maxCut[nChip], minCut[nChip];
    for(int ichip=0; ichip<nChip; ichip++) {
      if(ModeType[imode].Contains("Peak")) {
	minCut[ichip]  = sets->minPedCutPk;
	maxCut[ichip]  = sets->maxPedCutPk;
      }
      else if(ModeType[imode].Contains("Dec")) {
	minCut[ichip]  = sets->minPedCutDC;
	maxCut[ichip]  = sets->maxPedCutDC;
      }
      if( !(sets->pedCutType).CompareTo("Percentage") ) {
	maxCut[ichip]  = vped_avg[ichip] + maxCut[ichip]*fabs(vped_avg[ichip]);
	minCut[ichip]  = vped_avg[ichip] - minCut[ichip]*fabs(vped_avg[ichip]);
	DT_DEBUG_RANA("Pct mincut="<<minCut[ichip]<<" maxcut="<<maxCut[ichip] 
		      <<" vped_avg[ichip]= "<<vped_avg[ichip]);
      }     
    }// loop on chips

    int ichan = 0;
    for(int i=0; i<nChip; i++) 
      for(int j=0 ; j<NAPVCH; j++) {
	ichan = i * NAPVCH + j;     
	double val =h_ped ->GetBinContent(ichan+1);     
	//      DT_DEBUG_RANA("ichan = "<<ichan<<" et val = "<<val);      
	if(val > maxCut[i])    maskbadmap(badch,ichan,sets->flagPedestal_High);
	else if(val<minCut[i]) maskbadmap(badch,ichan,sets->flagPedestal_Low);
      } // loop on strips
    mbadchpedestal[ModeType[imode]] = badch;
    // here write to root files results as histogramm
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    const char*  dirname;
    TString softname = reader->getSoftware();
    if(softname.Contains("arc"))     dirname = "Pedestals"; // ARC
    else dirname = "Pedestal";
    reader->Write(dirname,"Bad_Pedestal",toth(badch,nchan),1,1);
    // here produce map with badch from all apvmode
    masktot(badchtot,badch);
  } //loop on modtype
  reader->SetRecord();
  reader->Write("Summary","TIbadchpedestal",&totv(badchtot),1,1); 
  // need to overwrite previous bad chan list in root file
  reader->Write("Summary","TIbadchped",&totv(badchtot),1,1); 

  printMap(mbadchpedestal,"Pedestal");// wait();
 
  return 1;
}
// ---------------------------------------------------------------------------
int TestAnalyzer::AnalyzePeakTime() 
{
  mbadchpeaktime.clear();
  LtMap badch; badch.clear();
  LtMap badchtot; badchtot.clear();
  
  PeakTimeDescriptor* sets = setparser->getPeakTimeDescriptor(modtype);
  if(!sets) return 0;
  //sets->print();// wait();
  for(int imode = 0; imode < NModeType; imode++) {
    badch.clear();
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    // PATCH FOR ARC vs LT differenciation
    TString softname = reader->getSoftware();
    TString hname;
    if(softname.Contains("arc")) hname = "PeakTime"; // ARC
    else if(softname.Contains("lt")) hname = "RiseTime"; //LT
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    TH1F*  h_pt = (TH1F*) reader->Find(hname);
    if(!h_pt) {
      DT_DEBUG_RANA("[TestAnalyzer::AnalyzePeakTime] WARNING: missing "<<gDirectory->GetPath());
      return -1;
      
    }
    TH1F* hcaltr_avsub = new TH1F("RiseTimeAveSub","RiseTimeAveSub",nChip*NAPVCH, 0.1,float(nChip*NAPVCH)+0.1);
    vector<double> vpt_avg;  vector<double> vpt_rms;
    CalcChipAvg(h_pt, vpt_avg, vpt_rms);
    double maxCut[nChip], minCut[nChip], minOSCut[nChip], minTSCut[nChip];
    for(int ichip=0; ichip<nChip; ichip++) {
      if(ModeType[imode].Contains("Peak")) {
	minCut[ichip]    = sets->minPeakTimeCutPk;
	maxCut[ichip]    = sets->maxPeakTimeCutPk;
	minOSCut[ichip]  = sets->minOSPeakTimeCutPk;
	minTSCut[ichip]  = sets->minTSPeakTimeCutPk;
      }
      else if(ModeType[imode].Contains("Dec")) {
	minCut[ichip]    = sets->minPeakTimeCutDC;
	maxCut[ichip]    = sets->maxPeakTimeCutDC;
	minOSCut[ichip]  = sets->minOSPeakTimeCutDC;
	minTSCut[ichip]  = sets->minTSPeakTimeCutDC;
      }
      if( !(sets->peaktimeCutType).CompareTo("Percentage") ) {
	maxCut[ichip]  = vpt_avg[ichip] + maxCut[ichip]*fabs(vpt_avg[ichip]);
	minCut[ichip]  = vpt_avg[ichip] - minCut[ichip]*fabs(vpt_avg[ichip]);
      	minOSCut[ichip] = vpt_avg[ichip]-minOSCut[ichip]*fabs(vpt_avg[ichip]);
      	minTSCut[ichip] = vpt_avg[ichip]-minTSCut[ichip]*fabs(vpt_avg[ichip]);
      }     
    }// loop on chips
    
    // fill status
    int ichan = 0;
    for(int i=0; i<nChip; i++) {
      for(int j=0 ; j<NAPVCH; j++) {
	ichan = i * NAPVCH + j;     
	double val = h_pt->GetBinContent(ichan+1) - vpt_avg[i];
	hcaltr_avsub->SetBinContent(ichan,val);
	if(ichan>700) DT_DEBUG_RANA("ichan = "<<ichan<<" val = "<<h_pt->GetBinContent(ichan+1)<<" et val -<mean> = "<<val);
	if(val > maxCut[i])  maskbadmap(badch,ichan,sets->flagPeakTime_High);
	else if(val<minCut[i] && val>minOSCut[i])  maskbadmap(badch,ichan,sets->flagPeakTime_OSUnbound); 
	else if(val<=minOSCut[i] && val>minTSCut[i]) maskbadmap(badch,ichan,sets->flagPeakTime_TSUnbound); 
	else if(val<=minTSCut[i])  maskbadmap(badch,ichan,sets->flagPeakTime_Low); 
	
      } // loop on strips
      //      wait();
    } // loop on chip
    
    mbadchpeaktime[ModeType[imode]] = badch;
    // here write to root files results as histogramm
    const char* dirname = "PulseShape";
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    reader->Write(dirname,"Bad_RiseTime",toth(badch,nchan),1,1);
    reader->Write(dirname,"RiseTimeAveSub",hcaltr_avsub,1,1);
    masktot(badchtot,badch);
  } //loop on modtype
  mpeaktime=badchtot;
  // here write the total badchlist into the root file
  reader->SetRecord();
  reader->Write("Summary","TIbadchpeaktime",&totv(badchtot),1); 
  printMap(mbadchpeaktime,"PeakTime"); //wait();
 
  return 1;
}
// ---------------------------------------------------------------------------
int TestAnalyzer::AnalyzeIV() 
{
  float v1 = 300.;
  float v2 = 450.;
  float i1 = 0.;
  float i2 = 0.;
  float iscale=1.;
  
  TString softname = reader->getSoftware();
  TString hname;
  if(softname.Contains("arc")) {      // ARC
    hname  = "IV Curve Measured"; 
    iscale = 1000.;                  //  muA->nA 
  }
  else if(softname.Contains("lt")) hname = "TGiv"; //LT
  reader->SetRecord();
  TGraph*  g_iv = (TGraph*) reader->Find(hname);
  if(!g_iv) {
    DT_DEBUG_RANA("[TestAnalyzer::AnalyzeIV] WARNING: "<<gDirectory->GetPath()
	    <<" is missing plots "<<hname);
    return -1;
  }
  double fv,fi;
  double bmin = 10.0; double bmax = 15.;
  for(int i = 0; i<=g_iv->GetN(); i++) {
    g_iv->GetPoint(i, fv, fi);
    if(fv/v1 > (v1-bmin)/v1 && fv/v1 < (v1+bmax)/v1) {
      i1=fi*iscale; // result in nA
      v1 = fv;
    }
    if(fv/v2 > (v2-bmin)/v2 && fv/v2 < (v2+bmax)/v2) {
      i2=fi*iscale; // result in nA
      v2 = fv; 
      break; // prevent from a bad voltage switchoff
    }
  }
  reader->SetRecord();
  reader->Write("IV","TFVtest1",&totv(v1),1,1); 
  reader->Write("IV","TFItest1",&totv(i1),1,1); 
  reader->Write("IV","TFVtest2",&totv(v2),1,1); 
  reader->Write("IV","TFItest2",&totv(i2),1,1); 
  
  return 1;
}
//
// --------------------------------------------------------------------------
// AnalyzeGain()
// --------------------------------------------------------------------------
int TestAnalyzer::AnalyzeGain() {
  mbadchgain.clear();
  LtMap badch; badch.clear();
  LtMap badchtot; badchtot.clear();
 
  GainDescriptor* sets = setparser->getGainDescriptor(modtype);
  if(sets==0) 
    return 0;
  
  for(int imode = 0; imode < NModeType; imode++) {
    badch.clear();
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    //** Get the raw noise histogramm from rootreader
    TH1F* h_gain_slope = (TH1F*) reader->Find("Gain_Slope");
    TH1F* h_gain_off   = (TH1F*) reader->Find("Gain_Offset");
    TH1F* h_gain_chi   = (TH1F*) reader->Find("Gain_Chi2");
    if( !h_gain_slope || !h_gain_off || !h_gain_chi) {
      DT_DEBUG_RANA("[TestAnalyzer::AnalyzeGain] WARNING: missing "<<gDirectory->GetPath());
	   
      return -1;

    }
    //** normalize/average histos for CM
    vector<double> vgain_avg;
    vector<double> vgain_rms;
    CalcChipAvg(h_gain_slope,vgain_avg, vgain_rms);
    
    //** Get cut values (if pct)
    double maxCut[nChip], minCut[nChip], minOSCut[nChip], maxPHCut[nChip];
    for(int ichip=0; ichip<nChip; ichip++) {
      if(ModeType[imode].Contains("Peak")) {
	minOSCut[ichip] = sets->minOSGainCutPk;
	minCut[ichip]   = sets->minGainCutPk;
	maxCut[ichip]   = sets->maxGainCutPk;
	maxPHCut[ichip] = sets->maxPHGainCutPk;
      }
      else if(ModeType[imode].Contains("Dec")) {
	minOSCut[ichip] = sets->minOSGainCutDC;
	minCut[ichip]   = sets->minGainCutDC;
	maxCut[ichip]   = sets->maxGainCutDC;
	maxPHCut[ichip] = sets->maxPHGainCutDC;
      }
      if( !(sets->gainCutType).CompareTo("Percentage") ) {
	minCut[ichip]= vgain_avg[ichip] - minCut[ichip]*fabs(vgain_avg[ichip]);
	maxCut[ichip]= vgain_avg[ichip] + maxCut[ichip]*fabs(vgain_avg[ichip]);
	minOSCut[ichip] = 
	  vgain_avg[ichip] - minOSCut[ichip]*fabs(vgain_avg[ichip]); 
	maxPHCut[ichip] = 
	  vgain_avg[ichip] + maxPHCut[ichip]*fabs(vgain_avg[ichip]); 
      }
    } // loop on chip
    
      //** Get status  
  int ichan = 0;
  for(int i=0; i<nChip; i++) {
    for(int j=0 ; j<NAPVCH; j++) {
      ichan = i * NAPVCH + j;     
      double val = h_gain_slope->GetBinContent(ichan+1);     
      //      DT_DEBUG_RANA("ichan = "<<ichan<<", val = "<<val<<", MaxPH="<<maxPHCut[i]<<", MinOS="<<minOSCut[i]<<", min="<<minCut[i]<<", max="<<maxCut[i]);      
      if(val >maxPHCut[i])  maskbadmap(badch,ichan,sets->flagGain_PH);
      else if(val<maxPHCut[i] &&  val>maxCut[i])  maskbadmap(badch,ichan,sets->flagGain_High); 
      else if(val< minCut[i]  &&  val>minOSCut[i]) maskbadmap(badch,ichan,sets->flagGain_OSUnbound); 
      else if(val<=minOSCut[i])  maskbadmap(badch,ichan,sets->flagGain_Low);
    } // loop on strips
  } // loop on chip
    mbadchgain[ModeType[imode]] = badch;
    // here write to root files results as histogramm
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    const char* dirname = "Gain";
    reader->Write(dirname,"Bad_Gain",toth(badch,nchan),1,1); 
    masktot(badchtot,badch);
  } //** end of loop on modetypes
  // here write the total badchlist into the root file
  reader->SetRecord();
  reader->Write("Summary","TIbadchgain",&totv(badchtot),1,1); 
  printMap(mbadchgain,"Gain"); //wait();
 
  return 1;
} 
//-----------------------------------------------------------------------
//
int TestAnalyzer::AnalyzePinhole() 
{
  const char* dirname = "Pinhole";
  mbadchpinhole.clear();
  LtMap badch; badch.clear();
  LtMap badchtot; badchtot.clear();
  int flg = 1;
 
  PinholeDescriptor* sets = setparser->getPinholeDescriptor(modtype);

  if(sets==0) { return 0;}

  for(int imode = 0; imode < NModeType; imode++) {
    badch.clear();
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    TH1F *h_ph_diff = (TH1F*) reader->Find("Pinhole_Diff");
    TH1F *h_ph_max  = (TH1F*) reader->Find("Pinhole_Max");
    if(!h_ph_diff || !h_ph_max) {
      DT_DEBUG_RANA("[TestAnalyzer::AnalyzePinhole] WARNING: "<<gDirectory->GetPath()
	      <<" is missing plots  Pinhole_Diff or Pinhole_Max");
      flg = -2;
      continue; // Only PeakOFF has pinhole should not exit!! 
    }
    vector<double> vph_avg;  vector<double> vph_rms;
    CalcChipAvg(h_ph_diff, vph_avg, vph_rms);
    double maxCut1[nChip], maxCut2[nChip];
    for(int ichip=0; ichip<nChip; ichip++) {
      if(ModeType[imode].Contains("Peak")) {
	maxCut1[ichip]  = sets->maxLowPHCutPk;
	maxCut2[ichip]  = sets->maxHighPHCutPk;
      }
      else if(ModeType[imode].Contains("Dec")) {
	maxCut1[ichip]  = sets->maxLowPHCutDC;
	maxCut2[ichip]  = sets->maxHighPHCutDC;
      }
      if( !(sets->pinholeCutType).CompareTo("Percentage") ) {
	maxCut1[ichip]  = vph_avg[ichip] + maxCut1[ichip]*fabs(vph_avg[ichip]);
	maxCut2[ichip]  = vph_avg[ichip] + maxCut2[ichip]*fabs(vph_avg[ichip]);
      }     
    }// loop on chips
    
    // get status
    int ichan = 0;
    for(int i=0; i<nChip; i++) {
      for(int j=0 ; j<NAPVCH; j++) {
	ichan = i * NAPVCH + j;     
	double val = h_ph_diff->GetBinContent(ichan+1);
	
	if(val > maxCut1[i])       badch[ichan] = sets->flagPinhole_Max1;
	if(val > maxCut2[i])  badch[ichan] = sets->flagPinhole_Max2; 
      } // loop on chip
    }    
    mbadchpinhole[ModeType[imode]] = badch;

    // here write to root files results as histogramm
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    reader->Write(dirname,"Bad_Pinhole",&totv(badch),1,1); 
    masktot(badchtot,badch);
    // HERE write the maximum pinhole leakage current
    TString    hname = "PinholeTest_LeakageCurrent";
    TGraph*  g_ph = (TGraph*) reader->Find(hname);
    if(!g_ph) {
      DT_DEBUG_RANA( "[TestAnalyzer::AnalyzePinhole] WARNING: "<<gDirectory->GetPath()
	   <<" is missing graph "<<hname);
    }
    else {
      float ipinhole = 0.;
      double fv,fi;
      for(int i = 0; i<=g_ph->GetN(); i++) {
	g_ph->GetPoint(i, fv, fi);
	if(fi > ipinhole) ipinhole = fi; // result in nA
      }
      reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
      reader->Write(dirname,"TFIphleak",&totv(ipinhole),1); 
    }
  } //loop on modtype
  // here write the total badchlist into the root file
  reader->SetRecord();
  reader->Write("Summary","TIbadchpinhole",&totv(badchtot),1,1); 
  printMap(mbadchpinhole,"Pinhole");
 
  return flg;
}
//-----------------------------------------------------------------------
//
int TestAnalyzer::AnalyzePulseHeight() 
{
  mbadchpulseheight.clear();
  LtMap badch; badch.clear();
  LtMap badchtot; badchtot.clear();
  
  PulseHeightDescriptor* sets = setparser->getPulseHeightDescriptor(modtype);
  if(!sets) return 0;
  for(int imode = 0; imode < NModeType; imode++) {
    badch.clear();
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    TH1F *h_pulse = (TH1F*) reader->Find("PulseHeight");
    if(!h_pulse) {
      DT_DEBUG_RANA("[TestAnalyzer::AnalyzePulseHeight] WARNING: "
	      <<gDirectory->GetPath()<<" is missing plots  PulseHeight");
      return -1;    
    }
    TH1F * hcala_norm= new TH1F("PulseHeightNorm","calPApeakNorm",nChip*NAPVCH, 0.1,float(nChip*NAPVCH)+0.1);    
    vector<double> vpulse_avg;  vector<double> vpulse_rms;
    CalcChipAvg(h_pulse, vpulse_avg, vpulse_rms);
    double maxCut[nChip], minCut[nChip];
    for(int ichip=0; ichip<nChip; ichip++) {      if(ModeType[imode].Contains("Peak")) {
      minCut[ichip]  = sets->minPulseHeightCutPk;
      maxCut[ichip]  = sets->maxPulseHeightCutPk;
    }
    else if(ModeType[imode].Contains("Dec")) {
      minCut[ichip]  = sets->minPulseHeightCutDC;
      maxCut[ichip]  = sets->maxPulseHeightCutDC;
    }
    if( !(sets->pulseheightCutType).CompareTo("Percentage") ) {
      maxCut[ichip]= vpulse_avg[ichip]+maxCut[ichip]*fabs(vpulse_avg[ichip]);
      minCut[ichip]= vpulse_avg[ichip]-minCut[ichip]*fabs(vpulse_avg[ichip]);
    }     
    }// loop on chips
    
    //    badch  = GetPulseHeightStatus(h_pulse, minCut, maxCut);
    //** Get status  
    int ichan = 0;
    for(int i=0; i<nChip; i++) {
      for(int j=0 ; j<NAPVCH; j++) {
	ichan = i * NAPVCH + j;     
	double val = h_pulse->GetBinContent(ichan+1);
	double val_norm = val / fabs(vpulse_avg[i]);
	hcala_norm->SetBinContent(ichan,val_norm);
	//      DT_DEBUG_RANA("ichan = "<<ichan<<" et val = "<<val);      
	TString softname = reader->getSoftware();
	if( softname.Contains("lt")) { // lt is already inversed
	  if (val>maxCut[i])      maskbadmap(badch,ichan,sets->flagPulseHeight_Max);
	  else if(val<minCut[i])  maskbadmap(badch,ichan,sets->flagPulseHeight_Min); 
	}
	if( softname.Contains("arc")) {
	  if(ModeType[imode].Contains("On")) {
	    if(val>maxCut[i])  maskbadmap(badch,ichan,sets->flagPulseHeight_Min);
	    else if(val<minCut[i])  
	      maskbadmap(badch,ichan,sets->flagPulseHeight_Max); 
	  }
	  else {
	  if (val>maxCut[i])      maskbadmap(badch,ichan,sets->flagPulseHeight_Max);
	  else if(val<minCut[i])  maskbadmap(badch,ichan,sets->flagPulseHeight_Min); 
	  }
	}
      } // loop on strips
    } // loop on chip
    mbadchpulseheight[ModeType[imode]] = badch;
    // here write to root files results as histogramm
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    const char* dirname = "PulseShape";
    const char*  objname="Bad_PulseHeight";
    reader->Write(dirname,objname,toth(badch,nchan),1,1);
    reader->Write(dirname,"PulseHeightNorm",hcala_norm,1,1);
    
    masktot(badchtot,badch);
  } //loop on modtype
  // here write the total badchlist into the root file
  // add also peak time
  masktot(badchtot,mpeaktime);
  reader->SetRecord();
  reader->Write("Summary","TIbadchcalprof",&totv(badchtot),1,1);
  
  printMap(mbadchpulseheight,"PulseHeight");// wait();
  //  wait();
  return 1;
}
// ---------------------------------------------------------------------
int TestAnalyzer::AnalyzePipeline() {
 
  mbadchpipeline.clear();
  LtMap badch; badch.clear();
  LtMap badchtot; badchtot.clear();
  PipelineDescriptor* sets = setparser->getPipelineDescriptor(modtype);

  if(sets==0) {DT_DEBUG_RANA( " PipelineDescriptor of " << modtype << " not found"
  ); return 0;}
  
  for(int imode = 0; imode < NModeType; imode++) {
    badch.clear();
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    
    // here analyze all profiles
  }
 
  return 1;
}


// ---------------------------------------------------------------------
int TestAnalyzer::AnalyzeCalAmp() {
  
  mbadchcalamp.clear();
  LtMap badch; badch.clear();
  LtMap badchtot; badchtot.clear();
 
  CalAmpDescriptor* sets = setparser->getCalAmpDescriptor(modtype);
  if(!sets) return 0;
  for(int imode = 0; imode < NModeType; imode++) {
     badch.clear();
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    TH1F *h = (TH1F*) reader->Find("CalAmp");
 
    if(!h) {
      DT_DEBUG_RANA("[TestAnalyzer::AnalyzeCalAMp] WARNING: "<<gDirectory->GetPath()
	   <<" is missing plots  CalAmp");
      return -1;

    }
    vector<double> vph_avg;  vector<double> vph_rms;
    CalcChipAvg(h, vph_avg, vph_rms);
    double maxCut[nChip], minCut[nChip];
    for(int ichip=0; ichip<nChip; ichip++) {
      if(ModeType[imode].Contains("Peak")) {
	maxCut[ichip]  = sets->maxHeight;
	minCut[ichip]  = sets->minHeight;
      }
      else if(ModeType[imode].Contains("Dec")) {
	maxCut[ichip]  = sets->maxHeight;
	minCut[ichip]  = sets->minHeight;
      }
      if( !(sets->calampCutType).CompareTo("Percentage") ) {
	maxCut[ichip]  = vph_avg[ichip] + maxCut[ichip]*fabs(vph_avg[ichip]);
	minCut[ichip]  = vph_avg[ichip] + minCut[ichip]*fabs(vph_avg[ichip]);
      }     
    }// loop on chips

     //** Get status  
  int ichan = 0;
  for(int i=0; i<nChip; i++) {
    for(int j=0 ; j<NAPVCH; j++) {
      ichan = i * NAPVCH + j;     
      double val = h->GetBinContent(ichan+1);     
      //      DT_DEBUG_RANA("ichan = "<<ichan<<" et val = "<<val);      
      if(val >maxCut[i])  maskbadmap(badch,ichan,sets->flagHeight_max);
      else if(val<minCut[i])  maskbadmap(badch,ichan,sets->flagHeight_min); 
  
    } // loop on strips
  } // loop on chip
    mbadchcalamp[ModeType[imode]] = badch;
    // here write to root files results as histogramm
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    const char* dirname= "Calibration";
    reader->Write(dirname,"Bad_CalAmp",toth(badch,nchan),1); 
    masktot(badchtot,badch);
  } //loop on modtype
  // here write the total badchlist into the root file
  reader->SetRecord();
  reader->Write("Summary","TIbadchcal",&totv(badchtot),1); 
  printMap(mbadchcalamp,"CalAmp");
  //  wait();

    return 1;
}

// ---------------------------------------------------------------------
int TestAnalyzer::AnalyzeFBias() {
 mbadchfbias.clear();
  LtMap badch; badch.clear();
  LtMap badchtot; badchtot.clear();

  FBiasDescriptor* sets = setparser->getFBiasDescriptor(modtype);
  if(!sets) return 0;
  for(int imode = 0; imode < NModeType; imode++) {
     badch.clear();
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    TH1F *h = (TH1F*) reader->Find("FBiasAmp");
 
    if(!h) {

      DT_DEBUG_RANA("[TestAnalyzer::AnalyzFBias] WARNING: "<<gDirectory->GetPath()
	   <<" is missing plots FBiasAmp  ");
      return -1;

    }
    vector<double> vph_avg;  vector<double> vph_rms;
    CalcChipAvg(h, vph_avg, vph_rms);
    double maxCut[nChip], minCut[nChip];
    for(int ichip=0; ichip<nChip; ichip++) {
      if(ModeType[imode].Contains("Peak")) {
	maxCut[ichip]  = sets->maxHeight;
	minCut[ichip]  = sets->minHeight;
      }
      else if(ModeType[imode].Contains("Dec")) {
	maxCut[ichip]  = sets->maxHeight;
	minCut[ichip]  = sets->minHeight;
      }
      if( !(sets->fbiasCutType).CompareTo("Percentage") ) {
	maxCut[ichip]  = vph_avg[ichip] + maxCut[ichip]*fabs(vph_avg[ichip]);
	minCut[ichip]  = vph_avg[ichip] + minCut[ichip]*fabs(vph_avg[ichip]);
      }     
    }// loop on chips

     //** Get status  
  int ichan = 0;
  for(int i=0; i<nChip; i++) {
    for(int j=0 ; j<NAPVCH; j++) {
      ichan = i * NAPVCH + j;     
      double val = h->GetBinContent(ichan+1);     
      //      DT_DEBUG_RANA("ichan = "<<ichan<<" et val = "<<val);      
      if(val >maxCut[i])  maskbadmap(badch,ichan,sets->flagHeight_max);
      else if(val<minCut[i])  maskbadmap(badch,ichan,sets->flagHeight_min); 
  
    } // loop on strips
  } // loop on chip
    mbadchfbias[ModeType[imode]] = badch;
    // here write to root files results as histogramm
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    const char* dirname = "ForwardBias";
    reader->Write(dirname,"Bad_FBiasAmp",toth(badch,nchan),1,1); 
    masktot(badchtot,badch);
  } //loop on modtype
// here write the total badchlist into the root file
  reader->SetRecord();
  reader->Write("Summary","TIbadchfb",&totv(badchtot),1,1); 
  printMap(mbadchcalamp,"FBias");
 

    return 1;
}
// ---------------------------------------------------------------------
int TestAnalyzer::AnalyzeBackPlane() 
{
  mbadchbackplane.clear();
  LtMap badch; badch.clear();
  LtMap badchtot; badchtot.clear();
 
  BackPlaneDescriptor* sets = setparser->getBackPlaneDescriptor(modtype);
  if(!sets) return 0;
  for(int imode = 0; imode < NModeType; imode++) {
    badch.clear();
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    TH1F *h = (TH1F*) reader->Find("BPlaneAmp");
    
    if(!h) {

  DT_DEBUG_RANA("[TestAnalyzer::AnalyzeBackPlane] WARNING: "
	   <<gDirectory->GetPath() << " is missing plots  BackPlane");
      return -1;

    }
    vector<double> vph_avg;  vector<double> vph_rms;
    CalcChipAvg(h, vph_avg, vph_rms);
    double maxCut[nChip], minCut[nChip];
    for(int ichip=0; ichip<nChip; ichip++) {
      if(ModeType[imode].Contains("Peak")) {
	maxCut[ichip]  = sets->maxHeight;
	minCut[ichip]  = sets->minHeight;
      }
      else if(ModeType[imode].Contains("Dec")) {
	maxCut[ichip]  = sets->maxHeight;
	minCut[ichip]  = sets->minHeight;
      }
      if( !(sets->backplaneCutType).CompareTo("Percentage") ) {
	maxCut[ichip]  = vph_avg[ichip] + maxCut[ichip]*fabs(vph_avg[ichip]);
	minCut[ichip]  = vph_avg[ichip] + minCut[ichip]*fabs(vph_avg[ichip]);
      }     
    }// loop on chips
    
     //** Get status  
    int ichan = 0;
    for(int i=0; i<nChip; i++) {
      for(int j=0 ; j<NAPVCH; j++) {
	ichan = i * NAPVCH + j;     
	double val = h->GetBinContent(ichan+1);     
	//      DT_DEBUG_RANA("ichan = "<<ichan<<" et val = "<<val);      
	if(val >maxCut[i])  maskbadmap(badch,ichan,sets->flagHeight_max);
	else if(val<minCut[i])  maskbadmap(badch,ichan,sets->flagHeight_min); 
	
      } // loop on strips
    } // loop on chip
    mbadchbackplane[ModeType[imode]] = badch;
    // here write to root files results as histogramm
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    const char* dirname = "BackPlane";
    reader->Write(dirname,"Bad_BPlaneAmp",toth(badch,nchan),1,1); 
    masktot(badchtot,badch);
  } //loop on modtype
  // here write the total badchlist into the root file
  reader->SetRecord();
  reader->Write("Summary","TIbadchbp",&totv(badchtot),1); 
  printMap(mbadchbackplane,"BackPlane");

  
  return 1;
}
// ---------------------------------------------------------------------------
int TestAnalyzer::AnalyzeCommonMode() 
{
 
  TH1F* h_cmnnoi_chiprms = new TH1F("CmnNoiseChipRms","Cmn Noise chip Rms",
				    nChip, 0., float (nChip));
  TVector v_cmnnoi_meanrms(1);

  TString softname = reader->getSoftware();
  TString hnamecaltr;
  string hname;
  if(softname.Contains("arc")) hname     = "CommonMode"; // ARC
  else if(softname.Contains("lt")) hname = "THcmn"; //LT
  const char* dirname = "CommonMode";
  for(int imode = 0; imode < NModeType; imode++)  {
    float rms = 0.;
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    gDirectory->cd(dirname);
    TKey *key;
    TIter next( gDirectory->GetListOfKeys());
    int iapv = 0;
    while ((key = (TKey *) next())) {
      TObject* obj = gDirectory->Get(key->GetName()); // copy object to memory 
      TString on = key->GetName();
      if( on.Contains(hname.c_str())) {
	TH1F*	h = (TH1F*) obj; 
	h_cmnnoi_chiprms->SetBinContent(iapv+1,h->GetRMS());
	rms += h->GetRMS();
	iapv++;
      }
    }
    reader->SetModeDir(ModeType[imode], reader->GetRecordNr());
    reader->Write(dirname, h_cmnnoi_chiprms->GetName(), h_cmnnoi_chiprms,1,1);
    v_cmnnoi_meanrms(0) = rms/nChip;
    reader->Write(dirname,"TVCmnNoiseMeanRms", &v_cmnnoi_meanrms,1,1);
  } // apvmode loop

  return 1;
}
//** --------------------------------------------------------------------------
//** CalcChipAvg()
//** --------------------------------------------------------------------------
int TestAnalyzer::CalcChipAvg(TH1F* h, vector<double>&avg, vector<double>& rms)
{
  avg.clear();
  rms.clear();
  double sum(0.), sum2(0.);
  double val(0.);
  for(int i=0; i<nChip; i++) {
    sum = sum2 = 0.;
    for(int j=0; j<NAPVCH; j++) {
      int ichan = i*NAPVCH + j;
      val   = h->GetBinContent(ichan+1);
      sum  += val;
      sum2 += val*val;
    } //loop on chanels
    double mean = sum/NAPVCH;
    avg.push_back(mean);
    rms.push_back(sqrt(sum2/NAPVCH - mean*mean));
  } // loop on chip
  return 1;
}
// --------------------------------------------------------------------------
int TestAnalyzer::CalcAvg(TH1F* h, double &avg, double &rms)
{
  double sum = 0.;
  double sum2=0.;
  double val = 0.;
  int n = h->GetNbinsX();
  for(int i=0; i < n ; i++) {
    val   = h->GetBinContent(i+1);
    sum  += val;
    sum2 += val*val;
  } //loop on chanels
  avg = sum/n;
  rms = sqrt(sum2/n - avg*avg);
  return 1;
}
//** --------------------------------------------------------------------------
//** printMap()
//** --------------------------------------------------------------------------
void TestAnalyzer::printMap(LtApvMap m, char* testname)
{
  DT_DEBUG_RANA("--------------------------------------------------------------");
  DT_DEBUG_RANA("[TestAnalyzer::printMap] Status for "<<testname<<" analyzis   ");
  for(LtApvMap::iterator itapv = m.begin(); itapv != m.end(); itapv++) {
    DT_DEBUG_RANA("Status for Mode "<<(*itapv).first);
    for(LtMap::iterator it = ((*itapv).second).begin(); 
	it != ((*itapv).second).end(); it++) {
      DT_DEBUG_RANA("Chanel "<<(*it).first<<" is flagged as "<<(*it).second);
    }
  }

}
// -------------------------------------------------------------------------
int TestAnalyzer::getNChip()
{
  if(nChip) return nChip;
  int napv=0;
  TString softname = reader->getSoftware();
  TVector* v;
  if(softname.Contains("lt"))
       {v=(TVector*) reader->Find("TInapvs"); }//LT
  else {v=(TVector*) reader->Find("TVNapvs");} //ARCS

  if(v!=NULL)     napv=tvtoint(v);
  else {
    // PATCH FOR THE MOMENT
    reader->SetModeDir("PeakInvOn", reader->GetRecordNr());
    TH1F* h = (TH1F*) reader->Find("CMSubtractedNoise");
    
    if(h==NULL) {
      DT_DEBUG_RANA( " APV nb can't be found, set to default value : 4"); 
      napv = 4;
    }
    else napv = h->GetNbinsX()/NAPVCH;
    if(napv!=4 && napv!=6) {
      DT_DEBUG_RANA( " APV nb can't be found, set to default value : 4"); 
      napv = 4;
    }
  }
  
  nchan=napv*NAPVCH;  
  return napv;
}
// -------------------------------------------------------------------------
float TestAnalyzer::getIleak()
{
  float Voltage = 450.;
  float il = 0.;
  reader->SetRecord();
  TString hname = "IV Curve Measured";
  TGraph*  g_iv = (TGraph*) reader->Find(hname);
  if(!g_iv) {
     DT_DEBUG_RANA("[TestAnalyzer::AnalyzeIV] WARNING: "<<gDirectory->GetPath()
	 <<" is missing plots "<<hname);
    return -1;
  }
  double fv,fi;
  for(int i = 0; i<=g_iv->GetN(); i++) {
    g_iv->GetPoint(i, fv, fi);
    if(fv == Voltage) il=fi; // result in uA
  }
  Ileak = il;
  return Ileak;
}
// -------------------------------------------------------------------------
float TestAnalyzer::getIsens()
{
  float Voltage = 450.;
  float il = 0.;
  reader->SetRecord();
  TString softname = reader->getSoftware();
  TString hname;
  if(softname.Contains("arc")) hname = "IV Curve from Sensortesting"; //arc
  else {
    DT_DEBUG_RANA(" -----------------------------------------------------------");
    DT_DEBUG_RANA("WARNING !!!!!");
    DT_DEBUG_RANA(" TestAnalyzer::getIsens() returns the default value 999.");
    DT_DEBUG_RANA(" -----------------------------------------------------------");
    return 999.;
  }
  TGraph*  g_iv = (TGraph*) reader->Find(hname);
  if(!g_iv) {
    DT_DEBUG_RANA("[TestAnalyzer::AnalyzeIV] WARNING: "<<gDirectory->GetPath()
	    <<" is missing plots "<<hname);
    return -1;
  }
  double fv,fi;
  for(int i = 0; i<=g_iv->GetN(); i++) {
    g_iv->GetPoint(i, fv, fi);
    if(fv == Voltage) il=fi; // result in uA
  }
  Ileak = il;
  // DT_DEBUG_RANA(Ileak); wait();
  return Ileak;
}
