#include "TextSummarizer.h"
#include "RecUtil.h"
#include "AnalSetParser.h"
#include "PedestalDescriptor.h"
#include "ModFlagsDescriptor.h"
#include "NoiseDescriptor.h"
#include "GainDescriptor.h"
#include "PeakTimeDescriptor.h"
#include "PinholeDescriptor.h"
#include "PulseHeightDescriptor.h"
#include "TestAnalyzer.h"
#include "ModAnalyzer.h"
#include "ModLtAnalyzer.h"
#include "StructLtAnalyzer.h"
#include "ModLtRootReader.h"
#include "ModRootReader.h"
#include "StructLtRootReader.h"

#include "DefaultPath.h"
#include "cmdlinedecode.h"
#include "daqerrormsg.h"

#ifndef NAPVCH
#define NAPVCH  128
#endif

using namespace RecUtil;
using namespace lttest;
using namespace std;

TextSummarizer::TextSummarizer(DefAnalyzer* ta, BaseAnalyzer* mb)
{
  a = ta->getTestAnalyzer();
  map = ta->getMapBadCh();
  if(StructLtAnalyzer* m = dynamic_cast<StructLtAnalyzer*>(mb)){
    rr =  (StructLtRootReader*) a->getRootReader();
  }
  else    if(ModAnalyzer* m = dynamic_cast<ModAnalyzer*>(mb)){
    rr =  (ModRootReader*) a->getRootReader();
  }
  else    if(ModLtAnalyzer* m = dynamic_cast<ModLtAnalyzer*>(mb)){
    rr =  (ModLtRootReader*) a->getRootReader();
  }
  init();
}
// ---------------------------------------------------------------------------
void TextSummarizer::init()
{
  cout <<"[TextSummarizer::init()]     "<<endl;
}
// --------------------------------------------------------------------------
void TextSummarizer::PrintSummary()
{
  cout <<"[TextSummarizer::PrintSummary()]                             "<<endl;
  for(int imode = 0; imode<NModeType; imode++) {
    TString modnum = (a->getRootReader())->GetId();
    int recnum = (a->getRootReader())->GetRecordNr();
    char *ofname = Form("%s/Summary_%s_Rec%d_%s.dat",DefaultDataPath().c_str(),
			modnum.Data(),recnum,ModeType[imode].Data());
    ofstream of(ofname, ios::out);
    if(of.bad()) DT_WARN("ERROR : unable to open file "<<ofname);
    
    PrintHeader(ModeType[imode], of);
    of << endl << endl << endl;
    PrintStatus(ModeType[imode], of);
    of.close();
    
  }
}
// ---------------------------------------------------------------------------
void TextSummarizer::PrintHeader( TString mode, ofstream& os)
{  
  os << "Summary for module " << rr->GetModuleNr()      
     << "; Record " << rr->GetRecordNr() << endl;
  os << Form("%15s","testMode: ")   << mode << endl;
  os << Form("%15s","configFile: ") << (a->getAnalSetParser())->getFileName() << endl;
  os << Form("%15s","Date: ")       << rr->GetDate()                   << endl;
  os << Form("%15s","TestCenter: ") << rr->GetTestCenter()             << endl;
  os << Form("%15s","Version: ")    << rr->GetVersion()                << endl;
  os << Form("%15s","Operator: ")   << rr->GetOperator()               << endl;
  return;
}
// ---------------------------------------------------------------------------
void TextSummarizer::PrintStatus( TString mode, ofstream& os)
{
  int nChip = a->getNChip();
  int chip,channel;
  for (chip=0;chip<nChip;chip++) {
    os << "------------" << endl;
    os << "-- Chip " << chip << " --" << endl;
    os << "------------" << endl;
    os << endl;
    
    // -- Print channel numbers for first half of chip
    os << Form("%13s",""); PrintDigit(os, chip, true,2);
    os << Form("%13s",""); PrintDigit(os, chip, true,1);
    os << Form("%13s",""); PrintDigit(os, chip, true,0);
    
    // -- Print results for first half of chip
    os << "Noise Stat   ";
    PrintStat(os,"noise",chip,mode); os << endl;
    os << "Peaktime Stat";
    PrintStat(os,"peaktime",chip,mode); os << endl;
    os << "Pinhole Stat ";
    PrintStat(os,"pinhole",chip,mode); os << endl;
    os << "PulseH Stat  ";
    PrintStat(os,"pulseheight",chip,mode); os << endl;
    os << "Ped Stat     ";
    PrintStat(os,"pedestal",chip,mode); os << endl;
    //    os << "Gain Stat    ";
    //    PrintStat(os,"gain",chip,mode); os << endl;
    os << "             "; PrintDashes(os,65);
    os << "Summary Stat ";
    PrintProblems(os,chip,mode);
    os << endl << endl;

    // -- Print channel numbers for second half of chip
    os << Form("%13s",""); PrintDigit(os, chip, false,2);
    os << Form("%13s",""); PrintDigit(os, chip, false,1);
    os << Form("%13s",""); PrintDigit(os, chip, false,0);
    
    // -- Print results for second half of chip
    os << "Noise Stat   ";
    PrintStat(os,"noise",chip,mode,kFALSE); os << endl;
    os << "Peaktime Stat";
    PrintStat(os,"peaktime",chip,mode,kFALSE); os << endl;
    os << "Pinhole Stat ";
    PrintStat(os,"pinhole",chip,mode,kFALSE); os << endl;
    os << "PulseH Stat  ";
    PrintStat(os,"pulseheight",chip,mode,kFALSE); os << endl;
    os << "Ped Stat     ";
    PrintStat(os,"pedestal",chip,mode,kFALSE); os << endl;
    //    os << "Gain Stat    ";
    //PrintStat(os,"gain",chip,mode,kFALSE); os << endl;
    os << "             "; PrintDashes(os,65);
    os << "Summary Stat ";
    PrintProblems(os,chip,mode,kFALSE);
    os << endl << endl;
    
    PrintChipSummary(os,chip,mode);
    os << endl << endl;
    PrintKey(os);
    os << endl << endl << endl;
  }

}
// ---------------------------------------------------------------------------
void TextSummarizer::PrintDigit(ofstream &os, int chipNum, bool isFirstHalf, 
				int powerTen)
{
  Int_t digit(0);
  Int_t lowBound = 1 + chipNum*NAPVCH;
  if ( !isFirstHalf ) lowBound += NAPVCH/2;
  Int_t highBound = lowBound + NAPVCH/2;
  
  Int_t counter=lowBound;
  Int_t hecto = counter/100;
  counter -= hecto*100;
  Int_t deca = counter/10;
  counter -= deca*10;
  
  TString digiStream;
  

  for (int ind=lowBound;ind<highBound;ind++) {
    switch (powerTen) {
    case 0 : digiStream.Append(Form("%d",counter)); break ;
    case 1 : digiStream.Append(Form("%d",deca));    break ;
    case 2 : digiStream.Append(Form("%d",hecto));   break ;
    }
    ++counter;
    if ( counter >= 10 ) {
      counter = 0;
      ++deca;
      if ( deca >= 10 ) {
        deca = 0;
        ++hecto;
      }
    }
  }
  os << digiStream << endl;
  return;
}
// ----------------------------------------------------------------------------
void TextSummarizer::PrintStat(ofstream& os, TString label, int chipNum, 
			       TString mode, bool isFirstHalf)
{
  Int_t offset = chipNum*NAPVCH;
  Int_t lowBound = offset;
  Int_t highBound = NAPVCH/2 + offset;
  if ( !isFirstHalf ) {
    lowBound=NAPVCH/2 + offset;
    highBound=NAPVCH + offset;
  }
  LtApvMap mbadchpedestal    = a->getMapPedestal();     
  LtApvMap mbadchnoise       = a->getMapNoise();	      
  LtApvMap mbadchpeaktime    = a->getMapPeakTime();    
  LtApvMap mbadchgain        = a->getMapGain();	      
  LtApvMap mbadchpinhole     = a->getMapPinhole();     
  LtApvMap mbadchpulseheight = a->getMapPulseHeight(); 
  LtApvMap mbadchpulseshape  = a->getMapPulseShape();  

  PedestalDescriptor fped;   fped.init();
  NoiseDescriptor fnoi;      fnoi.init();
  GainDescriptor fgai;       fgai.init();
  PeakTimeDescriptor fpt;    fpt.init();
  PinholeDescriptor fpin;    fpin.init();
  PulseHeightDescriptor fpu; fpu.init();

  for (int ind=lowBound ;ind<highBound ; ind++) {
    char *symbol = ".";
    if ( !label.CompareTo("pedestal") ) {
      if ( mbadchpedestal[mode][ind]==fped.flagPedestal_Low )  symbol="K";
      if ( mbadchpedestal[mode][ind]==fped.flagPedestal_High ) symbol="J";
    } else if ( !label.CompareTo("noise") ) {
      if ( mbadchnoise[mode][ind]==fnoi.flagNoise_OSUnbound )  symbol="O";
      if ( mbadchnoise[mode][ind]==fnoi.flagNoise_TSUnbound )  symbol="T";
      if ( mbadchnoise[mode][ind]==fnoi.flagNoise_Low )        symbol="P";
      if ( mbadchnoise[mode][ind]==fnoi.flagNoise_High )       symbol="N";
    } else if ( !label.CompareTo("gain") ) {
      if ( mbadchgain[mode][ind]==fgai.flagGain_PH )        symbol="P";
      if ( mbadchgain[mode][ind]==fgai.flagGain_High )      symbol="G";
      if ( mbadchgain[mode][ind]==fgai.flagGain_OSUnbound ) symbol="O";
      if ( mbadchgain[mode][ind]==fgai.flagGain_Low )       symbol="T";
    } else if ( !label.CompareTo("peaktime") ) {
      if ( mbadchpeaktime[mode][ind]==fpt.flagPeakTime_OSUnbound ) symbol="O";
      if ( mbadchpeaktime[mode][ind]==fpt.flagPeakTime_TSUnbound ) symbol="T";
      if ( mbadchpeaktime[mode][ind]==fpt.flagPeakTime_Low )       symbol="P";
      if ( mbadchpeaktime[mode][ind]==fpt.flagPeakTime_High )      symbol="N";
    } else if ( !label.CompareTo("pinhole") ) {
      if ( mbadchpinhole[apvmodedirname(PEAKINVOFF)][ind]==fpin.flagPinhole_Max1 )   symbol="P";
      if ( mbadchpinhole[apvmodedirname(PEAKINVOFF)][ind]==fpin.flagPinhole_Max1 )   symbol="X";
    } else if ( !label.CompareTo("pulseheight") ) {
      if ( mbadchpulseheight[mode][ind]==fpu.flagPulseHeight_Min )   symbol="L";
      if ( mbadchpulseheight[mode][ind]==fpu.flagPulseHeight_Max )   symbol="H";
    } else {
      cout << "[TextSummarizer::PrintStat()] WARNING : Unknown label " 
	   << label.Data() << endl
           << "                      Unable to decode status." << endl;
      return;
    }
    os << symbol;
  }
  return;
}
// -------------------------------------------------------------------------
void TextSummarizer::PrintDashes ( ofstream& os, int nDashes )
{
  Int_t i;
  for (i=0;i<nDashes;i++) {
    os << "-";
  }
  os << endl;
}
// --------------------------------------------------------------------------
void TextSummarizer::PrintProblems( ofstream &os, Int_t chipNum, TString mode, 
				    bool isFirstHalf)
{
  Int_t offset = chipNum*NAPVCH;
  Int_t lowBound = offset;
  Int_t highBound = NAPVCH/2 + offset;
  if ( !isFirstHalf ) {
    lowBound=NAPVCH/2 + offset;
    highBound=NAPVCH + offset;
  }
  ModFlagsDescriptor* f = a->getAnalSetParser()->getModFlagsDescriptor(a->getModType()); 

  if(f==0) {
    cout << " ModFlagsDescriptor of " << a->getModType() << " not found"
	 <<endl; return;
  }

  for (int ind=lowBound;ind<highBound;ind++) {
    char *symbol   = " ";
    if ( map[mode][ind] == f->chfunbound1)     symbol="O";
    if ( map[mode][ind] == f->chflikeunbound1) symbol="A";
    if ( map[mode][ind] == f->chfunbound2)     symbol="T";
    if ( map[mode][ind] == f->chflikeunbound2) symbol="B";
    if ( map[mode][ind] == f->chfpin)          symbol="P";
    if ( map[mode][ind] == f->chflikepin)      symbol="C";
    if ( map[mode][ind] == f->chflikemidopen)  symbol="D";
    if ( map[mode][ind] == f->chfnoisy)        symbol="N";
    if ( map[mode][ind] == f->chfshort)        symbol="S";
    if ( map[mode][ind] == f->chflikeshort)    symbol="E";
    if ( map[mode][ind] == f->chflikeopen)     symbol="F";
    if ( map[mode][ind] == f->chfunknown)      symbol="?";
    os << symbol ;    
  }
  return;
}
// ----------------------------------------------------------------------------
void TextSummarizer::PrintChipSummary( ofstream &os, int chip, TString mode )
{
  int nPedFail(0), nNoiFail(0), nGainFail(0), nPinhFail(0), nPHFail(0), nPeakTimeFail(0);
  
  LtApvMap mbadchpedestal    = a->getMapPedestal();     
  LtApvMap mbadchnoise       = a->getMapNoise();	      
  LtApvMap mbadchpeaktime    = a->getMapPeakTime();    
  LtApvMap mbadchgain        = a->getMapGain();	      
  LtApvMap mbadchpinhole     = a->getMapPinhole();     
  LtApvMap mbadchpulseheight = a->getMapPulseHeight(); 
  LtApvMap mbadchpulseshape  = a->getMapPulseShape();  
  
  nPedFail      = calcFailure(mbadchpedestal[mode],chip);
  nNoiFail      = calcFailure(mbadchnoise[mode],chip);
  nGainFail     = calcFailure(mbadchgain[mode],chip);
  nPHFail       = calcFailure(mbadchpulseheight[mode],chip);
  nPinhFail     = calcFailure(mbadchpinhole[apvmodedirname(PEAKINVOFF)],chip);
  nPeakTimeFail = calcFailure(mbadchpeaktime[mode],chip);
  
  os << "Totals:" << endl;
  os << Form("%20s","Failed pedestal") << Form("%4d",nPedFail) << endl;
  os << Form("%20s","Failed noise")    << Form("%4d",nNoiFail) << endl;
  os << Form("%20s","Failed gain")     << Form("%4d",nGainFail) << endl;
  os << Form("%20s","Failed pinhole")  << Form("%4d",nPinhFail) << endl;
  os << Form("%20s","Failed pulseheight")  << Form("%4d",nPHFail) << endl;
  os << Form("%20s","Failed peaktime")  << Form("%4d",nPeakTimeFail) << endl;
  os << endl;
  return;
}
// ---------------------------------------------------------------------------
int TextSummarizer::calcFailure( LtMap status, int chip )
{
  Int_t i;
  Int_t nFail(0);
  Int_t minChan = NAPVCH*chip;
  Int_t maxChan = NAPVCH*chip+NAPVCH;
  for (i=minChan;i<maxChan;i++) {
    if ( status[i]!=0 ) ++nFail;
  }
  return nFail;
}
// --------------------------------------------------------------------------
void TextSummarizer::PrintKey(ofstream &os)
{
  os << "       Test Failure Key         Channel Fault Key" << endl;
  os << "  O - one sensor unbonded       O - one sensor unbonded" << endl;
  os << "  T - two sensor unbonded       A - likely one sensor unbonded" << endl;
  os << "  N - noisy channel             T - two sensor unbonded" << endl;
  os << "  P - pinhole                   B - likely two sensor unbonded" << endl;
  os << "  X - high pulse diff (PH test) P - pinhole" << endl;
  os << "  G - high gain                 C - likely pinhole (saturated channel)" << endl;
  os << "  J - high pedestal             D - possible mid-sensor open" << endl;
  os << "  K - low pedestal              N - noisy channel" << endl;
  os << "  H - high pulse height         S - short" << endl;
  os << "  L - low pulse height          E - likey short" << endl;
  os << "  ? - unidentified problem      F - likely open (conflicting location results)" << endl;
  os << "                                ? - unidentified problem"<<endl;
}

// ---------------------------------------------------------------------------
void TextSummarizer::PrintKeyBad(ofstream &os)
{
  os << " Channel Fault Key" << endl;
  os << "OSO+ :one sensor unbonded (confirmed by two test)" << endl;
  os << "OSO- :likely one sensor unbonded (only seen in one test)" << endl;
  os << "TSO+ :two sensor unbonded (confirmed by two test)" << endl;
  os << "TSO- :likely two sensor unbonded(only seen in one test)" << endl;
  os << "PHL+ :pinhole (confirmed by all possible tests)" << endl;
  os << "PHL- :likely pinhole (saturated channel)" << endl;
  os << "MSO- :possible mid-sensor open" << endl;
  os << "NOIS :noisy channel" << endl;
  os << "SHT+ :short(low pulse shape + 1 at least one channel weird noise" << endl;
  os << "SHT- :likey short (only low pulse shape)" << endl;
  os << "OPN? :likely open (conflicting location results)" << endl;
  os << "???? :unidentified problem"<<endl;
}
