#include "XmlSetParser.h"
#include "AnalDescriptor.h"
#include "PedestalDescriptor.h"
#include "NoiseDescriptor.h"
#include "PeakTimeDescriptor.h"
#include "GainDescriptor.h"
#include "PinholeDescriptor.h"
#include "PulseHeightDescriptor.h"
#include "PulseShapeDescriptor.h"
#include "FBiasDescriptor.h"
#include "IVDescriptor.h"
#include "PipelineDescriptor.h"
#include "BackPlaneDescriptor.h"
#include "CalAmpDescriptor.h"
#include "CalLightDescriptor.h"
#include "ModFlagsDescriptor.h"


#include <util/PlatformUtils.hpp>
#include <util/XMLString.hpp>
#include <util/XMLUniDefs.hpp>
#include <framework/XMLFormatter.hpp>
#include <util/TranscodingException.hpp>
#include <dom/DOM_DOMException.hpp>
#include <parsers/DOMParser.hpp>
#include <dom/DOM.hpp>
#include <dom/DOM_NamedNodeMap.hpp>
#include "DOMTreeErrorReporter.hpp"
#include <cstdio>

using namespace std;


static bool                     gDoNamespaces          = false;
static bool                     gDoCreate              = false;
static XMLCh*                   gEncodingName          = 0;
//static XMLFormatter::UnRepFlags gUnRepFlags            = XMLFormatter::UnRep_CharRef;
static DOMParser::ValSchemes    gValScheme             = DOMParser::Val_Auto;
static XMLFormatter*            gFormatter             = 0;

XmlSetParser::XmlSetParser() {
   fFileName="none";
   clear();
   DT_NOTE(" init setting parser clear  done ");
}

XmlSetParser::~XmlSetParser(){ 
clear();
}

void XmlSetParser::clear()
{ 
  DT_DEBUG_SP( " Clear PedestalDescriptor "<<nPedestalDescriptors); 
  fPedestalDescriptorMap.clear();
  nPedestalDescriptors=0; 
  
  DT_DEBUG_SP(" Clear NoiseDescriptor "<<nNoiseDescriptors); 
  fNoiseDescriptorMap.clear();
  nNoiseDescriptors=0;
  
  DT_DEBUG_SP(" Clear PeakTimeDescriptor "<<nPeakTimeDescriptors); 
  fPeakTimeDescriptorMap.clear();
  nPeakTimeDescriptors=0;
  
  DT_DEBUG_SP(" Clear PulseHeighDescriptor "<<nPulseHeightDescriptors); 
  fPulseHeightDescriptorMap.clear();
  nPulseHeightDescriptors=0;

  DT_DEBUG_SP(" Clear PinholeDescriptor "<<nPinholeDescriptors); 
  fPinholeDescriptorMap.clear();
  nPinholeDescriptors=0;

  DT_DEBUG_SP(" Clear GainDescriptor "<<nGainDescriptors); 
  fGainDescriptorMap.clear();
  nGainDescriptors=0;

  DT_DEBUG_SP(" Clear CalAmpDescriptor "<<nCalAmpDescriptors); 
  fCalAmpDescriptorMap.clear();
  nCalAmpDescriptors=0;

  DT_DEBUG_SP(" Clear FBiasDescriptor "<<nFBiasDescriptors); 
  fFBiasDescriptorMap.clear();
  nFBiasDescriptors=0;

  DT_DEBUG_SP(" Clear BackPlaneDescriptor "<<nBackPlaneDescriptors); 
  fBackPlaneDescriptorMap.clear();
  nBackPlaneDescriptors=0;

  DT_DEBUG_SP(" Clear IVDescriptor "<<nIVDescriptors); 
  fIVDescriptorMap.clear();
  nIVDescriptors=0;


   
  DT_DEBUG_SP(" Clear PipelineDescriptor "<<nPipelineDescriptors); 
  fPipelineDescriptorMap.clear();
  nPipelineDescriptors=0;
  
  DT_DEBUG_SP(" Clear ModFlagsDescriptor "<<nModFlagsDescriptors); 
  fModFlagsDescriptorMap.clear();
  nModFlagsDescriptors=0;
  
 DT_DEBUG_SP(" IVDescriptor "<<fIVDescriptorMap.size()); 
 fIVDescriptorMap.clear();
 
 
 DT_NOTE(" clear done");
}
int  XmlSetParser::parseFile(const char* name) {    
  fFileName=name;
  DT_NOTE(" parseFile "<<fFileName);
  try
    {
      XMLPlatformUtils::Initialize();
    }
  
  catch(const XMLException& toCatch) { cerr << "Error during";
  cerr<<"Xerces-c Initialization.\n" << " Exception message:" <<
  DOMString(toCatch.getMessage()).transcode() << endl; return -2; }
 
  DOMParser *parser;
  parser = new DOMParser;

  parser->setValidationScheme(gValScheme);
  parser->setDoNamespaces(gDoNamespaces);
  parser->setCreateEntityReferenceNodes(gDoCreate);
  parser->setToCreateXMLDeclTypeNode(true);
  ErrorHandler *errReporter = new DOMTreeErrorReporter();
  
  parser->setErrorHandler(errReporter);
  
  bool errorsOccured = false;
  try
    {
      parser->parse(name);
      
    }
  
  catch (const XMLException& e)
    {
      cerr << "An error occured during parsing 1\n   Message: "
           << DOMString(e.getMessage()).transcode() << endl;
      errorsOccured = true;
    }
  
  
  catch (const DOM_DOMException& e)
    {
      cerr << "An error occured during parsing 2 \n   Message: "
           << DOMString(e.msg).transcode() << endl;
      errorsOccured = true;
    }
  
  catch (...)
    {
      cerr << "An error occured during parsing 4 \n " << endl;
      errorsOccured = true;
    }
  // If the parse was successful, output the document data from the DOM tree
  
  if (!errorsOccured)
    {
      DT_DEBUG_SP( " get doc===== ");
      DOM_Node doc = parser->getDocument();
      
      
      
      DOM_Node n1 = doc.getFirstChild();
      
      while (!n1.isNull())
        {
	  if (DOM_Node::TEXT_NODE == n1.getNodeType()) { DT_DEBUG_SP( "skip 2" );}
	  else
	    { DT_DEBUG_SP( n1.getNodeName().transcode() << " type "<< n1.getNodeType());
	    }
	  
          if (n1.getNodeType() == DOM_Node::ELEMENT_NODE) break;
          n1 = n1.getNextSibling();
        }

      DOM_Node n = n1.getFirstChild();
  
      if (n!=0 && n.isNull())
        cout <<" Bad element "<< n.getNodeName().transcode()<< endl;
      while( !n.isNull() )  { 
	// the node was really an element.
	DT_DEBUG_SP(" look for tags for node type  "<<n.getNodeType());
	if (DOM_Node::TEXT_NODE == n.getNodeType()) { 
	  DT_DEBUG_SP("skip 1");
	}
	else {
	  if (strcmp("PedestalDescriptor",n.getNodeName().transcode())==0)  {  
	    PedestalDescriptorParser(n);
	    DT_DEBUG_SP("pedestaldescriptor "<< n.getNodeName().transcode());
	  }
	  if (strcmp("NoiseDescriptor",n.getNodeName().transcode())==0)  {   
	    NoiseDescriptorParser(n);
	    DT_DEBUG_SP(" noisedescriptor"<< n.getNodeName().transcode());
	  }
         if (strcmp("PeakTimeDescriptor",n.getNodeName().transcode())==0) {  
	    PeakTimeDescriptorParser(n);
	    DT_DEBUG_SP("peaktimedescriptor"<< n.getNodeName().transcode());
	  }
        if (strcmp("PulseHeightDescriptor",n.getNodeName().transcode())==0) {  
	    PulseHeightDescriptorParser(n);
	    DT_DEBUG_SP("pulseheightdescriptor"<< n.getNodeName().transcode());
	  }
        if (strcmp("PinholeDescriptor",n.getNodeName().transcode())==0) {  
	    PinholeDescriptorParser(n);
	    DT_DEBUG_SP("pinholedescriptor"<< n.getNodeName().transcode());
	  }
       if (strcmp("GainDescriptor",n.getNodeName().transcode())==0) {  
	    GainDescriptorParser(n);
	    DT_DEBUG_SP("gaindescriptor"<< n.getNodeName().transcode());
	  }
	  if (strcmp("IVDescriptor",n.getNodeName().transcode())==0){  
	    IVDescriptorParser(n);
	    DT_DEBUG_SP("ivdescriptor"<< n.getNodeName().transcode());
	  }
	  if (strcmp("PipelineDescriptor",n.getNodeName().transcode())==0) {  
	    PipelineDescriptorParser(n);
	    DT_DEBUG_SP("pipelinedescriptor"<< n.getNodeName().transcode());
	  }

	  if (strcmp("CalAmpDescriptor",n.getNodeName().transcode())==0) {  
	    CalAmpDescriptorParser(n);
	    DT_DEBUG_SP("calampdescriptor"<< n.getNodeName().transcode());
	  }
	  if (strcmp("FBiasDescriptor",n.getNodeName().transcode())==0) {  
	    FBiasDescriptorParser(n);
	    DT_DEBUG_SP("fbiasdescriptor"<< n.getNodeName().transcode());
	  }
	  if (strcmp("BackPlaneDescriptor",n.getNodeName().transcode())==0) {  
	    BackPlaneDescriptorParser(n);
	    DT_DEBUG_SP("backplanedescriptor"<< n.getNodeName().transcode());
	  }



	  if (strcmp("ModFlagsDescriptor",n.getNodeName().transcode())==0) { 
	    ModFlagsDescriptorParser(n);
	    DT_DEBUG_SP("modflagsdescriptor"<< n.getNodeName().transcode());
	  } 

	}


          n = n.getNextSibling();
          if (n!=0 && n.isNull())
            cout << " Bad element "<< n.getNodeName().transcode() << endl;
        }
      DT_DEBUG_SP("file processed clean up parser" );
      DT_NOTE(" File parsed "<<int(errorsOccured));
    }

    
  delete gFormatter;

  delete gEncodingName;
  delete parser;
 
  return (int) errorsOccured ;
}

// descriptors  parsers 

int  XmlSetParser::PedestalDescriptorParser(DOM_Node &ei)
{
 DT_NOTE(" Pedestal ");
  DOM_NamedNodeMap attributes = ei.getAttributes();

  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");  
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
 PedestalDescriptor* a = new PedestalDescriptor();
 a->ParseSettings(ei);
 fPedestalDescriptorMap[temp]= *a; // copy object by value to map
 delete a;
 nPedestalDescriptors++ ; 
  return 0;
}

int  XmlSetParser::NoiseDescriptorParser(DOM_Node &ei)
{
 DT_NOTE(" Pedestal ");
  DOM_NamedNodeMap attributes = ei.getAttributes();

  string nameid;
  char temp[30];
  //** change NoiseDescriptor variables name
  DOM_Node e = attributes.getNamedItem("name"); 
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  NoiseDescriptor* a = new NoiseDescriptor();
  a->ParseSettings(ei);
  fNoiseDescriptorMap[temp]= *a;
  delete a;
  nNoiseDescriptors++ ; 
  return 0;
}

int  XmlSetParser::PulseHeightDescriptorParser(DOM_Node &ei)
{
 DT_NOTE(" PulseHeight ");
  DOM_NamedNodeMap attributes = ei.getAttributes();

  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name"); 
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  PulseHeightDescriptor* a = new PulseHeightDescriptor();
  a->ParseSettings(ei);
  fPulseHeightDescriptorMap[temp]= *a;
  delete a;
  nPulseHeightDescriptors++ ; 
  return 0;
}

int  XmlSetParser::PeakTimeDescriptorParser(DOM_Node &ei)
{
 DT_NOTE(" PulseHeight ");
  DOM_NamedNodeMap attributes = ei.getAttributes();

  string nameid;
  char temp[30];

  DOM_Node e = attributes.getNamedItem("name"); 
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  PeakTimeDescriptor* a = new PeakTimeDescriptor();
  a->ParseSettings(ei);
  fPeakTimeDescriptorMap[temp]= *a;
  delete a;
  nPeakTimeDescriptors++ ; 
  return 0;
}
//
int  XmlSetParser::PinholeDescriptorParser(DOM_Node &ei)
{
 DT_NOTE(" PulseHeight ");
  DOM_NamedNodeMap attributes = ei.getAttributes();

  string nameid;
  char temp[30];

  DOM_Node e = attributes.getNamedItem("name"); 
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  PinholeDescriptor* a = new PinholeDescriptor();
  a->ParseSettings(ei);
  fPinholeDescriptorMap[temp]= *a;
  delete a;
  nPinholeDescriptors++ ; 
  return 0;
}
//
int  XmlSetParser::GainDescriptorParser(DOM_Node &ei)
{
 DT_NOTE(" PulseHeight ");
  DOM_NamedNodeMap attributes = ei.getAttributes();

  string nameid;
  char temp[30];

  DOM_Node e = attributes.getNamedItem("name"); 
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  GainDescriptor* a = new GainDescriptor();
  a->ParseSettings(ei);
  fGainDescriptorMap[temp]= *a;
  delete a;
  nGainDescriptors++ ; 
  return 0;
}
//
int  XmlSetParser::PipelineDescriptorParser(DOM_Node &ei)
{
   DT_NOTE(" LatRun ");
  DOM_NamedNodeMap attributes = ei.getAttributes();
  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name"); 
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  PipelineDescriptor* a = new PipelineDescriptor();
  a->ParseSettings(ei); 
 fPipelineDescriptorMap[temp]= *a;
 delete a;
   nPipelineDescriptors++ ; 
  return 0;
}
//
int  XmlSetParser::CalAmpDescriptorParser(DOM_Node &ei)
{
   DT_NOTE(" CalAmpRun ");
  DOM_NamedNodeMap attributes = ei.getAttributes();
  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name"); 
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  CalAmpDescriptor* a = new CalAmpDescriptor();
  a->ParseSettings(ei); 
 fCalAmpDescriptorMap[temp]= *a;
 delete a;
   nCalAmpDescriptors++ ; 
  return 0;
}
//
int  XmlSetParser::FBiasDescriptorParser(DOM_Node &ei)
{
   DT_NOTE(" FBiasRun ");
  DOM_NamedNodeMap attributes = ei.getAttributes();
  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name"); 
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  FBiasDescriptor* a = new FBiasDescriptor();
  a->ParseSettings(ei); 
 fFBiasDescriptorMap[temp]= *a;
 delete a;
   nFBiasDescriptors++ ; 
  return 0;
}
//
int  XmlSetParser::BackPlaneDescriptorParser(DOM_Node &ei)
{
   DT_NOTE(" BackPlane ");
  DOM_NamedNodeMap attributes = ei.getAttributes();
  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name"); 
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  BackPlaneDescriptor* a = new BackPlaneDescriptor();
  a->ParseSettings(ei); 
 fBackPlaneDescriptorMap[temp]= *a;
 delete a;
   nBackPlaneDescriptors++ ; 
  return 0;
}
//
int  XmlSetParser::IVDescriptorParser(DOM_Node &ei)
{
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  // why ?
  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");
  
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  IVDescriptor* a = new IVDescriptor();
  a->ParseSettings(ei);  
 fIVDescriptorMap[temp]= *a;
 delete a;
   nIVDescriptors++ ; 
  return 0;
}
//
  int XmlSetParser::ModFlagsDescriptorParser(DOM_Node &ei)
{
  DOM_NamedNodeMap attributes = ei.getAttributes();
  //int attrCount = attributes.getLength();  //   
  string nameid;
  char temp[30];
  DOM_Node e = attributes.getNamedItem("name");
  
  sscanf(e.getNodeValue().transcode(),"%s",temp  );
  nameid=temp;
  ModFlagsDescriptor* a = new ModFlagsDescriptor();
 a->ParseSettings(ei); 
 fModFlagsDescriptorMap[temp]= *a;
 delete a;
   nModFlagsDescriptors++ ; 
  return 0;
}


AnalDescriptor*   XmlSetParser::getModuleDescriptor(string name) {
  if(getPedestalDescriptor(name)!=0) return  getPedestalDescriptor(name);
  if(getNoiseDescriptor(name)!=0) return  getNoiseDescriptor(name);
  if(getPulseHeightDescriptor(name)!=0) return  getPulseHeightDescriptor(name);
  if(getPeakTimeDescriptor(name)!=0) return  getPeakTimeDescriptor(name);
  if(getPipelineDescriptor(name)!=0) return  getPipelineDescriptor(name);
  if(getIVDescriptor(name)!=0) return  getIVDescriptor(name);
  if(getGainDescriptor(name)!=0) return  getGainDescriptor(name);
  if(getCalAmpDescriptor(name)!=0) return  getCalAmpDescriptor(name);
  if(getFBiasDescriptor(name)!=0) return  getFBiasDescriptor(name);
  if(getBackPlaneDescriptor(name)!=0) return  getBackPlaneDescriptor(name);
  if(getModFlagsDescriptor(name)!=0) return  getModFlagsDescriptor(name);
  return 0;
}

int  XmlSetParser::setDescriptor(AnalDescriptor* nd) {
    if(!nd) {
      DT_WARN("descriptor is null ... exit");
      return -1;
    }
    if(!(nd->Name).CompareTo("NoiseDescriptor")) {
      NoiseDescriptor* dtmp = new NoiseDescriptor();
      dtmp = (NoiseDescriptor*) nd;
      fNoiseDescriptorMap["dummy"] = *dtmp;
      fNoiseDescriptorMap["ghost"] = *dtmp;
      return 1;
    }
    else if(!(nd->Name).CompareTo("PedestalDescriptor")) {
      PedestalDescriptor* dtmp = new PedestalDescriptor();
      dtmp = (PedestalDescriptor*) nd;
      fPedestalDescriptorMap["dummy"] = *dtmp;
      fPedestalDescriptorMap["ghost"] = *dtmp;
      return 1;
    }
    else if(!(nd->Name).CompareTo("PeakTimeDescriptor")) {
      PeakTimeDescriptor* dtmp = new PeakTimeDescriptor();
      dtmp = (PeakTimeDescriptor*) nd;
      fPeakTimeDescriptorMap["dummy"] = *dtmp;
      fPeakTimeDescriptorMap["ghost"] = *dtmp;
      return 1;
    }
    else if(!(nd->Name).CompareTo("GainDescriptor")) {
      GainDescriptor* dtmp = new GainDescriptor();
      dtmp = (GainDescriptor*) nd;
      fGainDescriptorMap["dummy"] = *dtmp;
      fGainDescriptorMap["ghost"] = *dtmp;
      return 1;
    }
    else if(!(nd->Name).CompareTo("PinholeDescriptor")) {
      PinholeDescriptor* dtmp = new PinholeDescriptor();
      dtmp = (PinholeDescriptor*) nd;
      fPinholeDescriptorMap["dummy"] = *dtmp;
      fPinholeDescriptorMap["ghost"] = *dtmp;
      return 1;
    }
    else if(!(nd->Name).CompareTo("PulseHeightDescriptor")) {
      PulseHeightDescriptor* dtmp = new PulseHeightDescriptor();
      dtmp = (PulseHeightDescriptor*) nd;
      fPulseHeightDescriptorMap["dummy"] = *dtmp;
      fPulseHeightDescriptorMap["ghost"] = *dtmp;
      return 1;
    }
   else if(!(nd->Name).CompareTo("PipelineDescriptor")) {
      PipelineDescriptor* dtmp = new PipelineDescriptor();
      dtmp = (PipelineDescriptor*) nd;
      fPipelineDescriptorMap["dummy"] = *dtmp;
      fPipelineDescriptorMap["ghost"] = *dtmp;
      return 1;
    }
   else if(!(nd->Name).CompareTo("IVDescriptor")) {
      IVDescriptor* dtmp = new IVDescriptor();
      dtmp = (IVDescriptor*) nd;
      fIVDescriptorMap["dummy"] = *dtmp;
      fIVDescriptorMap["ghost"] = *dtmp;
      return 1;
    }
  else if(!(nd->Name).CompareTo("CalAmpDescriptor")) {
      CalAmpDescriptor* dtmp = new CalAmpDescriptor();
      dtmp = (CalAmpDescriptor*) nd;
      fCalAmpDescriptorMap["dummy"] = *dtmp;
      fCalAmpDescriptorMap["ghost"] = *dtmp;
      return 1;
    }
  else if(!(nd->Name).CompareTo("FBiasDescriptor")) {
      FBiasDescriptor* dtmp = new FBiasDescriptor();
      dtmp = (FBiasDescriptor*) nd;
      fFBiasDescriptorMap["dummy"] = *dtmp;
      fFBiasDescriptorMap["ghost"] = *dtmp;
      return 1;
    }
  else if(!(nd->Name).CompareTo("BackPlaneDescriptor")) {
      BackPlaneDescriptor* dtmp = new BackPlaneDescriptor();
      dtmp = (BackPlaneDescriptor*) nd;
      fBackPlaneDescriptorMap["dummy"] = *dtmp;
      fBackPlaneDescriptorMap["ghost"] = *dtmp;
      return 1;
    }
    else if(!(nd->Name).CompareTo("ModFlagsDescriptor")) {
      ModFlagsDescriptor* dtmp = new ModFlagsDescriptor();
      dtmp = (ModFlagsDescriptor*) nd;
      fModFlagsDescriptorMap["dummy"] = *dtmp;
      fModFlagsDescriptorMap["ghost"] = *dtmp;
      return 1;
    }
    else {
      std::cout<<"[AnaSetParser::setDescriptor] WARNING! "<<nd->Name
	  <<" is not a valid descriptor name"<<std::endl;
      return -1;
    }
  }

PedestalDescriptor*  XmlSetParser::getPedestalDescriptor( std::string selname )  {
      if ( fPedestalDescriptorMap.end() == fPedestalDescriptorMap.find(selname)) {
	DT_WARN( "PedestalDescriptor " << selname <<" not found" ); 
	if(getPedestalDescriptor("dummy")==0) return 0;
      } else {  return  &fPedestalDescriptorMap[ selname];}
    }
  
NoiseDescriptor*   XmlSetParser::getNoiseDescriptor( std::string selname )  {
      if ( fNoiseDescriptorMap.end() == fNoiseDescriptorMap.find(selname)) {
         DT_WARN( " NoiseDescriptor " << selname <<" not found" ); 
        if(getNoiseDescriptor("dummy")==0) return 0;
      } else {  return  &fNoiseDescriptorMap[ selname];}
     }
PeakTimeDescriptor*   XmlSetParser::getPeakTimeDescriptor( std::string selname )  {
      if ( fPeakTimeDescriptorMap.end() == fPeakTimeDescriptorMap.find(selname)){
          DT_WARN( " PeakTimeDescriptor " << selname <<" not found" ); 
      if(getPeakTimeDescriptor("dummy")==0) return 0;  
       } else {  return  &fPeakTimeDescriptorMap[ selname]; }
    }
GainDescriptor*   XmlSetParser::getGainDescriptor( std::string selname )  {
       if ( fGainDescriptorMap.end() == fGainDescriptorMap.find(selname)){
        DT_WARN( " GainDescriptor " << selname <<" not found" ); 
      if(getGainDescriptor("dummy")==0)  return 0;
      } else {  return  &fGainDescriptorMap[ selname]; }
    }
PinholeDescriptor*  XmlSetParser::getPinholeDescriptor( std::string selname ) {
    if ( fPinholeDescriptorMap.end() == fPinholeDescriptorMap.find(selname)) {
      DT_WARN( " PinholeDescriptor " << selname <<" not found" ); 
        if(getPinholeDescriptor("dummy")==0)    return 0;
      }else { return  &fPinholeDescriptorMap[ selname];}
     }
PulseHeightDescriptor*   XmlSetParser::getPulseHeightDescriptor( std::string selname )  {
     if ( fPulseHeightDescriptorMap.end() == fPulseHeightDescriptorMap.find(selname)){
       DT_WARN( " PulseHeightDescriptor " << selname <<" not found" ); 
         if(getPulseHeightDescriptor("dummy")==0)    return 0;
      }else {  return  &fPulseHeightDescriptorMap[ selname]; }
    }
PipelineDescriptor*  XmlSetParser::getPipelineDescriptor( std::string selname )  {
    if ( fPipelineDescriptorMap.end() == fPipelineDescriptorMap.find(selname)) {
      DT_WARN( " PipelineDescriptor " << selname <<" not found" ); 
     if(getPipelineDescriptor("dummy")==0)    return 0;
      } else {  return  &fPipelineDescriptorMap[ selname];}
    }
  
IVDescriptor*   XmlSetParser::getIVDescriptor( std::string selname )  {
     if ( fIVDescriptorMap.end() == fIVDescriptorMap.find(selname)) {
       DT_WARN( " IVDescriptor " << selname <<" not found" );  
  if(getIVDescriptor("dummy")==0)  return 0;
      }else {  return  &fIVDescriptorMap[ selname];}
    }
CalAmpDescriptor*   XmlSetParser::getCalAmpDescriptor( std::string selname )  {
     if ( fCalAmpDescriptorMap.end() == fCalAmpDescriptorMap.find(selname)) {
       DT_WARN( " CalAmpDescriptor " << selname <<" not found" );  
       if(getCalAmpDescriptor("dummy")==0)  return 0;
      }else {  return  &fCalAmpDescriptorMap[ selname];}
    }
FBiasDescriptor*   XmlSetParser::getFBiasDescriptor( std::string selname )  {
     if ( fFBiasDescriptorMap.end() == fFBiasDescriptorMap.find(selname)) {
       DT_WARN( " FBiasDescriptor " << selname <<" not found" );  
       if(getFBiasDescriptor("dummy")==0)  return 0;
      }else {  return  &fFBiasDescriptorMap[ selname];}
    }

BackPlaneDescriptor*   XmlSetParser::getBackPlaneDescriptor( std::string selname )  {
     if ( fBackPlaneDescriptorMap.end() == fBackPlaneDescriptorMap.find(selname)) {
       DT_WARN( " BackPlaneDescriptor " << selname <<" not found" ); 
         if(getBackPlaneDescriptor("dummy")==0)  return 0;
      }else {  return  &fBackPlaneDescriptorMap[ selname];}
    }


ModFlagsDescriptor*   XmlSetParser::getModFlagsDescriptor( std::string selname )  {
 if ( fModFlagsDescriptorMap.end() == fModFlagsDescriptorMap.find(selname))
      { DT_WARN( " ModFlagsDescriptor " << selname <<" not found" ); 
         if(getModFlagsDescriptor("dummy")==0)  return 0;
      }else {  return  &fModFlagsDescriptorMap[ selname];}
}
void   XmlSetParser::setFitmod(int mfit)
{
  for(map<std::string,PulseHeightDescriptor>::iterator it = fPulseHeightDescriptorMap.begin(); 
      it != fPulseHeightDescriptorMap.end(); it++)
    ((*it).second).fitmode = mfit;
}
