// ---------------------------------------------------------------------------
// TestModLtAnalyzer
// Description : This program shows how to analyzes a module file produced by 
//               Long Term  module tests
//               You can choose either an xml or ascii setting file
//               usage : ./bin/TestModLtAnalyzer -rootin rootfile.root
//               More information on 
//               http://hep.uia.ac.be/cms/testing/software/DefAna/defana.html
// Input    : - rootfile : $HOME/DefAna/data
//            - setting  : defanaset.xml in  $HOME/DefAna/config
//                         or LT_TIB.dat in  $HOME/DefAna/config
// Output : - ascii summary file : $DEFANAHOME/data/Summary_xxx_Recyyy_zzz.dat
//        : - eps controle plots : $DEFANAHOME/data/hNoise_xxx_Recyyy.eps
//                                 $DEFANAHOME/data/hPeakTime_xxx_Recyyy.eps
//                                 $DEFANAHOME/data/hPedestal_xxx_Recyyy.eps
//                                 $DEFANAHOME/data/hPinhole_xxx_Recyyy.eps
//                                 $DEFANAHOME/data/hPulseHeight_xxx_Recyyy.eps
//          where xxx is module number
//                yyy is Rec number
//                zzz is mode type
//          - an xml summary file in $DEFANAHOME/data
// Date        : 5/05/2004
// ---------------------------------------------------------------------------

#include <iostream>

#include <TROOT.h>

#include "XmlSetParser.h"
#include "BasicTextDescriptor.h"
#include "ModLtRootReader.h"
#include "ModLtAnalyzer.h"
#include "DbParser.h"
#include "PulseHeightDescriptor.h"

#include "DefaultPath.h"
#include "cmdlinedecode.h"
#include "RecUtil.h"

using namespace std;
using namespace lttest;
using namespace RecUtil;
// ---------------------------------------------------------------------------
// Configuration : change this 
static string setdat="defanaset.xml"; // xml setting file
//static string setdat="LT_TIB.dat"; // ascii setting file
static string defroot="reslt_M_30200020035266.root"; //default root file
static int mfit=3;
// ---------------------------end of configuration

void usage (string prgname){
  cout <<endl<<" usage example : "<<endl;
  cout <<"\t"<<prgname<< " -rootin file_.root " <<endl;
  cout <<"options:"<<endl;
  cout <<"\t-rootin  :the rootfile  (default "<<defroot<<")"<<endl;  
  cout <<"\t-settings :the settins (cuts) (default "<<setdat<<")"<<endl; 
}

int main(int argc, char **argv)
{
  cout<<"[TestModLtAnalyzer] ... enter"<<endl; 

  string cmdresult;
  string programname=argv[0];
  string filein=DefaultDataPath();
  filein=filein+defroot;
  //  cout << filein<<endl;
  if( "found"== CmdLineDecode(argc,argv,"h",cmdresult,false)){
    usage(programname);
    return 0;
  }
  if( "found"== CmdLineDecode(argc,argv,"rootin",cmdresult,true)){
    filein=cmdresult;
    if( filein[0] != '/') { filein=DefaultDataPath()+filein;  
    }
  }
  
  // initialize the root reader
  cout<<"[TestModLtAnalyzer] ... define a root reader"<<endl; 
  ModLtRootReader *rr =  new ModLtRootReader(filein.c_str());
  
  // initialize the settings
  if( "found" == CmdLineDecode(argc,argv,"fitmode",cmdresult,true)){
    stoint(cmdresult, mfit);
  }
  
  AnalSetParser *setparser;
  
  if( "found" == CmdLineDecode(argc,argv,"settings",cmdresult,true)){
    setdat=cmdresult;
  }
  
  string defdat = DefaultConfigPath() + setdat;
  
  if(setdat.find("xml") < setdat.size()) {
    cout<<"[TestModLtAnalyzer] ... define a XmlSetParser"<<endl; 
    setparser = new XmlSetParser();
    if(setparser->parseFile(defdat.c_str())){
      cout<<"[TestModLtAnalyzer] Can't parse file "<<defdat<<" exit !"<<endl;
      exit(0);
    }  
    //    setparser->setFitmod(mfit);
  }
  else  if(setdat.find("dat") < setdat.size()) {
    cout<<"[TestModLtAnalyzer] ... define a TextSetParser"<<endl; 
    setparser = new BasicTextDescriptor();   
    if(setparser->parseFile(defdat.c_str())){
      cout<<"[TestModLtAnalyzer] Can't parse file "<<defdat<<" exit !"<<endl;
      exit(0);
    }
    //    setparser->setFitmod(mfit);
  }
  else {
    cout<<"[TestModLtAnalyzer] "<<defdat<<" is not a valid file name"<<endl;
    exit(0);
  }
  
  ModLtAnalyzer *analyzer = new ModLtAnalyzer(setparser, rr);
  analyzer->Analyze();
  
  cout << "[TestModLtAnalyzer] ...  create DbParser " << endl;
  
  string xmlin=DefaultConfigPath()+"modlt2.xml";
  DbParser* db=new DbParser(xmlin.c_str());
  db->getDBLUT()->initmapCB();
  db->populate(rr); 
  db->write();
  delete db;
  
  delete setparser;
  delete rr;
  delete analyzer;
  
  cout<<"[TestModLtAnalyzer] ... end"<<endl; 
  
  return 0;
}
