// ---------------------------------------------------------------------------
// TestModuleAnalyzer
// Description : This program shows how to analyzes a module file produced by 
//               ARC
//               You can choose either an xml or ascii setting file
//               usage : ./bin/TestModuleAnalyzer -rootin file.root -recnr yyy
//               More information on 
//               http://neukerma.home.cern.ch/neukerma/DefAna/DefAna.html
// Input       : - rootfile : $DEFANAHOME/data
//               - setting  : defanaset.xml in  $DEFANAHOME/config
//                            or ARCS_TIB.dat in  $DEFANAHOME/config
// Output : - ascii summary file : $DEFANAHOME/data/Summary_xxx_Recyyy_zzz.dat
//        : - eps controle plots : $DEFANAHOME/data/hNoise_xxx_Recyyy.eps
//                                 $DEFANAHOME/data/hPeakTime_xxx_Recyyy.eps
//                                 $DEFANAHOME/data/hPedestal_xxx_Recyyy.eps
//                                 $DEFANAHOME/data/hPinhole_xxx_Recyyy.eps
//                                 $DEFANAHOME/data/hPulseHeight_xxx_Recyyy.eps
//          where xxx is module number
//                yyy is Rec number
//                zzz is mode type
//          - an xml summary file in $DEFANAHOME/data
// Date        : 8/06/2004
// ---------------------------------------------------------------------------

#include <iostream>

#include <TROOT.h>

#include "XmlSetParser.h"
#include "BasicTextDescriptor.h"
#include "AnalDescriptor.h"
#include "ModRootReader.h"
#include "ModLtAnalyzer.h"
#include "ModAnalyzer.h"
#include "DbParser.h"

#include "DefaultPath.h"
#include "cmdlinedecode.h"

using namespace std;
using namespace lttest;

// ---------------------------------------------------------------------------
// Configuration : change this 
static int recnb = 1;                 // default record number
static string setdat="defanaset.xml"; // xml setting file
//static string setdat="ARCS_TIB.dat"; // ascii setting file
static string defroot="30200020035266.root"; //default root file
// ---------------------------end of configuration

void usage (string prgname){
  cout <<endl<<" usage example : "<<endl;
  cout <<"\t"<<prgname<< " -rootin file_.root " <<endl;
  cout <<"options:"<<endl;
  cout <<"\t-rootin  :the rootfile  (default "<<defroot<<")"<<endl;
  cout <<"\t-settings :the settins (cuts) (default "<<setdat<<")"<<endl; 
  cout <<"\t-recnr :the record (default: last record)"<<endl; 
}

int main(int argc, char **argv)
{

  cout<<"[TestModuleAnalyzer] ... enter"<<endl; 

  string cmdresult;
  string programname=argv[0];
  string filein=DefaultDataPath();
  filein=filein+defroot;

  if( "found"== CmdLineDecode(argc,argv,"h",cmdresult,false)){
    usage(programname);
    std::exit(0);
  }
  if( "found"== CmdLineDecode(argc,argv,"rootin",cmdresult,true)){
    filein=cmdresult;
    if( filein[0] != '/')  filein=DefaultDataPath()+filein;  
  }
  if( "found" == CmdLineDecode(argc,argv,"settings",cmdresult,true)){
    setdat=cmdresult;
  }

  if( "found"== CmdLineDecode(argc,argv,"recnr",cmdresult,true)){
    recnb = (int) atoi(cmdresult.c_str());
  }
  
  // initialize the root reader
  cout<<"[TestModuleAnalyzer] ... define a root reader"<<endl; 
  ModRootReader *rr =  new ModRootReader(filein.c_str());
  if(rr->SetRecord(recnb)==kFALSE) {
    cout <<endl<<"[TestModuleAnalyzer] ATTENTION! Record"<<recnb
	 <<" does not exist ... exit"<<endl<<endl;
    return 0;;
  }

  // initialize the settings
  AnalSetParser *setparser;
  string defdat = DefaultConfigPath() + setdat;
  
  if(setdat.find("xml") < setdat.size()) {
    cout<<"[TestModuleAnalyzer] ... define a XmlSetParser"<<endl; 
    setparser = new XmlSetParser();
  }
  else  if(setdat.find("dat") < setdat.size()) {
    cout<<"[TestModuleAnalyzer] ... define a TextSetParser"<<endl; 
    setparser = new BasicTextDescriptor();   
  }
  else {
    cout<<"[TestModuleAnalyzer] "<<defdat<<" is not a valid file name"<<endl;
    exit(0);
  }
  
  if(setparser->parseFile(defdat.c_str())){
    cout<<"[TestModuleAnalyzer] Can't parse file "<<defdat<<" exit !"<<endl;
    exit(0);
  }  
  ModAnalyzer *analyzer = new ModAnalyzer(setparser, rr);
  
  analyzer->AnalyzeAndPrint();
  
  cout << "[TestModuleAnalyzer] ...  create DbParser " << endl;
  string xmlin=DefaultConfigPath();
  xmlin=xmlin+"modvalidation2.xml";
  DbParser* db=new DbParser(xmlin.c_str());
  cout <<"REC = "<<recnb<<endl; db->SetRecord(recnb);
  db->getDBLUT()->initmapARC();
  rr->SetRecord(recnb);
  db->SetRecord(recnb);
  db->populate(rr); 
  db->write();
  delete db;

  delete setparser;
  delete rr;
  delete analyzer;
  
  cout<<"[TestModuleAnalyzer] ... end"<<endl; 
  
  return 0;
}
