// ---------------------------------------------------------------------------
// TestStructLtAnalyzer
// Description : This program shows how to analyzes a module file produced by 
//               Long Term structure tests
//               You can choose either an xml or ascii setting file
//               usage : ./bin/TestStructLtAnalyzer -rootin rootfile.root
//               More information on 
//               http://hep.uia.ac.be/cms/testing/software/DefAna/defana.html
// Input       : - rootfile : $HOME/DefAna/data
//               - setting  : defanaset.xml in  $HOME/DefAna/config
//                            or TIB.dat in  $HOME/DefAna/config
// Output : - ascii summary file : $DEFANAHOME/data/Summary_xxx_Recyyy_zzz.dat
//        : - eps controle plots : $DEFANAHOME/data/hNoise_xxx_Recyyy.eps
//                                 $DEFANAHOME/data/hPeakTime_xxx_Recyyy.eps
//                                 $DEFANAHOME/data/hPedestal_xxx_Recyyy.eps
//                                 $DEFANAHOME/data/hPinhole_xxx_Recyyy.eps
//                                 $DEFANAHOME/data/hPulseHeight_xxx_Recyyy.eps
//          where xxx is module number
//                yyy is Rec number
//                zzz is mode type
//          - an xml summary file in $DEFANAHOME/data
// Date        : 19/04/2004
// ---------------------------------------------------------------------------

#include <iostream>

#include <TROOT.h>

#include "XmlSetParser.h"
#include "BasicTextDescriptor.h"
#include "StructLtRootReader.h"
#include "StructLtAnalyzer.h"
#include "DbParser.h"

#include "DefaultPath.h"
#include "cmdlinedecode.h"

using namespace std;
using namespace lttest;

// ---------------------------------------------------------------------------
// Configuration : change this 
static string setdat="defanaset.xml"; // xml setting file
//static string setdat="LT_TIB.dat"; // ascii setting file
// ---------------------------end of configuration

void usage (string prgname){
  cout <<endl<<" usage example : "<<endl;
  cout <<"\t"<<prgname<< " -rootin file_.root " <<endl;
  cout <<"options:"<<endl;
  cout <<"\t-rootin  :the rootfile  (default  reslt_M_30200020035266.root )"<<endl;  
}

int main(int argc, char **argv)
{

  cout<<"[TestStructLtAnalyzer] ... enter"<<endl; 

  string cmdresult;
  string programname=argv[0];
  string filein=DefaultDataPath();
  filein=filein+"reslt_M_30200020035266.root";
  if( "found"== CmdLineDecode(argc,argv,"h",cmdresult,false)){
    usage(programname);
  }
  if( "found"== CmdLineDecode(argc,argv,"rootin",cmdresult,true)){
    filein=cmdresult;
    if( filein[0] != '/') { filein=DefaultDataPath()+filein;  
    }
  }
  // initialize the root reader
  cout<<"[TestStructLtAnalyzer] ... define a root reader"<<endl; 
  StructLtRootReader *rr =  new StructLtRootReader(filein.c_str());

  // initialize the settings
  AnalSetParser *setparser;
  string defdat = DefaultConfigPath() + setdat;
  
  if(setdat.find("xml") < setdat.size()) {
    cout<<"[TestStructLtAnalyzer] ... define a XmlSetParser"<<endl; 
    setparser = new XmlSetParser();
  }
  else  if(setdat.find("dat") < setdat.size()) {
    cout<<"[TestStructLtAnalyzer] ... define a TextSetParser"<<endl; 
    setparser = new BasicTextDescriptor();   
  }
  else {
    cout<<"[TestStructLtAnalyzer] "<<defdat<<" is not a valid file name"<<endl;
    exit(0);
  }
  
  if(setparser->parseFile(defdat.c_str())){
    cout<<"[TestStructLtAnalyzer] Can't parse file "<<defdat<<" exit !"<<endl;
    exit(0);
  }  
  
  StructLtAnalyzer *analyzer = new StructLtAnalyzer(setparser, rr);
  
  analyzer->Analyze();
  
  cout << "[TestStructLtAnalyzer] ...  create DbParser " << endl;
  
  string xmlin=DefaultConfigPath()+"seclt1.xml";
  DbParser* db=new DbParser(xmlin.c_str());
  db->getDBLUT()->initmap();
  db->populate(rr); 
  db->write();
  delete db;
  
  delete setparser;
  delete rr;
  delete analyzer;
  
  cout<<"[TestStructLtAnalyzer] ... end"<<endl; 
  
  return 0;
}
