#include <iostream>

#include <TROOT.h>
#include "TApplication.h"
#include "TCanvas.h"
#include "TLegend.h"

#include "AnalSetParser.h"
#include "AnalDescriptor.h"
#include "BasicTextDescriptor.h"
#include "XmlSetParser.h"
#include "NoiseDescriptor.h"
#include "ModRootReader.h"
#include "ModLtRootReader.h"
#include "StructLtRootReader.h"
#include "ModLtAnalyzer.h"
#include "ModAnalyzer.h"
#include "StructLtAnalyzer.h"
#include "DbParser.h"
#include "DbLUT.h"
#include "DbForm.h"
#include "RecUtil.h"
#include "DbClient.h"
#include <qstring.h>
#include <qregexp.h>


#include "DefaultPath.h"
#include "cmdlinedecode.h"

using namespace std;
using namespace lttest;

extern void InitGui();
VoidFuncPtr_t initFuncs[] = {InitGui, 0};
TROOT root ("standalone", "plots", initFuncs);
//TApplication* theApp;


void usage (string prgname){
  cout <<endl<<"Usage example : "<<endl;
  cout <<"\t"<<prgname<< " -rootin file.root [-recnr x -settings setfile] [-comp 'file1.root file2.root ...' -rec '1 2 3...' -objname 'histname']" <<endl
  <<endl;
  cout <<"options:"<<endl;
  cout <<"\t-rootin  :the rootfile" << endl;
  cout <<"\tLtStruct root file name must be of the form: reslt_S_XXXXXXXXXX.root" <<endl;
  cout <<"\tLtMod root file name must be of the form: reslt_M_XXXXXXXXXX.root" <<endl;
  cout <<"\tARC root file name must be of the form: XXXXXXXXXXXX.root)"<<endl;  
  cout <<"\twhere XXXXXXXX is the module id number" << endl <<endl;
  cout <<"\t-recnr   :number of record containing valid info (only for ARC root file)"<<endl;
  cout <<"\tOptional, otherwise last record in file will be taken as valid" <<endl <<endl;
  cout <<"\t-settings   : cuts settings file to be used"<<endl;
  cout <<"\tOptional, otherwise default file will be used" <<endl <<endl;

  cout <<"\t-comp   :Comparison mode. Compares histogram objname in:" << endl << endl;
  cout <<"\t ~different records for the same file file1.root (more than 1 value is given in -rec= )" <<endl;
  cout <<"\tExample: mainAnalyzer -comp reslt_M_30200020035266.root -rec '18 19' -objname DecInvOn/Noise/RawNoise" <<endl <<endl;
  cout <<"\t ~different files for the same record -rec (just one value is given in -rec= )" << endl;
  cout <<"\tExample: mainAnalyzer -comp '../data/30200020035266.root reslt_M_30200020035266.root' -rec '1 19' -objname DecInvOn/Noise/RawNoise" <<endl; 
  cout <<"\tFirst record number is for file1.root. Second record number is for file2.root,..." <<endl << endl;

  cout <<"\tNOTE: if comparison mode is requested, mainAnalyzer is oblivious to -rootin entry" <<endl<<endl;

  cout <<"\t======== EXPERT MODE==================="<<endl;
  cout <<"\t"<<prgname<< " -expert"<<endl;
  cout <<"\t Demo of DBase parameters comparison using DbClient class" << endl;
  cout <<"\t For details see code in:"<<endl;
  cout <<"\t $DEFANAHOME/src/DefectAnalyzer/test/src/mainAnalyzer.cpp"<<endl;
//  cout <<"Note: root files must be in the Default data path" << endl;
}

int main(int argc, char **argv)
{
  cout<<"[mainAnalyzer] ... enter"<<endl; 


    AnalSetParser *setparser;

    bool expert=false;

    bool ltcb =false;
    bool ltst =false;
    int arcrecord=-1;

    int compmode =0;
    string objname;
    vector<Int_t> records;
    vector<string> files;

  string cmdresult, setdat, cutsfilename;
  string programname=argv[0];
  string filein=DefaultDataPath();
  cout << "Default data path: " << DefaultDataPath() <<endl << endl;

//  filein=filein+"30200020033334.root";
//  string fileinarc;
//  string fileinlt;
//  fileinlt=fileinlt+"reslt_M_30200020015008.root";
//  fileinlt=fileinlt+"reslt_M_30200020035266.root";


  if( "found"== CmdLineDecode(argc,argv,"h",cmdresult,false)){
    usage(programname);
    return 0;
  }

  else if("found"== CmdLineDecode(argc,argv,"comp",cmdresult,true)){
  compmode++;
  int pos = cmdresult.find("root");
//	cout << "first " << cmdresult.find("root") << " last " << cmdresult.find("root",pos+4) << endl;
//	cout << "first " << cmdresult.find_first_of("root") << " last " << cmdresult.find_last_of("root") << endl;
  	if(cmdresult.find("root",pos+4)==string::npos) { // one root file
	    filein=cmdresult;
 	    if(cmdresult.find("_M_")!=string::npos) ltcb =true;
 	    if(cmdresult.find("_S_")!=string::npos) ltst =true;
	    
	    if( filein[0] != '/' && filein[0] != '.')  {filein=DefaultDataPath()+filein;
	    cout << "No path was given for root file " << cmdresult <<" Default data path is assumed"  << endl <<endl;  }
	
				
	} //one root file

	else { //at least two root files
	compmode++;
		int i; bool jlast =false;
		if(cmdresult[cmdresult.size() -1]!=' ') jlast =true;
		string secta = cmdresult;
		do{
		i = secta.find_first_of(" ",0);
		string numa(secta,0,i);
	        if( numa[0] != '/' && numa[0] != '.')  {
	        cout << "No path was given for root file " << numa <<" Default data path is assumed"  << endl <<endl;
		numa=DefaultDataPath()+numa;}
		files.push_back(numa);
		secta = secta.substr(i+1);
		}while(secta.find_first_of(" ",0) != string::npos );
		// to avoid cases ending with no space
		if (jlast) {if( secta[0] != '/' && secta[0] != '.')  {
	        cout << "No path was given for root file " << secta <<" Default data path is assumed"  << endl <<endl;
		secta=DefaultDataPath()+secta;}
		files.push_back( secta);}
		
	} //more than 1 root file
	

	    if("found"== CmdLineDecode(argc,argv,"rec",cmdresult,true)){

	    	if( cmdresult.find_first_of(" ")!= cmdresult.find_last_of(" ",0)){
		int j; bool ilast =false;
		if(cmdresult[cmdresult.size() -1]!=' ') ilast =true;
		string sect = cmdresult;
		do{
		j = sect.find_first_of(" ",0);
		string num(sect,0,j);
		Int_t kk = (Int_t) atoi(num.c_str());
		records.push_back(kk);
		sect = sect.substr(j+1);
		}while(sect.find_first_of(" ",0) != string::npos );
		if (ilast) records.push_back((Int_t) atoi(sect.c_str()));
		} //more than 1 rec
		
		else {cout << "Error in -rec parameter: More than one record number is needed for comparison" <<endl;
			cout<< "Current record given: " << cmdresult << endl;
			cout<< "Use ' ' to allow reading of all input parameters by the program" <<endl;
			cout<<"See mainAnalyzer -h  for help" << endl <<endl;
			return 0;}
	    } //rec arg is present




 	    if("found"== CmdLineDecode(argc,argv,"objname",cmdresult,true)) objname=cmdresult;
	    
    if(compmode==1){cout << "Expect to process and compare in rootfile: " << filein << endl;
    cout << "Records:" <<endl;
    for(vector<Int_t>::const_iterator ir=records.begin();ir!=records.end();ir++){
    cout << "record " << *ir <<endl;
    }}

    else if(compmode==2){

    cout << "Expect to process and compare the following rootfiles and records: " << endl;
    vector<Int_t>::const_iterator ir=records.begin();
    for(vector<string>::const_iterator ifile=files.begin();ifile!=files.end();ifile++){
    cout << "File: " << *ifile << " Record: " << *ir <<endl;
    ir++;
    }
        if(records.size() != files.size()) {
	cout << endl << "Number of Records and number of Files given does not match!!!" << endl; 
	cout << "Please check your inputs!!!" << endl;
	return 0;}
    }
	    
	    cout << "\t Object to be compared: " << objname <<endl <<endl;

 
  } // comp check


  else if( "found"== CmdLineDecode(argc,argv,"rootin",cmdresult,true)){
    filein=cmdresult;

    if( filein[0] != '/' && filein[0] != '.')  {filein=DefaultDataPath()+filein;
    cout << "No path was given for root file. Default data path is assumed"  << endl <<endl;  }

 	if(cmdresult.find("_M_")!=string::npos) {ltcb =true; 
		cout << "LtMod root file " << filein << "will be analyzed" << endl << endl;}
 	else if(cmdresult.find("_S_")!=string::npos) {ltst =true; 
		cout << "LtStruct root file " << filein << "will be analyzed" << endl << endl;}
	else if("found"== CmdLineDecode(argc,argv,"recnr",cmdresult,true)){
	arcrecord=(int) atoi(cmdresult.c_str());
	cout << "Record number " << arcrecord << " in ARC root file" << filein << " will be used as valid info" <<endl <<endl;}
	else cout << "Last record in ARC root file " << filein << " will be used as valid info" << endl <<endl;

  if( "found" == CmdLineDecode(argc,argv,"settings",cmdresult,true)){
    setdat=cmdresult;
  }


 }


  else if ("found"== CmdLineDecode(argc,argv,"expert",cmdresult,false)) expert=true;
  else {cout <<"Type: mainAnalyzer -h   for help info" <<endl; return 0;}


// ------------------------------------------------------------------------------------------------------
// Begin processing after readout of input parameters
// ------------------------------------------------------------------------------------------------------


if(compmode>0){

	TApplication theApp("plots", 0, 0);

 Int_t col[10]={2,3,4,6,7,8,9,28,30,40};  

	if(compmode==2) filein = *(files.begin());
	TFile* f = new TFile(filein.c_str());
	TCanvas *c  = new TCanvas("mainAnalyzer comparison",objname.c_str(),8,25,750,750);
	RootReader *rr =  new RootReader(filein.c_str());
//	else     rr =  new ModLtRootReader(filein.c_str());

	c->Draw();
      	Int_t cf=0;
 
 Int_t nch=768;
 string cdirrec;
 string cpobjname(objname.c_str());
 char* histname = (char *) cpobjname.c_str();
    char *result = NULL;
    result = strtok( histname, "/" );
    while( result != NULL ) {
	histname = result;
        result = strtok( NULL, "/" );
    } 
TH1F* h=new TH1F(histname,histname,nch,0,nch);
h->SetStats(false);
vector<Int_t>::const_iterator ir=records.begin();
if(rr->SetRecord(*ir)){
TH1F* hpru=(TH1F*)rr->Find(objname.c_str(),false);
if(hpru!=0) {
h->SetMinimum(0.);
h->SetMaximum(hpru->GetMaximum()*1.25);

if(cpobjname.find("Time")!=string::npos) h->SetYTitle("[ns]");
else if(cpobjname.find("bad")!=string::npos) h->SetYTitle("");
else h->SetYTitle("ADC [counts]");
h->SetXTitle("Channel");
h->Draw();}}


	if(compmode==1){
     TLegend* leg=new TLegend(0.3,0.65,0.9,0.9,filein.c_str());
	for(ir=records.begin();ir!=records.end();ir++){
	  if(rr->SetRecord(*ir)) {
	  	if((TH1F*)rr->Find(objname.c_str(),false)!=0){
		TH1F* hd=(TH1F*)rr->Find(objname.c_str(),false);
		char kk[10];
		sprintf(kk,"Record%d",*ir);
		cdirrec=string(kk);
			if(hd) {
	 		hd->SetLineColor(col[cf]);
	 		hd->Draw("same"); 
			leg->AddEntry(hd,cdirrec.c_str(),"L");
			}
		}
	  }	  
	cf++;
	}
	leg->Draw();
	
	
	c->Update();


	}// compmode = 1



	else if(compmode==2){


     TLegend* leg=new TLegend(0.3,0.65,0.9,0.9);
     ir=records.begin();
	for(vector<string>::const_iterator ifi=files.begin();ifi!=files.end();ifi++){
	   rr = new RootReader((*ifi).c_str());
	  if(rr->SetRecord(*ir)) {
	  	if((TH1F*)rr->Find(objname.c_str(),false)!=0){
		TH1F* hd=(TH1F*)rr->Find(objname.c_str(),false);
		char kk[10];
		sprintf(kk,":Record%d",*ir);
		cdirrec =*ifi+string(kk);
			if(hd) {
	 		hd->SetLineColor(col[cf]);
	 		hd->Draw("same"); 
			leg->AddEntry(hd,cdirrec.c_str(),"L");
			}
		}
	  }
	cf++; ir++;
	}
	leg->Draw();
	
	
	c->Update();


	}// compmode = 2


  cout << "\t--------------------------------------------" <<endl;
  cout << "You can edit, save eps or change view of Canvas plots on screen." << endl;
  cout << "Quit root in any Canvas to finish the execution of the program." <<endl;
  cout << "\t--------------------------------------------" <<endl;

	Bool_t retrn = kTRUE;
	  theApp.Run(retrn);  


}
else if(!expert){

  string defdat=DefaultConfigPath(); 
  cout << "Default config path: " << DefaultConfigPath() <<endl << endl;

	if(setdat.size()!=0){
	
  		cutsfilename=defdat + setdat; 
		cout << "Using provided cut file: " <<endl;
  		cout << cutsfilename << endl <<endl;
	
		if(setdat.find("xml") < setdat.size())	{
 		setparser= new XmlSetParser();
		cout<<"[mainAnalyzer] ... define a XmlSetParser"<<endl;
		}
		
		else if(setdat.find("dat") < setdat.size()){
		setparser = new BasicTextDescriptor();
		cout<<"[mainAnalyzer] ... define a BasicTextDescriptor"<<endl;
		}
		
		else{
		cout<<"[mainAnalyzer] "<<setdat<<" is not a valid file name"<<endl;
		exit(0);  }

	}
	
	else{

	  cout << "Using default cut file:" << endl;

		if(ltcb || ltst){
  		cutsfilename=defdat+"defanaset.xml"; 
  		cout << cutsfilename << endl <<endl;
 		setparser= new XmlSetParser();
		cout<<"[mainAnalyzer] ... define a XmlSetParser"<<endl;
		}
		else{
		cutsfilename=defdat+"ARCS_TIB.dat";
		cout << cutsfilename << endl <<endl;
		setparser = new BasicTextDescriptor();
		cout<<"[mainAnalyzer] ... define a BasicTextDescriptor"<<endl;
		}
	}
// TApplication theApp("plots", 0, 0);
// TCanvas * mycan=new TCanvas("test","test",10,10,500,500);

	RecUtil::wait();


  cout<<"[mainAnalyzer] ... define a root reader"<<endl; 

  if(filein!=""){

	if(!ltcb && !ltst){	// ARCS root file

 TApplication theApp("plots", 0, 0);
// TCanvas * mycan=new TCanvas("test","test",10,10,500,500);

    ModRootReader *rr =  new ModRootReader(filein.c_str());
    
    if(arcrecord < 0) rr->GetLastRecord();
    else  rr->SetRecord(arcrecord);

 if(0!=setparser->parseFile(cutsfilename.c_str())) {
 cout<<"[mainAnalyzer] Can't parse file "<<cutsfilename<<" exit !"<<endl;
 exit(0);}

NoiseDescriptor* txtnoisedescriptor =
setparser->getNoiseDescriptor(rr->getModType());

txtnoisedescriptor->print();


ModAnalyzer *arcanalyzer = new ModAnalyzer(setparser, rr);

  cout << "[mainAnalyzer] ...  analyzing ARC root file " << endl;
arcanalyzer->AnalyzeAndPrint();

  cout << "\t--------------------------------------------" <<endl;
  cout << "You can edit, save eps or change view of Canvas plots on screen." << endl;
  cout << "Quit root in any Canvas to continue the execution of program." <<endl;
  cout << "\t--------------------------------------------" <<endl;

Bool_t retrn = kTRUE;
  theApp.Run(retrn);  

  cout << "[mainAnalyzer] ...  create DbParser " << endl;
string dbxmlfile=  defdat+"modvalidation2.xml";
DbParser* db=new DbParser(dbxmlfile.c_str());

    db->getDBLUT()->initmapARC();
    db->populate(rr); 
    db->write();
    delete db;

  delete setparser;
  delete rr;
  delete arcanalyzer;
  }
  
  else if(ltcb){ // LtMod root file reslt_M_XXXXXXXX.root
   TApplication theApp("plots", 0, 0);

    ModLtRootReader *rr =  new ModLtRootReader(filein.c_str());

 if(0!=setparser->parseFile(cutsfilename.c_str())) {
 cout<<"[mainAnalyzer] Can't parse file "<<cutsfilename<<" exit !"<<endl;
 exit(0);}

NoiseDescriptor* xmlnoisedescriptor =
setparser->getNoiseDescriptor(rr->getModType());

xmlnoisedescriptor->print();

ModLtAnalyzer *ltanalyzer = new ModLtAnalyzer(setparser, rr);

  cout << "[mainAnalyzer] ...  analyzing LtMod root file " << endl;
ltanalyzer->AnalyzeAndPrint();

  cout << "\t--------------------------------------------" <<endl;
  cout << "You can edit, save eps or change view of Canvas plots on screen." << endl;
  cout << "Quit root in any Canvas to continue the execution of program." <<endl;
  cout << "\t--------------------------------------------" <<endl;

Bool_t retrn = kTRUE;
  theApp.Run(retrn); 

  cout << "[mainAnalyzer] ...  create DbParser " << endl;
string dbxmlfile2=defdat+"modlt2.xml";
DbParser* db=new DbParser(dbxmlfile2.c_str());

    db->getDBLUT()->initmapCB();
    db->populate(rr); 
    db->write();
    delete db;

  delete setparser;
  delete rr;
  delete ltanalyzer;
  }
  
    else if(ltst){ // Ltstructure root file reslt_S_XXXXXXXX.root

    StructLtRootReader *rr =  new StructLtRootReader(filein.c_str());

 if(0!=setparser->parseFile(cutsfilename.c_str())) {
 cout<<"[mainAnalyzer] Can't parse file "<<cutsfilename<<" exit !"<<endl;
 exit(0);}

NoiseDescriptor* xmlnoisedescriptor =
setparser->getNoiseDescriptor(rr->getModType());

xmlnoisedescriptor->print();

StructLtAnalyzer *ltanalyzer = new StructLtAnalyzer(setparser, rr);

  cout << "[mainAnalyzer] ...  analyzing Ltstruct root file " << endl;
ltanalyzer->Analyze();

/*  cout << "\t--------------------------------------------" <<endl;
  cout << "You can edit, save eps or change view of Canvas plots on screen." << endl;
  cout << "Quit root in any Canvas to continue the execution of program." <<endl;
  cout << "\t--------------------------------------------" <<endl;

Bool_t retrn = kTRUE;
  theApp.Run(retrn);  */

  cout << "[mainAnalyzer] ...  create DbParser " << endl;
string dbxmlfile3=defdat+"seclt1.xml";
DbParser* db=new DbParser(dbxmlfile3.c_str());

    db->getDBLUT()->initmap();
    db->populate(rr); 
    db->write();
    delete db;

  delete setparser;
  delete rr;
  delete ltanalyzer;
  }
  
 }  //closes filein!=""
  
  
  cout << "\t--------------------------------------------" <<endl;
  cout << "\tXML files are located in execution directory" <<endl;
  cout << "\t--------------------------------------------" <<endl;


}// closes compmode=0

// ---------EXPERT MODE CODE PART----------------------------------
if(expert){
  cout<<"[mainAnalyzer] ... Expert mode DEMO"<<endl; 

string modId = "30200020035049";
//for a complete analysis of modId mod create a DbReader
//DbReader *dbr = new DbReader(modId);
//otherwise it is not needed

DbClient * dbc = new DbClient();

// mod type
string mod_type;
if(dbc->giveModType(modId,mod_type)>0)
cout << "Mod_type for module Id: "<<modId <<" is: " << mod_type << endl;


// Components in module
vector<string> compList, idsList;
if(dbc->giveComponentsIds(modId,compList, idsList)>0){
cout << "Module " << modId << " contains:"<<endl;
for(int i=0;i<compList.size();i++){
cout << compList[i] << " with Id: " << idsList[i] <<endl;
}}


// IV for sensor
string senId = "30211234387903";
vector<string> voltage, current;
if(dbc->giveSensorIV(senId,voltage,current)>0){
cout << "VOLTAGE_V  I_TOT_NA" << endl;
for(int i=0; i<voltage.size();i++){
cout << voltage[i] << "     " << current[i] <<endl;
}}


// DCUid
vector<string> dcu_id;
if(dbc->giveDcuId(modId,dcu_id)>0)
cout << "DCUid for module Id: " <<modId<< " is: "<<
 dbc->decodeDcuId(atoi(dcu_id[0].c_str()),atoi(dcu_id[1].c_str()),atoi(dcu_id[2].c_str()))<< endl;


// DCUid from Hybrid
string hybId = "30216631500646";
if(dbc->giveDcuIdfromHyb(hybId,dcu_id)>0)
cout << "DCUid for Hybrid Id: " <<hybId<< " is: "<<
 dbc->decodeDcuId(atoi(dcu_id[0].c_str()),atoi(dcu_id[1].c_str()),atoi(dcu_id[2].c_str()))<< endl;


// any object in format string
modId="30200020035266";
string objNAME="NBadChan";  // some examples.... name as in DB xml template
objNAME="BadChanList";
//objNAME="chflag"; //ETC,.......
string table="MODULLTLAST";
//table = MODVALIDATION, MODULLTFIRST, MODULLTCOLD, MODULLTLAST 

QString kk = QString(objNAME.c_str());
kk=kk.upper(); // DB answer is always in capital letters....
string objname = kk.latin1();

filein+="reslt_M_30200020035266.root";
RootReader *rr =  new RootReader(filein.c_str());

string answer = dbc->giveObject(modId,objname,table);

if(answer!=""){
cout << "Answer from DataBase:" <<endl;
cout << "Variable " << objNAME <<" in table: " << table << endl;
cout << "has value " << answer << endl << endl;

DbParser* db=new DbParser(rr);
if(filein.find("_M_")!=string::npos) db->getDBLUT()->initmapCB();
else db->getDBLUT()->initmapARC();

 if(db->SetRecord(19)) {// record to compare in root file
QString result = db->find(QString(objNAME.c_str()));
cout << "Result in file:" <<endl;
cout << "Variable " << objNAME <<" in file: " << filein+":Record19/" << endl;
cout << "has value " << result << endl << endl;

}// closes setRecord
delete db;

}//closes answer!=""



// pedestal, noise and calibration comparison
objname="CALPAPION";
//objname="PEDPION";
//objname="CALPTPION"; // some examples.... DB name 
//objname="RAWNOISEPION";
//objname="NOISEDIOFF";

table="MODULLTCOLD";

TH1F h;
string name= objname + " from DB for ModId "+modId+" in "+table+" table";
h = TH1F(name.c_str(),name.c_str(),768,1,768);

if(dbc->giveHist(modId,objname,table,h)>0){
	TApplication theApp("plots", 0, 0);
	TCanvas *c  = new TCanvas("mainAnalyzer comparison",objname.c_str(),8,25,750,750);

h.SetStats(false);
h.SetMinimum(0.);
h.SetMaximum(h.GetMaximum()*1.25);

QString objectName = QString(objname.c_str());
objectName=objectName.lower();
objectName=objectName.replace(QRegExp("pt"),"PT");
objectName=objectName.replace(QRegExp("pa"),"PA");

DbLUT * dblut = new DbLUT();
if(filein.find("_M_")!=string::npos)    dblut->initmapCB();
else dblut->initmapARC();

DbForm dlf=dblut->find(objectName);
QString s=dlf.getName();
float m=dlf.getM();

if(s!=0){
if(s.contains("Time",TRUE)>0) h.SetYTitle("[ns]");
else if(s.contains("bad",TRUE)>0) h.SetYTitle("");
else h.SetYTitle("ADC [counts]");

h.SetXTitle("Channel");
h.Draw();

 Int_t col[10]={2,3,4,6,7,8,9,28,30,40};  


     TLegend* leg=new TLegend(0.3,0.65,0.9,0.9);

	  if(rr->SetRecord(18)) {
	  	if((TH1F*)rr->Find(s,false)!=0){
		TH1F* hd=(TH1F*)rr->Find(s,false);
	
		string namelocal = filein+":Record18/"+s.latin1();
			if(hd) {
			if(m!=0) hd->Scale(m);
	 		hd->SetLineColor(col[0]);
			if(h.GetMaximum()<hd->GetMaximum()) {h.SetMaximum(hd->GetMaximum()*1.25); h.Draw();}
	 		hd->Draw("same"); 
			leg->AddEntry(hd,namelocal.c_str(),"L");
			}
		}
	  }

	h.SetLineColor(col[1]);
	h.Draw("same");
	leg->AddEntry(&h,name.c_str(),"L");
	leg->Draw();
	c->Update();

  cout << "\t--------------------------------------------" <<endl;
  cout << "You can edit, save eps or change view of Canvas plots on screen." << endl;
  cout << "Quit root in any Canvas to continue the execution of program." <<endl;
  cout << "\t--------------------------------------------" <<endl;


Bool_t retrn = kTRUE;
  theApp.Run(retrn);  

delete dblut;
}//close s!=0

else cout << "Object " << s << " not found in DbLUT!!!!!!!" << endl;

} //closes getHist >0

  cout << "\t--------------------------------------------" <<endl;
  cout << "\tend of DEMO output" <<endl;
  cout << "\t--------------------------------------------" <<endl;

}//closes expert mode demo


  cout<<"[mainAnalyzer] ... end"<<endl; 

return 0;
}
