#ifndef DbClient_h
#define DbClient_h

#include <qstringlist.h>
#include <string>
#include <vector>
#include <TH1F.h>

/** \brief Database Client 
           reads information from Central Db
           trough Socket,h and ClientSocket.h classes
           The correspondance is defined by lookup map 

              
     \author J. Fernandez, UniKa  
     \version 1.0 
     \date 11/2003
 
*/

class DbClient{

public:

DbClient();
~DbClient();

	
  // Data transmission

	// maybe this must be static??? 
	int readDB(std::string obj_id, std::string s_data, std::string fromtable, 
		std::string whereselec, std::string and_data, std::string& reply);

	int parseReply(std::string replyFromDB, 
		QStringList &columnOutput, QStringList &valueOutput);

	int checkpList(std::vector<std::string> &pList); // checks string parameters list

	void printQuery(std::string query);  // cout of string query for the moment

	std::string decodeDcuId(int H, int M, int L); //gives DCU ID decoded from H,M,L values given by user

	// IMPORTANT!!!!! The following methods store answer from DB in arguments.
	// The following methods  return response 0 = not found in DB
	// return != 0 means it was found in DB
	
	int giveComponentsIds(std::string modId, 
			      std::vector<std::string> &compList, 
			      std::vector<std::string> &idsList); // fetch modId and get Sen and Hyb ids

	int giveDcuId(std::string mod_id, std::vector<std::string> &dcu_id);   // gets DCU id of this module from hybrid assembly

	int giveDcuIdfromHyb(std::string hyb_id, std::vector<std::string> &dcu_id);  // gets DCU id of this module from hybrid

	int giveModType(std::string mod_id,std::string & mod_type);  //gets mod type of mod_id
	
	int giveSensorIV(std::string sen_id,std::vector<std::string> &voltage,std::vector<std::string> &current);  // gets I-V

	// pedestals, calibration and noise check
	int giveHist (std::string modId, std::string objectName, std::string table, TH1F & h); // gets number of channels in histogram
		//objectName = NOISEPION,NOISEPIOFF,RAWNOISEPION,RAWNOISEDIOFF,PEDPION, PEDPIOFF,...
		//table = MODVALIDATION, MODULLTFIRST, MODULLTCOLD, MODULLTLAST


	// Next method returns value of objectName requested:

	// Variable check into DB
	std::string giveObject (std::string modId, std::string objectName, std::string table); // returns objectName value to compare
		//objectName = NBADCHAN, BADCHANLIST,....
		//table = MODVALIDATION, MODULLTFIRST, MODULLTCOLD, MODULLTLAST 

	// I bias values check
	float giveIbias (std::string senId, std::string objectName); // returns I Bias value from DB
		//objectName = I_TOT_V1_NA, I_TOT_V2_NA


	// To be implemented.......
	
	bool uploadXml(const char* fname);  // upload produced modvalidation.xml file
	bool uploadAllXml(const char* dirname);  // upload all from dir


/*
	vector<float>& getDcuCal(long modid);   // return vector of the dcu cal.

	vector<float>& getNoise(long modid);  // get noise from previous tests
	
	map<int, int, less<int>  >& getBadchList(long modid);
*/
};


#endif
