#include "DbParser.h"
#include "DbForm.h"
#include <TNamed.h> 
#include <TVector.h> 
#include "ReadUtil.h"
#include "daqerrormsg.h"
#include <fstream>
#include <iostream>
using namespace std;
 using namespace ReadUtil;

DbParser::DbParser(RootReader* rec) {
  therecreader=rec;   
  thedblut=new DbLUT();
  ncomp=-1;
  record = -1;
}

DbParser::DbParser(const char* filename) {  
  DT_NOTE("DbParser: construct; filename "<<filename);
  templatename=filename;
  thedblut=new DbLUT();
   ncomp=-1;
   record=-1;
  QString sfile(filename);
  if(getTree(sfile)) DT_NOTE("  parsing into Db file ");
}
/**/
DbParser::~DbParser() {}
/**/
bool DbParser::getTree(const QString filename) {
  DT_DEBUG_LTAPPL("DbParser:getTree");
  QFile infile(filename);
  if( !infile.open( IO_ReadOnly )) {
    DT_WARN("*******  cannot open "<<filename) ;
    return false; 
  }
  DT_DEBUG_LTAPPL(" opened "<<filename);
  if(!domTree.setContent(&infile)) {
    DT_WARN("****** cannot parse as an XML "<<filename);
    infile.close(); return false;
  }
  infile.close();

  DT_DEBUG_LTAPPL("DbParser: getTree  setContent ");
  return true;
}
/**/
void DbParser::getCompositeAction( QDomElement &composite) {  
 DT_DEBUG_LTAPPL(" getCompositeAction ");     
  QDomNode node = composite.firstChild();
  while( !node.isNull() ) {
    if(node.isElement() ) {
      if(node.nodeName() == "action_description" ) {
	DT_DEBUG_LTAPPL(" node = composite action_description ");      
	QDomElement act=node.toElement();    
	if( !act.isNull() ) {
	  DbActionDescriptor*  ad=new DbActionDescriptor(
	    act.attribute("name"),
	    act.attribute("version"),
	    act.attribute("object_name"),
	    act.attribute("object_type"),
	    act.attribute("input_id"));
	  compositeactiondescriptor=ad;

// Cloning of SECLTMX xml template
if((compositeactiondescriptor->name).contains("SECLTM",TRUE)==0 &&
	(compositeactiondescriptor->name)!="SECLT" &&
	(compositeactiondescriptor->name).contains("SECLT",TRUE)==1){
if(!setRecord(compositeactiondescriptor->name)) 
{DT_WARN("found no Record for "<< compositeactiondescriptor->name); return;}

	QDomNode composite2 = node.nextSibling();
	//to avoid comments....
	while(composite2.isComment()){composite2 = composite2.nextSibling();}

	for(int j=2; j<therecreader->ModCounter()+1; j++){
	QDomNode newcomposite = composite2.cloneNode("TRUE");
	QDomNode changeN = newcomposite.firstChild();

	if(changeN.isElement()){
		if(changeN.nodeName() == "action_description" ){
		QDomElement curr=changeN.toElement();
		char str[8];
		sprintf(str,"SECLTM%d",j); 
		QString seclt = QString(str);
		curr.setAttribute("name",seclt);
		sprintf(str,"%d",j); 
		seclt = QString(str);
		curr.setAttribute("position_in_parent",seclt);
		}
	}
	else cout << "changeN is not element" << endl;
	QDomNode insert = composite.insertBefore(newcomposite,composite.lastChild());
	if(insert.isNull()) {DT_WARN("node could not be inserted");return;}
	}
}

// Module directory setting
if((compositeactiondescriptor->name).contains("SECLTM",TRUE)==1){
QString modnumber = (compositeactiondescriptor->name).section("SECLTM",-1);
therecreader->SetRecord(record);
if(!therecreader->SetModule(modnumber.toInt())) 
{DT_WARN("found no Mod");return;}
else SetModule(modnumber.toInt());}  


if((compositeactiondescriptor->name).contains("MODULLT",TRUE)==1){
if(!setRecord(compositeactiondescriptor->name)) 
{DT_WARN("found no Record"); cout<<"for "<< compositeactiondescriptor->name <<endl; return;}
}

// ARC root file Record folder identification
if((compositeactiondescriptor->name)=="MODVALIDATION"){

if(!setRecord(compositeactiondescriptor->name)) 
{DT_WARN("found no Record"); cout<<"for "<< compositeactiondescriptor->name <<endl; return;}
}



        } //closes !act.isNull
	} //closes node.nodeName() == "action_description"

	
      if(node.nodeName() == "action" ) {
     
	DT_DEBUG_LTAPPL(" node =  basic action");
        QDomElement action=node.toElement();

        if(!action.isNull() )  getBasicAction(action);
        else DT_WARN(" empty basic action ");
      }

      if(node.nodeName() == "composite" ) {
	ncomp++;
	DT_DEBUG_LTAPPL(" node = composite action"<<ncomp); 
        QDomElement comp2action=node.toElement();
        if(!comp2action.isNull() )  getCompositeAction(comp2action);
        else DT_WARN(" empty composite action ");
      }
    } // closes node.isElement
    node=node.nextSibling();
  } // closes node.isNull
}
//
void DbParser::getBasicAction(const QDomElement &action)  {
 QDomNode node = action.firstChild();
 while( !node.isNull() ) {
   if(node.isElement() ) {
     if(node.nodeName() =="action_description" ) {
       DT_DEBUG_LTAPPL(" node= basic action description"); 
       QDomElement act=node.toElement();    
       if( !act.isNull() ) {
	 DbActionDescriptor*  ad=new DbActionDescriptor(
	   act.attribute("name"),
	   act.attribute("version"),
	   act.attribute("object_name"),
	   act.attribute("object_type"),
	   act.attribute("input_id"));

 if(ad->name=="SECLTSUMMARY") {
	   if(!setRecord("SECLTLAST")) 
	{DT_WARN("found  no Record for summary");return;} 
	   if(!therecreader->SetModule(1)) 
	{DT_WARN("found  no Mod for summary");return;} }

 if(ad->name=="MODULLTSUMMARY") {
	   if(!setRecord("MODULLTLAST")) 
	{DT_WARN("found  no Record for summary");return;} }


	 basicactiondescriptor.push_back(ad);
       }  

     }
     if(node.nodeName() == "input" ) { 
       DT_DEBUG_LTAPPL(" node= input "); }

      if(node.nodeName() == "result" ) { 
       DT_DEBUG_LTAPPL("node= result ");
       QDomElement result=node.toElement();
       setAttribute(result);
     }
   }
   node=node.nextSibling();
 }
}
//
void DbParser::setAttribute(QDomElement &result)  {
  DT_DEBUG_LTAPPL(" setattribute ");
  if( !result.isNull() )  { 
    QString sname=result.attribute("name");
    const  QString data=find(sname);
    DT_DEBUG_LTAPPL(" attribute "<<sname<<" data "<<data);
  
      if(data!="-0")  
	result.setAttribute("value",data);

  }
  DT_DEBUG_LTAPPL(" attribute set ");
}
/*
 * define recordreader and  records for composite actions
 */
void DbParser::populate(RootReader* rec) {
 DT_DEBUG_LTAPPL(" populate rec ");
therecreader=rec;

/*  TString  id=therecreader->GetId();
  idq=QString(id.Data());
  QString ffs=idq.section('_',-1);
  idq=ffs.section('.',0,0);
  idq=idq.section('/',-1);*/ //old method from filename

 if(record==-1)  record=therecreader->GetLastRecord(false);
 idq=find("id");

 fillresults(); 
}

/* parse  domTree
 */
void DbParser::fillresults() {
  DT_DEBUG_LTAPPL(" fillresults ");
  QDomElement root=domTree.documentElement();
  QDomNode node0=root.firstChild();
  QDomNodeList nlist=node0.childNodes();
  QDomNode node=nlist.item(0);
  while( !node.isNull() ) {  

       if(node.isElement() && node.nodeName() == "object" ) {
	   QDomElement obj=node.toElement();
//	   DT_DEBUG_LTAPPL(" populate xml "<<therecreader->GetModuleNr());
//	   obj.setAttribute("id",therecreader->GetModuleNr());
	   DT_DEBUG_LTAPPL(" populate xml "<<idq);
	   obj.setAttribute("id",idq);
	 }       
    if(node.isElement() && node.nodeName() == "composite" )  {
//      if(!therecreader->setRecord(ncomp)) {DT_WARN("found  no Record"<<ncomp);return;}
	ncomp++;
      DT_DEBUG_LTAPPL("  composite action  "<<ncomp);
      QDomElement composite=node.toElement();
      if(!composite.isNull())  getCompositeAction(composite);
      else DT_WARN(" empty composite action ");
      break;
    }
    node=node.nextSibling();
  }
}
/*
 * write to Xml file
*/
void DbParser::write(const QString filename)  {
DT_DEBUG_LTAPPL("write ______________"<<filename);
  ofstream outfile(filename);
 DT_DEBUG_LTAPPL("write ______________");
DT_DEBUG_LTAPPL(domTree.toString());
DT_DEBUG_LTAPPL("_____________done");
DT_DEBUG_LTAPPL(domTree.toString()); 
} 
/**/
void DbParser::write()  {
  ofstream outfile;
  DT_DEBUG_LTAPPL(" write  "<<templatename<<" "<<idq);
  // returns the template file name 
  QString  dir=templatename.section('/',-1);
  if(dir!=NULL){
    dir='_'+dir;
    dir=idq+dir;
  } 
  else { dir="NN_";
    dir = dir+templatename;
  }
  outfile.open(dir);
DT_DEBUG_LTAPPL("write______________ "<<dir);
DT_DEBUG_LTAPPL(domTree.toString());
DT_DEBUG_LTAPPL("_____________done");
  outfile<<domTree.toString()<<endl; 
} 

bool DbParser::setRecord(const QString& s) {
  char str[40];


if(s=="MODVALIDATION"){record=therecreader->GetRecordNr();
		return true;} //record has to be set by standalone...... check this in future

therecreader->GoToFirstRecord(true);	
do{
    
      sprintf(str,"TNcdb");
	TNamed* ncdb=(TNamed*)therecreader->Find(str,false);
	if(ncdb) {	  
		if(ncdb->GetTitle()==s)    {
		DT_DEBUG_LTAPPL(" setRecord " << therecreader->GetRecordNr() << " for "	<< s );
				record=therecreader->GetRecordNr();
				therecreader->SetRecord(record);
				return true;}	}
	

      }while(therecreader->GoToNextRecord() > 0);
DT_DEBUG_LTAPPL(" no Record Found for " << s << " up to RecordNr: " << therecreader->GetRecordNr() );

}
/**/
QString DbParser::find(const QString& ss){

  DbForm dlf=thedblut->find(ss);

  QString s=dlf.getName();
  DT_DEBUG_LTAPPL(" try to find "<< s);

//Let's make sure we are in the right directory
	therecreader->SetRecord(record);
if(s.contains("Module",TRUE)>0 ) {therecreader->SetModule(module);
				s=s.section('/',1);}

if(ss.contains("Lt",TRUE)>0 || ss=="MODULLTSUMMARY_val") therecreader->cd();
if(s.contains("SecLtSummary",TRUE)>0 ) therecreader->cd();


if(s !="0"){
   if(ss=="status") return "reference";
   if(ss=="tool_id") return "708";
   if(s.contains("_C", TRUE)>0) {
     if((TNamed*)therecreader->Find(s,false)!=0)  return  Root2Str((TNamed*)therecreader->Find(s,false),dlf); }
   if(s.contains("TV", TRUE)>0) {  
     if((TVector*)therecreader->Find(s,false)!=0)  return Root2Str((TVector*)therecreader->Find(s,false),dlf);  }
  if(s.contains("_F", TRUE)>0) {  
     if((TVector*)therecreader->Find(s,false)!=0)  return Root2Str((TVector*)therecreader->Find(s,false),dlf);}
  if(s.contains("TF", TRUE)>0) {  
     if((TVector*)therecreader->Find(s,false)!=0)  return Root2Str((TVector*)therecreader->Find(s,false),dlf);}
  if(s.contains("TI", TRUE)>0) {  
     if((TVector*)therecreader->Find(s,false)!=0)  return Root2StrI((TVector*)therecreader->Find(s,false),dlf);}
 if(s.contains("_I", TRUE)>0) {  
     if((TVector*)therecreader->Find(s,false)!=0)  return Root2StrI((TVector*)therecreader->Find(s,false),dlf);}
  if(s.contains("TL", TRUE)>0) {  
     if((TVector*)therecreader->Find(s,false)!=0)  return Root2StrL((TVector*)therecreader->Find(s,false),dlf);}
   if(s.contains("TH", TRUE)>0) {
     if((TH1F*)therecreader->Find(s,false)!=0)  return  Root2Str((TH1F*)therecreader->Find(s,false),dlf); }
   if(s.contains("TN", TRUE)>0) {
     if((TNamed*)therecreader->Find(s,false)!=0)  return  Root2Str((TNamed*)therecreader->Find(s,false),dlf); }
   if(s.contains("TG", TRUE )>0) {
     if((TGraph*)therecreader->Find(s,false)!=0)  return  Root2Str((TGraph*)therecreader->Find(s,false),dlf); }
   if(s.contains("TP", TRUE)>0) {
     if((TProfile *)therecreader->Find(s,false)!=0)  return  Root2Str((TProfile*)therecreader->Find(s,false),dlf); }
   if(s.startsWith("IV")) {
      if(ss=="voltageIV"  && (TGraph*)therecreader->Find(s,false)!=0) return  Root2StrX((TGraph*)therecreader->Find(s,false),dlf);
      if(ss=="currentIV"  && (TGraph*)therecreader->Find(s,false)!=0) return  Root2StrY((TGraph*)therecreader->Find(s,false),dlf);
    }
//else
   TH1F* h=(TH1F*)therecreader->Find(s,false); 
   if(h!=0){ QString sout=Root2Str(h,dlf);  return sout;}

 
   DT_DEBUG_LTAPPL("=====not found "<<s);
   return "";
}
return "";
}
/**/
