// Implementation of the Socket class.
#include "Socket.h"
//#include "string.h" wb???
#include <iostream>
#include <cstring>
#include <errno.h>
#include <fcntl.h>

using namespace std;

Socket::Socket() :
  m_sock (-1)
{
  memset (&m_addr, 0, sizeof(m_addr));
}

Socket::~Socket() {
  if (is_valid())
    ::close(m_sock);
}

bool Socket::create() {
  m_sock = socket(AF_INET, SOCK_STREAM, 0);
  if (! is_valid())
    return false;

  // TIME_WAIT - argh
  int on = 1;
  if (setsockopt(m_sock, SOL_SOCKET, SO_REUSEADDR, (const char*) &on, sizeof (on)) == -1)
    return false;

  return true;
}

bool Socket::bind (int port) {
  if (!is_valid()) {
    return false;
  }
  m_addr.sin_family = AF_INET;
  m_addr.sin_addr.s_addr = INADDR_ANY;
  m_addr.sin_port = htons (port);
  int bind_return = ::bind (m_sock,(struct sockaddr *) &m_addr, sizeof(m_addr));
  if (bind_return == -1) {
    return false;
  }

  return true;
}

bool Socket::listen() const {
  if (! is_valid()) {
    return false;
  }
  int listen_return = ::listen(m_sock, MAXCONNECTIONS);
  if (listen_return == -1) {
    return false;
  }

  return true;
}

bool Socket::accept (Socket& new_socket) const {
  int addr_length = sizeof(m_addr);
  new_socket.m_sock = ::accept (m_sock, (sockaddr *) &m_addr, (socklen_t *) &addr_length);

  if (new_socket.m_sock <= 0)
    return false;
  else
    return true;
}

bool Socket::send (const std::string s) const {
  int status = ::send (m_sock, s.c_str(), s.size(), MSG_NOSIGNAL);
  if (status == -1) {
    return false;
  }
  else
  {
    return true;
  }
}

int Socket::recv (std::string& s) const {
  char buf[MAXRECV + 1];

  s = "";
  memset (buf, 0, MAXRECV + 1);

  int status = ::recv(m_sock, buf, sizeof(buf), MSG_WAITALL);
  if (status == -1) {
    std::cout << "status == -1   errno == " << errno << "  in Socket::recv\n";
    return 0;
  }
  else if (status == 0) {
    return 0;
  }
  else {
    buf[status] = '\0';
    s = buf;
    return status;
  }
}

bool Socket::connect (const std::string host, const int port) {
  if (!is_valid()) return false;

  m_addr.sin_family = AF_INET;
  m_addr.sin_port = htons ( port );
  m_addr.sin_addr.s_addr = gethostaddress(host.c_str());

  int status = ::connect (m_sock, (sockaddr *) &m_addr, sizeof(m_addr));

  if (status == 0)
    return true;
  else
    return false;
}

void Socket::set_non_blocking (const bool b) {
  int opts;
  opts = fcntl (m_sock,F_GETFL);

  if (opts < 0) {
    return;
  }

  if (b)
    opts = (opts | O_NONBLOCK);
  else
    opts = (opts & ~O_NONBLOCK);

  fcntl(m_sock, F_SETFL,opts);
}
/*
 * Do I need to delete <CODE>host_ptr</CODE>
 */
unsigned int Socket::gethostaddress (const char* hostname) {
  struct hostent* host_ptr;
  char host[256];
  int type;
  unsigned int addr;
  struct in_addr ad;

  if (inet_aton(hostname, &ad)) {
     memcpy(&addr, &ad.s_addr, sizeof(ad.s_addr));
     type = AF_INET;
     if ((host_ptr = gethostbyaddr((const char *)&addr, sizeof(addr), AF_INET)))
        strcpy(host,host_ptr->h_name);
     else
        strcpy(host,"UnNamedHost");
  }
  else if ((host_ptr = gethostbyname(hostname))) {
    // Check the address type for an internet host
    if (host_ptr->h_addrtype != AF_INET) {
      cerr << hostname << " is not an internet hostn" << endl;
      return false;
    }
    memcpy(&addr, host_ptr->h_addr, host_ptr->h_length);
    strcpy(host,host_ptr->h_name);
    type = host_ptr->h_addrtype;
  }
  else {
    cerr << "**** Unknown host " << hostname << endl;
    return false;
  }

  if (0) {
    ad.s_addr = addr;
    cout << "Official name: " << host << endl;
    cout << "IP addres = " << inet_ntoa(ad) << endl;
  }

  return addr;
}
