#ifndef LTEXCEPT
#define LTEXCEPT


#include <iostream>
#include <exception>
#include <string>
#include <cstring>
#include <time.h>
/** 
    use example :
 throw LtExcept(LtExcept::DEBUGERR, LtExcept::SERVERITYNOTDEFINED,
		   __PRETTY_FUNCTION__,__LINE__,"22 not allowed","pure debug",
		);
		
   try {
   }
   catch (LtExcept le) {
    if( le.getSeverity() >= LtExcept::FATALERROLEVEL) {
      DAQEXIT("Exit because of fatal error" <<le.getUserMsg());
    }   
    else{ 
      if(le.getSeverity() >= LtExcept::NONEFATAL) {
	DT_WARN("Warning : "<<le.getUserMsg());
      }
      //else let crash for the moment
    }	
  }
   
   }

*/
class LtExcept:public std::exception  {
public:
  enum lt_errcode { NOERR,ERRNOTDEFINED,FMUXI2CERR,MODINITERR,DEBUGERR };
  enum lt_severity {
    // no action to be taken                
    SERVERITYNOTDEFINED,MESSAGE,
    // warnings etc 
    WARNING,NONEFATAL,
    // all below are suppose to be fatal errors 
    FATALERROLEVEL,
    FATALHWERROR, 
    EXIT
  };

protected:
  lt_errcode   fErrorCode ;
  std::string fUserMessage;
  std::string fDevelopperMessage ;
  lt_severity   fFaultSeverity;
  std::string fModuleName;
  int fLine;
  time_t fTimeStamp;

public: 
  LtExcept()throw ( ) {
    fTimeStamp = time (NULL) ;
    fErrorCode =ERRNOTDEFINED;
    fUserMessage="none" ;
    fDevelopperMessage="none" ;
    fFaultSeverity=SERVERITYNOTDEFINED;
    fLine=0;
    fModuleName="not defined";    
  }
  /** detailed exception call 
      @param ErrorCode  the error code required
      @param FaultSeverity  
      @param Module name string with the funtion name ( __PRETTY_FUNCTION__ can be used)
      @param line the line where the throw occures 
      @param UserMessage a message for the users
      @param DeveloperMessage : extra debug info 
  */
  LtExcept( lt_errcode ErrorCode,lt_severity  FaultSeverity=SERVERITYNOTDEFINED,
	    std::string ModuleName="not defined",int line=-1,
	    std::string UserMessage="not defined",
	    std::string DevelopperMessage="not defined"
	    )throw ( ) { 
    fErrorCode=ErrorCode ;
    fUserMessage=UserMessage;
    fDevelopperMessage=DevelopperMessage ;
    fFaultSeverity=FaultSeverity;
    fModuleName=ModuleName;
    fTimeStamp= time (NULL) ;
    fLine=line;
  }

  virtual ~LtExcept() throw() {}

  std::string getUserMsg(){ return fUserMessage;}
  
  lt_severity  getSeverity(){ return  fFaultSeverity;}
  
  lt_errcode getErrorCode(){return fErrorCode;}
  std::string getDeveloperMsg(){ return fDevelopperMessage;}

  /*  std::string getErrMsg(){  */
     
/*      std::sstream ss<<"except @ "<< fModuleName << "user  msg"; */
/*      ss<<fUserMessage << " dev mesage" << fDevelopperMessage; */
/*      ss<" err code "<< fErrorCode <<" Severity "<<fFaultSeverity; */
/*      ss<< " at " << fTimeStamp; */

/*      string s=ss; */
/*      return s; */
/*    } */

};

#endif



